@@ -1,7 +1,7 @@
 // qtractor.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2010, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2011, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -44,6 +44,11 @@
 #include "qtractorVstPlugin.h"
 #endif
 
+#ifdef CONFIG_LV2_GTK_UI
+#undef signals // Collides with GTK symbology
+#include <gtk/gtk.h>
+#endif
+
 #ifdef CONFIG_XUNIQUE
 
 #include <QX11Info>
@@ -275,7 +280,8 @@ void stacktrace ( int signo )
 	signal(signo, SIG_DFL);
 
 	static const char *shell  = "/bin/sh";
-	static const char *format = "gdb -q --batch --pid=%d --eval-command=bt";
+	static const char *format = "gdb -q --batch --pid=%d"
+		" --eval-command='thread apply all bt'";
 
 	snprintf(cmd, sizeof(cmd), format, (int) getpid());
 
@@ -324,15 +330,18 @@ int main ( int argc, char **argv )
 	signal(SIGBUS,  stacktrace);
 #endif
 #endif
+#if defined(Q_WS_X11)
 #ifdef CONFIG_VST
 #ifdef CONFIG_XINITTHREADS
-#if defined(Q_WS_X11)
 	if (!XInitThreads()) {
 		qFatal("Error: XInitThreads() failed. Stop.");
 		return 1;
 	}
 #endif
 #endif
+#ifdef CONFIG_LV2_GTK_UI
+	gtk_init(&argc, &argv);
+#endif
 #endif
 	qtractorApplication app(argc, argv);
 
