@@ -1,7 +1,7 @@
 // qtractorAudioBuffer.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2010, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2011, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -25,7 +25,6 @@
 
 #include "qtractorTimeStretcher.h"
 
-
 // Glitch, click, pop-free ramp length (in frames).
 #define QTRACTOR_RAMP_LENGTH	32
 
@@ -34,40 +33,125 @@
 // class qtractorAudioBufferThread -- Ring-cache manager thread.
 //
 
-class qtractorAudioBufferThread : public QThread
+// Constructor.
+qtractorAudioBufferThread::qtractorAudioBufferThread (
+	unsigned int iSyncSize ) : QThread()
 {
-public:
+	m_iSyncSize = (64 << 1);
+	while (m_iSyncSize < iSyncSize)
+		m_iSyncSize <<= 1;
+	m_iSyncMask = (m_iSyncSize - 1);
+	m_ppSyncItems = new qtractorAudioBuffer * [m_iSyncSize];
+	m_iSyncRead   = 0;
+	m_iSyncWrite  = 0;
 
-	// Constructor.
-	qtractorAudioBufferThread(qtractorAudioBuffer *pAudioBuffer);
+	m_bRunState = false;
+}
 
-	// Thread run state accessors.
-	void setRunState(bool bRunState);
-	bool runState() const;
+// Destructor.
+qtractorAudioBufferThread::~qtractorAudioBufferThread (void)
+{
+	delete [] m_ppSyncItems;
+}
 
-	// Wake from executive wait condition (RT-safe).
-	void sync();
+// Run state accessor.
+void qtractorAudioBufferThread::setRunState ( bool bRunState )
+{
+	QMutexLocker locker(&m_mutex);
 
-	// Bypass executive wait condition (non RT-safe).
-	void syncExport();
+	m_bRunState = bRunState;
+}
 
-protected:
+bool qtractorAudioBufferThread::runState (void) const
+{
+	return m_bRunState;
+}
 
-	// The main thread executive.
-	void run();
 
-private:
+// Wake from executive wait condition (RT-safe).
+void qtractorAudioBufferThread::sync ( qtractorAudioBuffer *pAudioBuffer )
+{
+	if (pAudioBuffer == NULL) {
+		unsigned int r = m_iSyncRead;
+		while (r != m_iSyncWrite) {
+			m_ppSyncItems[r]->setSyncFlag(qtractorAudioBuffer::WaitSync, false);
+			++r &= m_iSyncMask;
+		}
+		m_iSyncRead = r;
+	} else if (pAudioBuffer->isSyncFlag(qtractorAudioBuffer::WaitSync)) {
+		return;
+	} else {
+		unsigned int n;
+		unsigned int w = m_iSyncWrite;
+		unsigned int r = m_iSyncRead;
+		if (w > r) {
+			n = ((r - w + m_iSyncSize) & m_iSyncMask) - 1;
+		} else if (r > w) {
+			n = (r - w) - 1;
+		} else {
+			n = m_iSyncSize - 1;
+		}
+		if (n > 0) {
+			pAudioBuffer->setSyncFlag(qtractorAudioBuffer::WaitSync);
+			m_ppSyncItems[m_iSyncWrite] = pAudioBuffer;
+			++m_iSyncWrite &= m_iSyncMask;
+		}
+	}
 
-	// Instance variable.
-	qtractorAudioBuffer* m_pAudioBuffer;
+	if (m_mutex.tryLock()) {
+		m_cond.wakeAll();
+		m_mutex.unlock();
+	}
+#ifdef CONFIG_DEBUG_0
+	else qDebug("qtractorAudioBufferThread[%p]::sync(): tryLock() failed.", this);
+#endif
+}
 
-	// Whether the thread is logically running.
-	bool m_bRunState;
 
-	// Thread synchronization objects.
-	QMutex m_mutex;
-	QWaitCondition m_cond;
-};
+// Bypass executive wait condition (non RT-safe).
+void qtractorAudioBufferThread::syncExport (void)
+{
+	QMutexLocker locker(&m_mutex);
+
+	process();
+}
+
+
+// Thread run executive.
+void qtractorAudioBufferThread::run (void)
+{
+#ifdef CONFIG_DEBUG_0
+	qDebug("qtractorAudioBufferThread[%p]::run(): started.", this);
+#endif
+
+	m_bRunState = true;
+
+	m_mutex.lock();
+	while (m_bRunState) {
+		// Do whatever we must, then wait for more...
+		//m_mutex.unlock();
+		process();
+		//m_mutex.lock();
+		m_cond.wait(&m_mutex);
+	}
+	m_mutex.unlock();
+
+#ifdef CONFIG_DEBUG_0
+	qDebug("qtractorAudioBufferThread[%p]::run(): stopped.", this);
+#endif
+}
+
+
+// Thread run executive.
+void qtractorAudioBufferThread::process (void)
+{
+	unsigned int r = m_iSyncRead;
+	while (r != m_iSyncWrite) {
+		m_ppSyncItems[r]->sync();
+		++r &= m_iSyncMask;
+	}
+	m_iSyncRead = r;
+}
 
 
 //----------------------------------------------------------------------
@@ -75,21 +159,24 @@ private:
 //
 
 // Constructors.
-qtractorAudioBuffer::qtractorAudioBuffer ( unsigned short iChannels,
-	unsigned int iSampleRate )
+qtractorAudioBuffer::qtractorAudioBuffer (
+	qtractorAudioBufferThread *pSyncThread,
+	unsigned short iChannels, unsigned int iSampleRate )
 {
+	m_pSyncThread    = pSyncThread;
+
 	m_iChannels      = iChannels;
 	m_iSampleRate    = iSampleRate;
 
 	m_pFile          = NULL;
 
 	m_pRingBuffer    = NULL;
-	m_pSyncThread    = NULL;
 
 	m_iThreshold     = 0;
 	m_iBufferSize    = 0;
-	m_bInitSync      = false;
-	m_bReadSync      = false;
+
+	m_syncFlags      = 0;
+
 	m_iReadOffset    = 0;
 	m_iWriteOffset   = 0;
 	m_iFileLength    = 0;
@@ -102,7 +189,6 @@ qtractorAudioBuffer::qtractorAudioBuffer ( unsigned short iChannels,
 	m_iLoopEnd       = 0;
 
 	m_iSeekOffset    = 0;
-
 	ATOMIC_SET(&m_seekPending, 0);
 
 	m_ppFrames       = NULL;
@@ -115,7 +201,6 @@ qtractorAudioBuffer::qtractorAudioBuffer ( unsigned short iChannels,
 
 	m_pTimeStretcher = NULL;
 
-	m_fPrevGain      = 0.0f;
 	m_fNextGain      = 1.0f;
 	m_iRampGain      = 1;
 
@@ -236,13 +321,21 @@ bool qtractorAudioBuffer::open ( const QString& sFilename, int iMode )
 	}
 
 	// Allocate ring-buffer now.
-	m_pRingBuffer = new qtractorRingBuffer<float> (iChannels, m_iLength);
+	unsigned int iBufferSize = m_iLength;
+	if (iBufferSize == 0)
+		iBufferSize = (m_iSampleRate >> 1);
+	else
+	if (iBufferSize > (m_iSampleRate << 1))
+		iBufferSize = (m_iSampleRate << 1);
+
+	m_pRingBuffer = new qtractorRingBuffer<float> (iChannels, iBufferSize);
 	m_iThreshold  = (m_pRingBuffer->bufferSize() >> 2);
 	m_iBufferSize = (m_iThreshold >> 2);
+
 #ifdef CONFIG_LIBSAMPLERATE
 	if (m_bResample && m_fResampleRatio < 1.0f) {
-		unsigned int iMinBufferSize = (unsigned int) framesOut(m_iBufferSize);
-		while (m_iBufferSize < iMinBufferSize)
+		iBufferSize = (unsigned int) framesOut(m_iBufferSize);
+		while (m_iBufferSize < iBufferSize)
 			m_iBufferSize <<= 1;
 	}
 #endif
@@ -276,9 +369,9 @@ bool qtractorAudioBuffer::open ( const QString& sFilename, int iMode )
 #endif
 
 	// Make it sync-managed...
-	m_pSyncThread = new qtractorAudioBufferThread(this);
-	m_pSyncThread->start(QThread::HighPriority);
-
+	if (m_pSyncThread)
+		m_pSyncThread->sync(this);
+	
 	return true;
 }
 
@@ -289,16 +382,10 @@ void qtractorAudioBuffer::close (void)
 	if (m_pFile == NULL)
 		return;
 
-	// Not sync-managed anymore...
-	if (m_pSyncThread) {
-		if (m_pSyncThread->isRunning()) do {
-			m_pSyncThread->setRunState(false);
-		//	m_pSyncThread->terminate();
-			m_pSyncThread->sync();
-		} while (!m_pSyncThread->wait(100));
-		delete m_pSyncThread;
-		m_pSyncThread = NULL;
-	}
+	// Avoid any other interference (ought to be atomic)...
+	while (m_pSyncThread && isSyncFlag(WaitSync))
+		m_pSyncThread->syncExport();
+	//setSyncFlag(WaitSync, false);
 
 	// Write-behind any remains, if applicable...
 	if (m_pFile->mode() & qtractorAudioFile::Write) {
@@ -334,8 +421,9 @@ void qtractorAudioBuffer::close (void)
 	// Reset all relevant state variables.
 	m_iThreshold   = 0;
 	m_iBufferSize  = 0;
-	m_bInitSync    = false;
-	m_bReadSync    = false;
+
+	m_syncFlags    = 0;
+
 	m_iReadOffset  = 0;
 	m_iWriteOffset = 0;
 	m_iFileLength  = 0;
@@ -344,7 +432,6 @@ void qtractorAudioBuffer::close (void)
 	m_iSeekOffset  = 0;
 	ATOMIC_SET(&m_seekPending, 0);
 
-	m_fPrevGain = 0.0f;
 	m_fNextGain = 1.0f;
 	m_iRampGain = 1;
 
@@ -358,8 +445,6 @@ int qtractorAudioBuffer::read ( float **ppFrames, unsigned int iFrames,
 {
 	if (m_pRingBuffer == NULL)
 		return -1;
-	if (m_pFile == NULL)
-		return -1;
 
 #ifdef DEBUG_0
 	dump_state(QString("+read(%1)").arg(iFrames));
@@ -367,8 +452,15 @@ int qtractorAudioBuffer::read ( float **ppFrames, unsigned int iFrames,
 
 	int nread;
 
-	// Are we self-contained (ie. got integral file in buffer) and looping?
+	// Are we off decoded file limits (EoS)?
 	unsigned long ro = m_iReadOffset;
+	if (ro >= m_iFileLength && !isSyncFlag(InitSync)) {
+		// Force out-of-sync...
+		setSyncFlag(ReadSync, false);
+		return 0;
+	}
+
+	// Are we self-contained (ie. got integral file in buffer) and looping?
 	unsigned long ls = m_iLoopStart;
 	unsigned long le = m_iLoopEnd;
 	// Are we in the middle of the loop range ?
@@ -399,7 +491,7 @@ int qtractorAudioBuffer::read ( float **ppFrames, unsigned int iFrames,
 	m_iReadOffset = (ro + nread);
 	if (m_iReadOffset >= m_iOffset + m_iLength) {
 		// Force out-of-sync...
-		m_bReadSync = false;
+		setSyncFlag(ReadSync, false);
 	}
 
 #ifdef DEBUG_0
@@ -407,8 +499,9 @@ int qtractorAudioBuffer::read ( float **ppFrames, unsigned int iFrames,
 #endif
 
 	// Time to sync()?
-	if (m_pSyncThread && m_pRingBuffer->writable() > m_iThreshold)
-		m_pSyncThread->sync();
+	if (!m_bIntegral &&
+		m_pSyncThread && m_pRingBuffer->writable() > m_iThreshold)
+		m_pSyncThread->sync(this);
 
 	return nread;
 }
@@ -420,8 +513,6 @@ int qtractorAudioBuffer::write ( float **ppFrames, unsigned int iFrames,
 {
 	if (m_pRingBuffer == NULL)
 		return -1;
-	if (m_pFile == NULL)
-		return -1;
 
 	unsigned short iBuffers = m_pRingBuffer->channels();
 	if (iChannels < 1)
@@ -493,7 +584,7 @@ int qtractorAudioBuffer::write ( float **ppFrames, unsigned int iFrames,
 
 	// Time to sync()?
 	if (m_pSyncThread && m_pRingBuffer->readable() > m_iThreshold)
-		m_pSyncThread->sync();
+		m_pSyncThread->sync(this);
 
 	return nwrite;
 }
@@ -505,8 +596,6 @@ int qtractorAudioBuffer::readMix ( float **ppFrames, unsigned int iFrames,
 {
 	if (m_pRingBuffer == NULL)
 		return -1;
-	if (m_pFile == NULL)
-		return -1;
 
 #ifdef DEBUG_0
 	dump_state(QString("+readMix(%1)").arg(iFrames));
@@ -514,8 +603,15 @@ int qtractorAudioBuffer::readMix ( float **ppFrames, unsigned int iFrames,
 
 	int nread;
 
-	// Are we self-contained (ie. got integral file in buffer) and looping?
+	// Are we off decoded file limits (EoS)?
 	unsigned long ro = m_iReadOffset;
+	if (ro >= m_iFileLength && !isSyncFlag(InitSync)) {
+		// Force out-of-sync...
+		setSyncFlag(ReadSync, false);
+		return 0;
+	}
+
+	// Are we self-contained (ie. got integral file in buffer) and looping?
 	unsigned long ls = m_iLoopStart;
 	unsigned long le = m_iLoopEnd;
 	// Are we in the middle of the loop range ?
@@ -548,7 +644,7 @@ int qtractorAudioBuffer::readMix ( float **ppFrames, unsigned int iFrames,
 	m_iReadOffset = (ro + nread);
 	if (m_iReadOffset >= m_iOffset + m_iLength) {
 		// Force out-of-sync...
-		m_bReadSync = false;
+		setSyncFlag(ReadSync, false);
 	}
 
 #ifdef DEBUG_0
@@ -556,8 +652,9 @@ int qtractorAudioBuffer::readMix ( float **ppFrames, unsigned int iFrames,
 #endif
 
 	// Time to sync()?
-	if (m_pSyncThread && m_pRingBuffer->writable() > m_iThreshold)
-		m_pSyncThread->sync();
+	if (!m_bIntegral &&
+		m_pSyncThread && m_pRingBuffer->writable() > m_iThreshold)
+		m_pSyncThread->sync(this);
 
 	return nread;
 }
@@ -568,34 +665,36 @@ bool qtractorAudioBuffer::seek ( unsigned long iFrame )
 {
 	if (m_pRingBuffer == NULL)
 		return false;
-	if (m_pFile == NULL)
-		return false;
-
-	// Must not break while on initial sync...
-	if (!m_bInitSync)
-		return false;
 
 	// Seek is only valid on read-only mode.
+	if (m_pFile == NULL)
+		return false;
 	if (m_pFile->mode() & qtractorAudioFile::Write)
 		return false;
 
+	// Must not break while on initial sync...
+	if (!isSyncFlag(InitSync))
+		return false;
+
 	// Reset running gain...
-	m_fPrevGain = 0.0f;
 	m_fNextGain = 1.0f;
 	m_iRampGain = 1;
 
-	// Force (premature) out-of-sync...
-	m_bReadSync = false;
-
 	// Are we off-limits?
 	if (iFrame >= m_iLength)
 		return false;
 
+	// Force (premature) out-of-sync...
+	setSyncFlag(ReadSync, false);
+	setSyncFlag(WaitSync, false);
+
 	// Special case on integral cached files...
 	if (m_bIntegral) {
 		m_pRingBuffer->setReadIndex(iFrame);
 	//	m_iWriteOffset = m_iOffset + iFrame;
 		m_iReadOffset  = m_iOffset + iFrame;
+		// Maybe (always) in-sync...
+		setSyncFlag(ReadSync);
 		return true;
 	}
 
@@ -614,6 +713,8 @@ bool qtractorAudioBuffer::seek ( unsigned long iFrame )
 		m_pRingBuffer->setReadIndex(ri + iFrame - ro);
 	//	m_iWriteOffset += iFrame - ro;
 		m_iReadOffset  += iFrame - ro;
+		// Maybe (late) in-sync...
+		setSyncFlag(ReadSync);
 		return true;
 	}
 
@@ -629,89 +730,116 @@ bool qtractorAudioBuffer::seek ( unsigned long iFrame )
 	// Force (late) out-of-sync...
 	//	m_bReadSync = false;
 	m_iReadOffset = m_iOffset + m_iLength + 1; // An unlikely offset!
-	m_iSeekOffset = iFrame;
 
+	m_iSeekOffset = iFrame;
 	ATOMIC_INC(&m_seekPending);
 
 	// readSync();
 	if (m_pSyncThread)
-		m_pSyncThread->sync();
+		m_pSyncThread->sync(this);
 
 	return true;
 }
 
 
+// Sync thread state flags accessors (ought to be inline?).
+void qtractorAudioBuffer::setSyncFlag ( SyncFlag flag, bool bOn )
+{
+	if (bOn)
+		m_syncFlags |=  (unsigned char) (flag);
+	else
+		m_syncFlags &= ~(unsigned char) (flag);
+}
+
+bool qtractorAudioBuffer::isSyncFlag ( SyncFlag flag ) const
+{
+	return (m_syncFlags & (unsigned char) (flag));
+}
+
+
 // Initial thread-sync executive (if file is on read mode,
 // check whether it can be cache-loaded integrally).
 bool qtractorAudioBuffer::initSync (void)
 {
+	if (m_pRingBuffer == NULL)
+		return false;
+
+	// Initialization is only valid on read-only mode.
 	if (m_pFile == NULL)
 		return false;
+	if (m_pFile->mode() & qtractorAudioFile::Write)
+		return false;
+
+	if (isSyncFlag(InitSync))
+		return false;
 
 	// Reset all relevant state variables.
-	m_bInitSync    = false;
-	m_bReadSync    = false;
+	// m_bReadSync = false;
+	setSyncFlag(ReadSync, false);
+
 	m_iReadOffset  = 0;
 	m_iWriteOffset = 0;
 	m_iFileLength  = 0;
 	m_bIntegral    = false;
 
 	m_iSeekOffset  = 0;
-
 	ATOMIC_SET(&m_seekPending, 0);
 
 	// Reset running gain...
-	m_fPrevGain = 0.0f;
 	m_fNextGain = 1.0f;
 	m_iRampGain = 1;
 
-	// Read-ahead a whole bunch, if applicable...
-	if (m_pFile->mode() & qtractorAudioFile::Read) {
-		// Set to initial offset...
+	// Set to initial offset...
+	m_iSeekOffset = m_iOffset;
+	ATOMIC_INC(&m_seekPending);
+
+	// Initial buffer read in...
+	readSync();
+
+	// We're mostly done with initialization...
+	// m_bInitSync = true;
+	setSyncFlag(InitSync);
+
+	// Check if fitted integrally...
+	if (m_iFileLength < m_iOffset + m_pRingBuffer->bufferSize() - 1) {
+		m_bIntegral = true;
+		deleteIOBuffers();
+	}
+	else // Re-sync if loop falls short in initial area... 
+	if (m_iLoopStart < m_iLoopEnd
+		&& m_iWriteOffset >= m_iOffset + m_iLoopEnd) {
+		// Will do it again, but now we're
+		// sure we aren't fit integral...
 		m_iSeekOffset = m_iOffset;
 		ATOMIC_INC(&m_seekPending);
-		// Initial buffer read in...
+		// Initial buffer re-read in...
 		readSync();
-		// We're mostly done with initialization...
-		m_bInitSync = true;
-		// Check if fitted integrally...
-		if (m_iFileLength < m_iOffset + m_pRingBuffer->bufferSize() - 1) {
-			m_bIntegral = true;
-			deleteIOBuffers();
-		}
-		else // Re-sync if loop falls short in initial area... 
-		if (m_iLoopStart < m_iLoopEnd
-			&& m_iWriteOffset >= m_iOffset + m_iLoopEnd) {
-			// Will do it again, but now we're
-			// sure we aren't fit integral...
-			m_iSeekOffset = m_iOffset;
-			ATOMIC_INC(&m_seekPending);
-			// Initial buffer re-read in...
-			readSync();
-		}
 	}
 
-	// Just carry on, if not integrally cached...
-	return !m_bIntegral;
+	// Done.
+	return isSyncFlag(InitSync);
 }
 
 
 // Base-mode sync executive.
 void qtractorAudioBuffer::sync (void)
 {
-	if (m_ppFrames == NULL)
-		return;
-	if (m_pRingBuffer == NULL)
-		return;
 	if (m_pFile == NULL)
 		return;
 
-	int mode = m_pFile->mode();
-	if (mode & qtractorAudioFile::Read)
-		readSync();
-	else
-	if (mode & qtractorAudioFile::Write)
-		writeSync();
+	if (!isSyncFlag(WaitSync))
+		return;
+
+	if (!initSync()) {
+		int mode = m_pFile->mode();
+		if (mode & qtractorAudioFile::Read)
+			readSync();
+		else
+		if (mode & qtractorAudioFile::Write)
+			writeSync();
+	}
+
+	setSyncFlag(WaitSync, false);
 }
 
 
@@ -719,23 +847,23 @@ void qtractorAudioBuffer::sync (void)
 bool qtractorAudioBuffer::inSync (
 	unsigned long iFrameStart, unsigned long iFrameEnd )
 {
-	if (!m_bInitSync)
+	if (!isSyncFlag(InitSync))
 		return false;
 
-	if (!m_bReadSync) {
+	if (!isSyncFlag(ReadSync)) {
 	#ifdef CONFIG_DEBUG
 		qDebug("qtractorAudioBuffer[%p]::inSync(%lu, %lu) (%ld)",
 			this, iFrameStart, iFrameEnd,
 			(long) m_iReadOffset - (iFrameStart + m_iOffset));
 	#endif
 		if (m_iReadOffset == iFrameStart + m_iOffset) {
-			m_bReadSync = true;
+			setSyncFlag(ReadSync);
 		} else {
 			seek(iFrameEnd);
 		}
 	}
 
-	return m_bReadSync;
+	return isSyncFlag(ReadSync);
 }
 
 
@@ -748,36 +876,23 @@ void qtractorAudioBuffer::syncExport (void)
 
 // Read-mode sync executive.
 void qtractorAudioBuffer::readSync (void)
-{
-	// Keep seeking...
-	do {
-
-		// Check whether we have some hard-seek pending...
-		if (ATOMIC_TAZ(&m_seekPending)) {
-			// Do it...
-			if (!seekSync(m_iSeekOffset))
-				return;
-			// Refill the whole buffer....
-			m_pRingBuffer->reset();
-			// Override with new intended offset...
-			m_iWriteOffset = m_iSeekOffset;
-			m_iReadOffset  = m_iSeekOffset;
-		}
-
-		// Internal block-buffering...
-		readSyncIn();
-	}
-	while (ATOMIC_GET(&m_seekPending));
-}
-
-
-// Internal read-mode sync executive
-void qtractorAudioBuffer::readSyncIn (void)
 {
 #ifdef DEBUG
 	dump_state("+readSyncIn()");
 #endif
 
+	// Check whether we have some hard-seek pending...
+	if (ATOMIC_TAZ(&m_seekPending)) {
+		// Do it...
+		if (!seekSync(m_iSeekOffset))
+			return;
+		// Refill the whole buffer....
+		m_pRingBuffer->reset();
+		// Override with new intended offset...
+		m_iWriteOffset = m_iSeekOffset;
+		m_iReadOffset  = m_iSeekOffset;
+	}
+
 	unsigned int ws = m_pRingBuffer->writable();
 	if (ws == 0)
 		return;
@@ -785,11 +900,11 @@ void qtractorAudioBuffer::readSyncIn (void)
 	unsigned int nahead = ws;
 	unsigned int ntotal = 0;
 
-	while (nahead > 0 && !ATOMIC_GET(&m_seekPending)) {
+	while (nahead > 0 && isSyncFlag(WaitSync) && !ATOMIC_GET(&m_seekPending)) {
 		// Take looping into account, if any...
 		unsigned long ls = m_iOffset + m_iLoopStart;
 		unsigned long le = m_iOffset + m_iLoopEnd;
-		bool bLooping = (ls < le && m_iWriteOffset < le && m_bInitSync);
+		bool bLooping = (ls < le && m_iWriteOffset < le && isSyncFlag(InitSync));
 		// Adjust request for sane size...
 		if (nahead > m_iBufferSize)
 			nahead = m_iBufferSize;
@@ -853,7 +968,7 @@ void qtractorAudioBuffer::writeSync (void)
 	unsigned int nbehind = rs;
 	unsigned int ntotal  = 0;
 
-	while (nbehind > 0) {
+	while (nbehind > 0 && isSyncFlag(WaitSync)) {
 		// Adjust request for sane size...
 		if (nbehind > m_iBufferSize)
 			nbehind = m_iBufferSize;
@@ -1075,17 +1190,21 @@ int qtractorAudioBuffer::readMixFrames (
 		return 0;
 
 	// Reset running gain...
+	float fPrevGain = m_fNextGain;
 	m_fNextGain = fGain;
 
 	// HACK: Case of clip ramp out-set...
-	if (m_fNextGain < 1E-6f && m_fPrevGain > (1.0f - 1E-6f)) {
+	if (m_fNextGain < 1E-6f && fPrevGain > (1.0f - 1E-6f)) {
 		// Final micro fade-out...
 		// (anti-glitch out-set ramp)
-		m_fPrevGain = m_fNextGain = 1.0f;
+		m_fNextGain = 1.0f;
 		m_iRampGain = -1;
 	}
 
-	float fGainStep = (m_fNextGain - m_fPrevGain) / float(iFrames);
+	if (m_iRampGain)
+		fPrevGain = m_fNextGain;
+
+	float fGainStep = (m_fNextGain - fPrevGain) / float(iFrames);
 
 	if (iFrames > rs)
 		iFrames = rs;
@@ -1111,7 +1230,7 @@ int qtractorAudioBuffer::readMixFrames (
 		for (i = 0; i < iBuffers; ++i) {
 			pFrames = ppFrames[i] + iOffset;
 			pBuffer = ppBuffer[i];
-			fGainIter = m_fPrevGain;
+			fGainIter = fPrevGain;
 			for (n = 0; n < n1; ++n, fGainIter += fGainStep)
 				*pFrames++ += fGainIter * pBuffer[n + r];
 			for (n = 0; n < n2; ++n, fGainIter += fGainStep)
@@ -1123,7 +1242,7 @@ int qtractorAudioBuffer::readMixFrames (
 		for (i = 0; i < iChannels; ++i) {
 			pFrames = ppFrames[i] + iOffset;
 			pBuffer = ppBuffer[j];
-			fGainIter = m_fPrevGain;
+			fGainIter = fPrevGain;
 			for (n = 0; n < n1; ++n, fGainIter += fGainStep)
 				*pFrames++ += fGainIter * pBuffer[n + r];
 			for (n = 0; n < n2; ++n, fGainIter += fGainStep)
@@ -1137,7 +1256,7 @@ int qtractorAudioBuffer::readMixFrames (
 		for (j = 0; j < iBuffers; ++j) {
 			pFrames = ppFrames[i] + iOffset;
 			pBuffer = ppBuffer[j];
-			fGainIter = m_fPrevGain;
+			fGainIter = fPrevGain;
 			for (n = 0; n < n1; ++n, fGainIter += fGainStep)
 				*pFrames++ += fGainIter * pBuffer[n + r];
 			for (n = 0; n < n2; ++n, fGainIter += fGainStep)
@@ -1165,9 +1284,6 @@ int qtractorAudioBuffer::readMixFrames (
 		m_iRampGain = (m_iRampGain < 0 ? 1 : 0);
 	}
 	
-	// Remember last gain.
-	m_fPrevGain = m_fNextGain;
-
 	return iFrames;
 }
 
@@ -1218,8 +1334,6 @@ void qtractorAudioBuffer::reset ( bool bLooping )
 {
 	if (m_pRingBuffer == NULL)
 		return;
-	if (m_pFile == NULL)
-		return;
 
 #ifdef DEBUG
 	dump_state("+reset()");
@@ -1229,8 +1343,13 @@ void qtractorAudioBuffer::reset ( bool bLooping )
 
 	// If looping, we'll reset to loop-start point,
 	// otherwise it's a buffer full-reset...
-	if (bLooping && m_iLoopStart < m_iLoopEnd)
+	if (bLooping && m_iLoopStart < m_iLoopEnd) {
 		iFrame = m_iLoopStart;
+		// Make sure we're not already there...
+		// (force out-of-sync with an unlikely offset!)
+		if (!m_bIntegral)
+			m_iReadOffset = m_iOffset + m_iLength + 1;
+	}
 
 	seek(iFrame);
 
@@ -1316,7 +1435,7 @@ void qtractorAudioBuffer::setLoop ( unsigned long iLoopStart,
 	}
 
 	// Force out-of-sync...
-	m_bReadSync = false;
+	setSyncFlag(ReadSync, false);
 	m_iReadOffset = m_iOffset + m_iLength + 1; // An unlikely offset!
 //	seek(m_iReadOffset - m_iOffset);
 }
@@ -1448,77 +1567,4 @@ void qtractorAudioBuffer::dump_state ( const QString& sPrefix ) const
 #endif
 
 
-//----------------------------------------------------------------------
-// class qtractorAudioBufferThread -- Ring-cache manager thread.
-//
-
-// Constructor.
-qtractorAudioBufferThread::qtractorAudioBufferThread (
-	qtractorAudioBuffer *pAudioBuffer ) : QThread()
-{
-	m_pAudioBuffer = pAudioBuffer;
-	m_bRunState    = false;
-}
-
-
-// Run state accessor.
-void qtractorAudioBufferThread::setRunState ( bool bRunState )
-{
-	QMutexLocker locker(&m_mutex);
-
-	m_bRunState = bRunState;
-}
-
-bool qtractorAudioBufferThread::runState (void) const
-{
-	return m_bRunState;
-}
-
-
-// Wake from executive wait condition (RT-safe).
-void qtractorAudioBufferThread::sync (void)
-{
-	if (m_mutex.tryLock()) {
-		m_cond.wakeAll();
-		m_mutex.unlock();
-	}
-#ifdef CONFIG_DEBUG_0
-	else qDebug("qtractorAudioBufferThread[%p]::sync(): tryLock() failed.", this);
-#endif
-}
-
-
-// Bypass executive wait condition (non RT-safe).
-void qtractorAudioBufferThread::syncExport (void)
-{
-	QMutexLocker locker(&m_mutex);
-
-	m_pAudioBuffer->sync();
-}
-
-
-// Thread run executive.
-void qtractorAudioBufferThread::run (void)
-{
-#ifdef CONFIG_DEBUG_0
-	qDebug("qtractorAudioBufferThread[%p]::run(): started.", this);
-#endif
-
-	m_mutex.lock();
-	m_bRunState = m_pAudioBuffer->initSync();
-	while (m_bRunState) {
-		// Do whatever we must, then wait for more...
-		//m_mutex.unlock();
-		m_pAudioBuffer->sync();
-		//m_mutex.lock();
-		m_cond.wait(&m_mutex);
-	}
-	m_mutex.unlock();
-
-#ifdef CONFIG_DEBUG_0
-	qDebug("qtractorAudioBufferThread[%p]::run(): stopped.", this);
-#endif
-}
-
-
 // end of qtractorAudioBuffer.cpp
