@@ -149,7 +149,8 @@ bool qtractorAudioClip::openAudioFile ( const QString& sFilename, int iMode )
 	if (bWrite && iChannels < 1)
 		return false;
 
-	m_pBuff = new qtractorAudioBuffer(iChannels, pSession->sampleRate());
+	m_pBuff = new qtractorAudioBuffer(
+		pTrack->syncThread(), iChannels, pSession->sampleRate());
 	m_pBuff->setOffset(clipOffset());
 	m_pBuff->setLength(clipLength());
 	m_pBuff->setTimeStretch(m_fTimeStretch);
@@ -331,7 +332,7 @@ void qtractorAudioClip::draw ( QPainter *pPainter, const QRect& clipRect,
 	// Needed an even number of polygon points...
 	bool bZoomedIn = (clipRect.width() > int(nframes));
 	unsigned int iPolyPoints
-		= (bZoomedIn ? nframes : (clipRect.width() >> 1)) << 1;
+		= (bZoomedIn ? nframes: (clipRect.width() >> 1)) << 1;
 	if (iPolyPoints < 2)
 		return;
 
@@ -355,33 +356,34 @@ void qtractorAudioClip::draw ( QPainter *pPainter, const QRect& clipRect,
 	int h2 = (h1 >> 1);
 	int h2gain = (h2 * m_fractGain.num); 
 	int ymax, yrms;
-	unsigned int n;
+	unsigned int n, n2;
 	int x, y;
 
 	// Build polygonal vertexes...
 	if (bZoomedIn) {
 		// Zoomed in...
 		// - trade peak-frames for pixels.
+		n2 = nframes - 1;
 		for (n = 0; n < nframes; ++n) {
-			x = clipRect.x() + (n * clipRect.width()) / nframes;
+			x = clipRect.x() + (n * clipRect.width()) / n2;
 			y = clipRect.y() + h2;
-			for (i = 0; i < iChannels; ++i, ++pframes) {
+			for (i = 0; i < iChannels; ++i) {
 				ymax = (h2gain * pframes->max) >> m_fractGain.den;
 				yrms = (h2gain * pframes->rms) >> m_fractGain.den;
 				pPolyMax[i]->setPoint(n, x, y + ymax);
 				pPolyMax[i]->setPoint(iPolyPoints - n - 1, x, y - ymax);
 				pPolyRms[i]->setPoint(n, x, y + yrms);
 				pPolyRms[i]->setPoint(iPolyPoints - n - 1, x, y - yrms);
-				y += h1;
+				y += h1; ++pframes;
 			}
 		}
 	} else {
 		// Zoomed out...
 		// - trade (2) pixels for peak-frames (expensiver).
 		int x2, k = 0;
-		unsigned int n2, n0 = 0;
+		unsigned int n0 = 0;
 		unsigned char v, vmax, vrms;
-		for (x2 = 0; x2 < clipRect.width(); x2 += 2, ++k) {
+		for (x2 = 0; x2 < clipRect.width(); x2 += 2) {
 			x = clipRect.x() + x2;
 			y = clipRect.y() + h2;
 			n = (iChannels * x2 * nframes) / clipRect.width();
@@ -405,6 +407,7 @@ void qtractorAudioClip::draw ( QPainter *pPainter, const QRect& clipRect,
 				y += h1;
 			}
 			n0 = n + iChannels;
+			++k;
 		}
 	}
 
@@ -445,8 +448,8 @@ QString qtractorAudioClip::toolTip (void) const
 				sToolTip += QObject::tr("\n\t(%1% time stretch)")
 					.arg(100.0f * m_pBuff->timeStretch(), 0, 'g', 3);
 			if (m_pBuff->isPitchShift())
-				sToolTip += QObject::tr("\n\t(%1% pitch shift)")
-					.arg(100.0f * m_pBuff->pitchShift(), 0, 'g', 3);
+				sToolTip += QObject::tr("\n\t(%1 semitones pitch shift)")
+					.arg(12.0f * ::logf(m_pBuff->pitchShift()) / M_LN2, 0, 'g', 2);
 		}
 	}
 
@@ -519,8 +522,8 @@ bool qtractorAudioClip::clipExport ( ClipExport pfnClipExport, void *pvArg,
 	if (iLength < 1)
 		iLength = clipLength();
 
-	qtractorAudioBuffer *pBuff
-		= new qtractorAudioBuffer(iChannels, pSession->sampleRate());
+	qtractorAudioBuffer *pBuff = new qtractorAudioBuffer(
+		pTrack->syncThread(), iChannels, pSession->sampleRate());
 	pBuff->setOffset(iOffset);
 	pBuff->setLength(iLength);
 	pBuff->setTimeStretch(timeStretch());
