@@ -1,7 +1,7 @@
 // qtractorAudioClip.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2010, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2011, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -77,7 +77,7 @@ public:
 	void set_loop(unsigned long iLoopStart, unsigned long iLoopEnd);
 
 	// Clip close-commit (record specific)
-	void close(bool bForce);
+	void close();
 
 	// Audio clip special process cycle executive.
 	void process(unsigned long iFrameStart, unsigned long iFrameEnd);
@@ -98,10 +98,8 @@ public:
 protected:
 
 	// Virtual document element methods.
-	bool loadClipElement(qtractorSessionDocument *pDocument,
-		QDomElement *pElement);
-	bool saveClipElement(qtractorSessionDocument *pDocument,
-		QDomElement *pElement);
+	bool loadClipElement(qtractorDocument *pDocument, QDomElement *pElement);
+	bool saveClipElement(qtractorDocument *pDocument, QDomElement *pElement) const;
 
 private:
 
