@@ -36,6 +36,8 @@
 #include "qtractorPlugin.h"
 #include "qtractorClip.h"
 
+#include "qtractorCurveFile.h"
+
 #ifdef CONFIG_JACK_SESSION
 #include <jack/session.h>
 #endif
@@ -314,13 +316,17 @@ qtractorAudioEngine::qtractorAudioEngine ( qtractorSession *pSession )
 {
 	m_pJackClient = NULL;
 
+	m_iSampleRate = 0;
 	m_iBufferSize = 0;
 
 	m_iBufferOffset = 0;
 
 	// Audio-export freewheeling (internal) state.
 	m_bFreewheel = false;
-	
+
+	// Common audio buffer sync thread.
+	m_pSyncThread = NULL;
+
 	// Audio-export (in)active state.
 	m_bExporting   = false;
 	m_pExportBus   = NULL;
@@ -407,7 +413,7 @@ jack_client_t *qtractorAudioEngine::jackClient (void) const
 // Internal sample-rate accessor.
 unsigned int qtractorAudioEngine::sampleRate (void) const
 {
-	return (m_pJackClient ? jack_get_sample_rate(m_pJackClient) : 0);
+	return m_iSampleRate;
 }
 
 // Buffer size accessor.
@@ -458,15 +464,20 @@ bool qtractorAudioEngine::init (void)
 	if (m_pJackClient == NULL)
 		return false;
 
-	// ATTN: First thing to remember is get initial buffer size.
+	// ATTN: First thing to remember is initial sample-rate and buffer size.
+	m_iSampleRate = jack_get_sample_rate(m_pJackClient);
 	m_iBufferSize = jack_get_buffer_size(m_pJackClient);
 
-	// ATTN: First thing to remember to set session sample rate.
+	// ATTN: Second is setting proper session client name.
 	pSession->setClientName(
 		QString::fromUtf8(jack_get_client_name(m_pJackClient)));
 
-	// ATTN: Second thing to remember to set session sample rate.
-	pSession->setSampleRate(sampleRate());
+	// ATTN: Third is setting session sample rate.
+	pSession->setSampleRate(m_iSampleRate);
+
+	// Our dedicated audio buffer thread...
+	m_pSyncThread = new qtractorAudioBufferThread();
+	m_pSyncThread->start(QThread::HighPriority);
 
 	// Open player/metronome buses, at least try...
 	openPlayerBus();
@@ -533,6 +544,9 @@ bool qtractorAudioEngine::activate (void)
 			qtractorAudioEngine_sync, this);
 	}
 
+    // Reset all dependable monitoring...
+    resetAllMonitors();
+
 	// Time to activate ourselves...
 	jack_activate(m_pJackClient);
 
@@ -572,7 +586,10 @@ bool qtractorAudioEngine::start (void)
 	if (!isActivated())
 	    return false;
 
-	// Make sure we have an actual session cursor...
+    // Reset all dependables...
+    resetAllMonitors();
+
+    // Make sure we have an actual session cursor...
 	resetMetro();
 
 	// Start transport rolling...
@@ -624,6 +641,17 @@ void qtractorAudioEngine::clean (void)
 	deletePlayerBus();
 	deleteMetroBus();
 
+	// Terminate common player/metro sync thread...
+	if (m_pSyncThread) {
+		if (m_pSyncThread->isRunning()) do {
+			m_pSyncThread->setRunState(false);
+		//	m_pSyncThread->terminate();
+			m_pSyncThread->sync();
+		} while (!m_pSyncThread->wait(100));
+		delete m_pSyncThread;
+		m_pSyncThread = NULL;
+	}
+
 	// Audio-export stilll around? weird...
 	if (m_pExportFile) {
 		delete m_pExportFile;
@@ -697,6 +725,8 @@ int qtractorAudioEngine::process ( unsigned int nframes )
 				m_pExportFile->write(m_pExportBus->out(), nframes);
 				// Prepare advance for next cycle...
 				pAudioCursor->seek(iFrameEnd);
+				// HACK! Freewheeling observers update (non RT safe!)...
+				qtractorSubject::flushQueue();
 			}	// Are we trough?
 			else m_bExportDone = true;
 		}
@@ -745,7 +775,7 @@ int qtractorAudioEngine::process ( unsigned int nframes )
 		if (m_bPlayerBus && m_pPlayerBus)
 			m_pPlayerBus->process_commit(nframes);
 		else
-			iOutputBus++;
+			++iOutputBus;
 		ATOMIC_SET(&m_playerLock, 0);
 	}
 
@@ -758,7 +788,7 @@ int qtractorAudioEngine::process ( unsigned int nframes )
 		while (pMidiManager) {
 			pMidiManager->process(iTimeStart, iTimeEnd);
 			if (!pMidiManager->isAudioOutputBus())
-				iOutputBus++;
+				++iOutputBus;
 			pMidiManager = pMidiManager->next();
 		}
 	}
@@ -794,7 +824,7 @@ int qtractorAudioEngine::process ( unsigned int nframes )
 								(pTrack->pluginList())->process(
 									pOutputBus->buffer(), nframes);
 							pOutputBus->buffer_commit(nframes);
-							iOutputBus++;
+							++iOutputBus;
 						}
 					}
 				}
@@ -1122,6 +1152,12 @@ bool qtractorAudioEngine::fileExport ( const QString& sExportPath,
 		return false;
 	}
 
+	// We'll be busy...
+	pSession->lock();
+
+	// HACK! reset subject/observers queue...
+	qtractorSubject::resetQueue();
+
 	// Start with fixing the export range...
 	m_bExporting   = true;
 	m_pExportBus   = pExportBus;
@@ -1169,7 +1205,8 @@ bool qtractorAudioEngine::fileExport ( const QString& sExportPath,
 	bool bResult = m_bExporting;
 
 	// Free up things here.
-	delete m_pExportFile;
+	delete m_pExportFile;	
+
 	m_bExporting   = false;
 	m_pExportBus   = NULL;
 	m_pExportFile  = NULL;
@@ -1177,6 +1214,9 @@ bool qtractorAudioEngine::fileExport ( const QString& sExportPath,
 	m_iExportEnd   = 0;
 	m_bExportDone  = true;
 
+	// Back to business..
+	pSession->unlock();
+
 	// Done whether successfully.
 	return bResult;
 }
@@ -1290,11 +1330,6 @@ void qtractorAudioEngine::resetMetroBus (void)
 void qtractorAudioEngine::setMetroBarFilename ( const QString& sFilename )
 {
 	m_sMetroBarFilename = sFilename;
-
-	if (m_pMetroBarBuff) {
-		m_pMetroBarBuff->setLength(0);
-		m_pMetroBarBuff->open(m_sMetroBarFilename);
-	}
 }
 
 const QString& qtractorAudioEngine::metroBarFilename (void) const
@@ -1319,11 +1354,6 @@ float qtractorAudioEngine::metroBarGain (void) const
 void qtractorAudioEngine::setMetroBeatFilename ( const QString& sFilename )
 {
 	m_sMetroBeatFilename = sFilename;
-
-	if (m_pMetroBeatBuff) {
-		m_pMetroBeatBuff->setLength(0);
-		m_pMetroBeatBuff->open(m_sMetroBeatFilename);
-	}
 }
 
 const QString& qtractorAudioEngine::metroBeatFilename() const
@@ -1397,9 +1427,12 @@ bool qtractorAudioEngine::openMetroBus (void)
 	}
 
 	// We got it...
-	m_pMetroBarBuff = new qtractorAudioBuffer(iChannels, sampleRate());
+	unsigned int iSampleRate = sampleRate();
+	m_pMetroBarBuff = new qtractorAudioBuffer(
+		m_pSyncThread, iChannels, iSampleRate);
 	m_pMetroBarBuff->open(m_sMetroBarFilename);
-	m_pMetroBeatBuff = new qtractorAudioBuffer(iChannels, sampleRate());
+	m_pMetroBeatBuff = new qtractorAudioBuffer(
+		m_pSyncThread, iChannels, iSampleRate);
 	m_pMetroBeatBuff->open(m_sMetroBeatFilename);
 
 	return true;
@@ -1409,18 +1442,6 @@ bool qtractorAudioEngine::openMetroBus (void)
 // Close audio metronome stuff.
 void qtractorAudioEngine::closeMetroBus (void)
 {
-	if (m_pMetroBarBuff) {
-		m_pMetroBarBuff->close();
-		delete m_pMetroBarBuff;
-		m_pMetroBarBuff = NULL;
-	}
-
-	if (m_pMetroBeatBuff) {
-		m_pMetroBeatBuff->close();
-		delete m_pMetroBeatBuff;
-		m_pMetroBeatBuff = NULL;
-	}
-
 	if (m_bMetroBus && m_pMetroBus) {
 		m_pMetroBus->close();
 		removeBusEx(m_pMetroBus);
@@ -1436,6 +1457,18 @@ void qtractorAudioEngine::deleteMetroBus (void)
 {
 	closeMetroBus();
 
+	if (m_pMetroBarBuff) {
+		m_pMetroBarBuff->close();
+		delete m_pMetroBarBuff;
+		m_pMetroBarBuff = NULL;
+	}
+
+	if (m_pMetroBeatBuff) {
+		m_pMetroBeatBuff->close();
+		delete m_pMetroBeatBuff;
+		m_pMetroBeatBuff = NULL;
+	}
+
 	if (m_bMetroBus && m_pMetroBus)
 		delete m_pMetroBus;
 
@@ -1574,7 +1607,8 @@ bool qtractorAudioEngine::openPlayerBus (void)
 	}
 
 	// We got it...
-	m_pPlayerBuff = new qtractorAudioBuffer(iChannels, sampleRate());
+	m_pPlayerBuff = new qtractorAudioBuffer(
+		m_pSyncThread, iChannels, sampleRate());
 
 	return true;
 }
@@ -1709,6 +1743,47 @@ int qtractorAudioEngine::ramping (void) const
 }
 
 
+// Absolute number of frames elapsed since engine start.
+unsigned long qtractorAudioEngine::jackFrame (void) const
+{
+	return (m_pJackClient ? jack_frame_time(m_pJackClient) : 0);
+}
+
+
+// Reset all audio monitoring...
+void qtractorAudioEngine::resetAllMonitors (void)
+{
+    // There must a session reference...
+    qtractorSession *pSession = session();
+    if (pSession == NULL)
+        return;
+
+	// Reset all audio bus monitors...
+	for (qtractorBus *pBus = buses().first();
+			pBus; pBus = pBus->next()) {
+		qtractorAudioBus *pAudioBus
+			= static_cast<qtractorAudioBus *> (pBus);
+		if (pAudioBus) {
+		//	if (pAudioBus->audioMonitor_in())
+		//		pAudioBus->audioMonitor_in()->reset();
+			if (pAudioBus->audioMonitor_out())
+				pAudioBus->audioMonitor_out()->reset();
+		}
+	}
+	
+	// Reset all audio track channel monitors...
+	for (qtractorTrack *pTrack = pSession->tracks().first();
+			pTrack; pTrack = pTrack->next()) {
+		if (pTrack->trackType() == qtractorTrack::Audio) {
+			qtractorAudioMonitor *pAudioMonitor
+				= static_cast<qtractorAudioMonitor *> (pTrack->monitor());
+			if (pAudioMonitor)
+				pAudioMonitor->reset();
+		}
+	}
+}
+
+
 //----------------------------------------------------------------------
 // class qtractorAudioBus -- Managed JACK port set
 //
@@ -1724,17 +1799,21 @@ qtractorAudioBus::qtractorAudioBus ( qtractorAudioEngine *pAudioEngine,
 	if (busMode & qtractorBus::Input) {
 		m_pIAudioMonitor = new qtractorAudioMonitor(iChannels);
 		m_pIPluginList   = createPluginList(qtractorPluginList::AudioInBus);
+		m_pICurveFile    = new qtractorCurveFile(m_pIPluginList->curveList());
 	} else {
 		m_pIAudioMonitor = NULL;
 		m_pIPluginList   = NULL;
+		m_pICurveFile    = NULL;
 	}
 
 	if (busMode & qtractorBus::Output) {
 		m_pOAudioMonitor = new qtractorAudioMonitor(iChannels);
 		m_pOPluginList   = createPluginList(qtractorPluginList::AudioOutBus);
+		m_pOCurveFile    = new qtractorCurveFile(m_pOPluginList->curveList());
 	} else {
 		m_pOAudioMonitor = NULL;
 		m_pOPluginList   = NULL;
+		m_pOCurveFile    = NULL;
 	}
 
 	m_bAutoConnect = bAutoConnect;
@@ -1769,6 +1848,11 @@ qtractorAudioBus::~qtractorAudioBus (void)
 	if (m_pOAudioMonitor)
 		delete m_pOAudioMonitor;
 
+	if (m_pICurveFile)
+		delete m_pICurveFile;
+	if (m_pOCurveFile)
+		delete m_pOCurveFile;
+
 	if (m_pIPluginList)
 		delete m_pIPluginList;
 	if (m_pOPluginList)
@@ -2016,11 +2100,17 @@ void qtractorAudioBus::updateBusMode (void)
 			m_pIAudioMonitor = new qtractorAudioMonitor(m_iChannels);
 		if (m_pIPluginList == NULL)
 			m_pIPluginList = createPluginList(qtractorPluginList::AudioInBus);
+		if (m_pICurveFile == NULL)
+			m_pICurveFile = new qtractorCurveFile(m_pIPluginList->curveList());
 	} else {
 		if (m_pIAudioMonitor) {
 			delete m_pIAudioMonitor;
 			m_pIAudioMonitor = NULL;
 		}
+		if (m_pICurveFile) {
+			delete m_pICurveFile;
+			m_pICurveFile = NULL;
+		}
 		if (m_pIPluginList) {
 			delete m_pIPluginList;
 			m_pIPluginList = NULL;
@@ -2033,11 +2123,17 @@ void qtractorAudioBus::updateBusMode (void)
 			m_pOAudioMonitor = new qtractorAudioMonitor(m_iChannels);
 		if (m_pOPluginList == NULL)
 			m_pOPluginList = createPluginList(qtractorPluginList::AudioOutBus);
+		if (m_pOCurveFile == NULL)
+			m_pOCurveFile = new qtractorCurveFile(m_pOPluginList->curveList());
 	} else {
 		if (m_pOAudioMonitor) {
 			delete m_pOAudioMonitor;
 			m_pOAudioMonitor = NULL;
 		}
+		if (m_pOCurveFile) {
+			delete m_pOCurveFile;
+			m_pOCurveFile = NULL;
+		}
 		if (m_pOPluginList) {
 			delete m_pOPluginList;
 			m_pOPluginList = NULL;
@@ -2228,6 +2324,108 @@ qtractorPluginList *qtractorAudioBus::pluginList_out (void) const
 }
 
 
+// Automation curve list accessors.
+qtractorCurveList *qtractorAudioBus::curveList_in (void) const
+{
+	return (m_pIPluginList ? m_pIPluginList->curveList() : NULL);
+}
+
+qtractorCurveList *qtractorAudioBus::curveList_out (void) const
+{
+	return (m_pOPluginList ? m_pOPluginList->curveList() : NULL);
+}
+
+
+// Automation curve serializer accessors.
+qtractorCurveFile *qtractorAudioBus::curveFile_in (void) const
+{
+	return m_pICurveFile;
+}
+
+qtractorCurveFile *qtractorAudioBus::curveFile_out (void) const
+{
+	return m_pOCurveFile;
+}
+
+
+// Audio I/O port latency accessors.
+unsigned int qtractorAudioBus::latency_in (void) const
+{
+	if (m_ppIPorts == NULL)
+		return 0;
+
+	qtractorAudioEngine *pAudioEngine
+		= static_cast<qtractorAudioEngine *> (engine());
+	if (pAudioEngine == NULL)
+		return 0;
+
+	unsigned int iLatencyIn = pAudioEngine->bufferSize();
+
+#ifdef CONFIG_JACK_LATENCY
+	jack_nframes_t range_min = 0;
+	jack_latency_range_t range;
+	for (unsigned int i = 0; i < m_iChannels; ++i) {
+		if (m_ppIPorts[i] == NULL)
+			continue;
+		jack_port_get_latency_range(m_ppIPorts[i], JackCaptureLatency, &range);
+		if (range_min > range.min || i == 0)
+			range_min = range.min;
+	}
+	iLatencyIn += range_min;
+#else
+	jack_nframes_t lat, lat_min = 0;
+	for (unsigned int i = 0; i < m_iChannels; ++i) {
+		if (m_ppIPorts[i] == NULL)
+			continue;
+		lat = jack_port_get_latency(m_ppIPorts[i]);
+		if (lat_min > lat || i == 0)
+			lat_min = lat;
+	}
+	iLatencyIn += lat_min;
+#endif
+
+	return iLatencyIn;
+}
+
+unsigned int qtractorAudioBus::latency_out (void) const
+{
+	if (m_ppOPorts == NULL)
+		return 0;
+
+	qtractorAudioEngine *pAudioEngine
+		= static_cast<qtractorAudioEngine *> (engine());
+	if (pAudioEngine == NULL)
+		return 0;
+
+	unsigned int iLatencyOut = pAudioEngine->bufferSize();
+
+#ifdef CONFIG_JACK_LATENCY
+	jack_nframes_t range_min = 0;
+	jack_latency_range_t range;
+	for (unsigned int i = 0; i < m_iChannels; ++i) {
+		if (m_ppOPorts[i] == NULL)
+			continue;
+		jack_port_get_latency_range(m_ppOPorts[i], JackPlaybackLatency, &range);
+		if (range_min > range.min || i == 0)
+			range_min = range.min;
+	}
+	iLatencyOut += range_min;
+#else
+	jack_nframes_t lat, lat_min = 0;
+	for (unsigned int i = 0; i < m_iChannels; ++i) {
+		if (m_ppOPorts[i] == NULL)
+			continue;
+		lat = jack_port_get_latency(m_ppOPorts[i]);
+		if (lat_min > lat || i == 0)
+			lat_min = lat;
+	}
+	iLatencyOut += lat_min;
+#endif
+
+	return iLatencyOut;
+}
+
+
 // Create plugin-list properly.
 qtractorPluginList *qtractorAudioBus::createPluginList ( int iFlags ) const
 {
@@ -2308,7 +2506,7 @@ int qtractorAudioBus::updateConnects ( qtractorBus::BusMode busMode,
 
 	// For each channel...
 	ConnectItem item;
-	for (item.index = 0; item.index < m_iChannels; item.index++) {
+	for (item.index = 0; item.index < m_iChannels; ++item.index) {
 		// Get port connections...
 		const char **ppszClientPorts = jack_port_get_all_connections(
 			pAudioEngine->jackClient(), ppPorts[item.index]);
@@ -2331,7 +2529,7 @@ int qtractorAudioBus::updateConnects ( qtractorBus::BusMode busMode,
 				}
 				else if (!bConnect)
 					connects.append(new ConnectItem(item));
-				iClientPort++;
+				++iClientPort;
 			}
 			::free(ppszClientPorts);
 		}
@@ -2377,7 +2575,7 @@ int qtractorAudioBus::updateConnects ( qtractorBus::BusMode busMode,
 			if (iItem >= 0) {
 				connects.removeAt(iItem);
 				delete pItem;
-				iUpdate++;
+				++iUpdate;
 			}
 		}
 	}
@@ -2418,6 +2616,9 @@ bool qtractorAudioBus::loadElement (
 					eProp.text().toFloat());
 		} else if (eProp.tagName() == "input-controllers") {
 			qtractorAudioBus::loadControllers(&eProp, qtractorBus::Input);
+		} else if (eProp.tagName() == "input-curve-file") {
+			qtractorAudioBus::loadCurveFile(&eProp, qtractorBus::Input,
+				qtractorAudioBus::curveFile_in());
 		} else if (eProp.tagName() == "input-plugins") {
 			if (qtractorAudioBus::pluginList_in())
 				qtractorAudioBus::pluginList_in()->loadElement(
@@ -2435,6 +2636,9 @@ bool qtractorAudioBus::loadElement (
 					eProp.text().toFloat());
 		} else if (eProp.tagName() == "output-controllers") {
 			qtractorAudioBus::loadControllers(&eProp, qtractorBus::Output);
+		} else if (eProp.tagName() == "output-curve-file") {
+			qtractorAudioBus::loadCurveFile(&eProp, qtractorBus::Output,
+				qtractorAudioBus::curveFile_out());
 		} else if (eProp.tagName() == "output-plugins") {
 			if (qtractorAudioBus::pluginList_out())
 				qtractorAudioBus::pluginList_out()->loadElement(
@@ -2475,11 +2679,23 @@ bool qtractorAudioBus::saveElement (
 				QString::number(qtractorAudioBus::monitor_in()->panning()),
 					pElement);
 		}
+		// Save input bus controllers...
 		QDomElement eInputControllers
 			= pDocument->document()->createElement("input-controllers");
 		qtractorAudioBus::saveControllers(pDocument,
 			&eInputControllers, qtractorBus::Input);
 		pElement->appendChild(eInputControllers);
+		// Save input bus automation curves...
+		qtractorCurveList *pInputCurveList = qtractorAudioBus::curveList_in();
+		if (pInputCurveList && !pInputCurveList->isEmpty()) {
+			qtractorCurveFile cfile(pInputCurveList);
+			QDomElement eInputCurveFile
+				= pDocument->document()->createElement("input-curve-file");
+			qtractorAudioBus::saveCurveFile(pDocument,
+				&eInputCurveFile, qtractorBus::Input, &cfile);
+			pElement->appendChild(eInputCurveFile);
+		}
+		// Save input bus plugins...
 		if (qtractorAudioBus::pluginList_in()) {
 			QDomElement eInputPlugins
 				= pDocument->document()->createElement("input-plugins");
@@ -2487,6 +2703,7 @@ bool qtractorAudioBus::saveElement (
 				pDocument, &eInputPlugins);
 			pElement->appendChild(eInputPlugins);
 		}
+		// Save input bus connections...
 		QDomElement eAudioInputs
 			= pDocument->document()->createElement("input-connects");
 		qtractorBus::ConnectList inputs;
@@ -2504,11 +2721,23 @@ bool qtractorAudioBus::saveElement (
 				QString::number(qtractorAudioBus::monitor_out()->panning()),
 					pElement);
 		}
+		// Save output bus controllers...
 		QDomElement eOutputControllers
 			= pDocument->document()->createElement("output-controllers");
 		qtractorAudioBus::saveControllers(pDocument,
 			&eOutputControllers, qtractorBus::Output);
 		pElement->appendChild(eOutputControllers);
+		// Save output bus automation curves...
+		qtractorCurveList *pOutputCurveList = qtractorAudioBus::curveList_out();
+		if (pOutputCurveList && !pOutputCurveList->isEmpty()) {
+			qtractorCurveFile cfile(pOutputCurveList);
+			QDomElement eOutputCurveFile
+				= pDocument->document()->createElement("output-curve-file");
+			qtractorAudioBus::saveCurveFile(pDocument,
+				&eOutputCurveFile, qtractorBus::Output, &cfile);
+			pElement->appendChild(eOutputCurveFile);
+		}
+		// Save output bus plugins...
 		if (qtractorAudioBus::pluginList_out()) {
 			QDomElement eOutputPlugins
 				= pDocument->document()->createElement("output-plugins");
@@ -2516,6 +2745,7 @@ bool qtractorAudioBus::saveElement (
 				pDocument, &eOutputPlugins);
 			pElement->appendChild(eOutputPlugins);
 		}
+		// Save output bus connections...
 		QDomElement eAudioOutputs
 			= pDocument->document()->createElement("output-connects");
 		qtractorBus::ConnectList outputs;
