@@ -36,6 +36,7 @@ class qtractorAudioBuffer;
 class qtractorAudioMonitor;
 class qtractorAudioFile;
 class qtractorPluginList;
+class qtractorCurveList;
 
 
 //----------------------------------------------------------------------
@@ -192,6 +193,12 @@ public:
 	void setRamping(int iRamping);
 	int ramping() const;
 
+	// Absolute number of frames elapsed since engine start.
+	unsigned long jackFrame() const;
+
+    // Reset all audio monitoring...
+    void resetAllMonitors();
+
 protected:
 
 	// Concrete device (de)activation methods.
@@ -225,7 +232,8 @@ private:
 	// JACK Session UUID.
 	QString m_sSessionId;
 
-	// Initial buffer size;
+	// Initial sample-rate and buffer size.
+	unsigned int m_iSampleRate;
 	unsigned int m_iBufferSize;
 
 	// Partial buffer offset state;
@@ -235,6 +243,9 @@ private:
 	// Audio-export freewheeling (internal) state.
 	bool m_bFreewheel;
 
+	// Common audio buffer sync thread.
+	qtractorAudioBufferThread *m_pSyncThread;
+
 	// Audio-export (in)active state.
 	volatile bool        m_bExporting;
 	qtractorAudioBus    *m_pExportBus;
@@ -336,6 +347,18 @@ public:
 	qtractorPluginList *pluginList_in()  const;
 	qtractorPluginList *pluginList_out() const;
 
+	// Automation curve list accessors.
+	qtractorCurveList *curveList_in()  const;
+	qtractorCurveList *curveList_out() const;
+
+	// Automation curve serializer accessors.
+	qtractorCurveFile *curveFile_in()  const;
+	qtractorCurveFile *curveFile_out() const;
+
+	// Audio I/O port latency accessors.
+	unsigned int latency_in()  const;
+	unsigned int latency_out() const;
+
 	// Retrieve/restore client:port connections;
 	// return the effective number of connection attempts...
 	int updateConnects(BusMode busMode,
@@ -375,6 +398,10 @@ private:
 	qtractorPluginList *m_pIPluginList;
 	qtractorPluginList *m_pOPluginList;
 
+	// Automation curve serializer instances.
+	qtractorCurveFile  *m_pICurveFile;
+	qtractorCurveFile  *m_pOCurveFile;
+
 	// Specific JACK ports stuff.
 	jack_port_t **m_ppIPorts;
 	jack_port_t **m_ppOPorts;
