@@ -27,6 +27,7 @@
 
 #include "qtractorSession.h"
 #include "qtractorAudioClip.h"
+#include "qtractorAudioEngine.h"
 #include "qtractorMidiClip.h"
 #include "qtractorFiles.h"
 
@@ -60,6 +61,8 @@ qtractorClipCommand::~qtractorClipCommand (void)
 
 	qDeleteAll(m_trackCommands);
 	m_trackCommands.clear();
+
+	m_clips.clear();
 }
 
 
@@ -85,6 +88,8 @@ void qtractorClipCommand::fileClip ( qtractorClip *pClip,
 	pItem->filename = sFilename;
 	pItem->trackChannel = iTrackChannel;
 	m_items.append(pItem);
+
+	reopenClip(pClip, true);
 }
 
 
@@ -115,7 +120,7 @@ void qtractorClipCommand::moveClip ( qtractorClip *pClip,
 
 void qtractorClipCommand::resizeClip ( qtractorClip *pClip,
 	unsigned long iClipStart, unsigned long iClipOffset,
-	unsigned long iClipLength, float fTimeStretch )
+	unsigned long iClipLength, float fTimeStretch, float fPitchShift )
 {
 	Item *pItem = new Item(ResizeClip, pClip, pClip->track());
 	pItem->clipStart  = iClipStart;
@@ -141,7 +146,12 @@ void qtractorClipCommand::resizeClip ( qtractorClip *pClip,
 				static_cast<qtractorMidiClip *> (pClip), fTimeStretch);
 		}
 	}
+	if (fPitchShift > 0.0f)
+		pItem->pitchShift = fPitchShift;
 	m_items.append(pItem);
+
+	if (pItem->editCommand == NULL)
+		reopenClip(pClip, fTimeStretch > 0.0f);
 }
 
 
@@ -179,6 +189,8 @@ void qtractorClipCommand::timeStretchClip ( qtractorClip *pClip,
 	Item *pItem = new Item(TimeStretchClip, pClip, pClip->track());
 	pItem->timeStretch = fTimeStretch;
 	m_items.append(pItem);
+
+	reopenClip(pClip, true);
 }
 
 
@@ -188,6 +200,8 @@ void qtractorClipCommand::pitchShiftClip ( qtractorClip *pClip,
 	Item *pItem = new Item(PitchShiftClip, pClip, pClip->track());
 	pItem->pitchShift = fPitchShift;
 	m_items.append(pItem);
+
+	reopenClip(pClip);
 }
 
 
@@ -203,24 +217,28 @@ void qtractorClipCommand::resetClip ( qtractorClip *pClip,
 			pItem->fadeOutLength = iFadeOutLength;
 	}
 	m_items.append(pItem);
+
+	reopenClip(pClip);
+}
+
+
+void qtractorClipCommand::reopenClip ( qtractorClip *pClip, bool bClose )
+{
+	QHash<qtractorClip *, bool>::ConstIterator iter
+		= m_clips.constFind(pClip);
+	if (iter == m_clips.constEnd())
+		m_clips.insert(pClip, bClose);
 }
 
 
 // Special clip record nethod.
-bool qtractorClipCommand::addClipRecord ( qtractorTrack *pTrack )
+bool qtractorClipCommand::addClipRecord (
+	qtractorTrack *pTrack, unsigned long iClipEnd )
 {
 	qtractorClip *pClip = pTrack->clipRecord();
 	if (pClip == NULL)
 		return false;
 
-	qtractorSession *pSession = qtractorSession::getInstance();
-	if (pSession == NULL)
-		return false;
-
-	// Arrange for formal clip length...
-	unsigned long iClipEnd = (pSession->isPunching()
-		? pSession->punchOut() : pSession->framePos());
-
 	unsigned long iClipStart = pClip->clipStart();
 	if (iClipStart >= iClipEnd)
 		return false;
@@ -246,6 +264,10 @@ bool qtractorClipCommand::addClipRecord ( qtractorTrack *pTrack )
 			pAudioClip = new qtractorAudioClip(*pAudioClip);
 			pAudioClip->setClipStart(iClipStart);
 			pAudioClip->setClipLength(iClipLength);
+			qtractorAudioBus *pAudioBus
+				= static_cast<qtractorAudioBus *> (pTrack->inputBus());
+			if (pAudioBus)
+				pAudioClip->setClipOffset(pAudioBus->latency_in());
 			addClip(pAudioClip, pTrack);
 			if (pMainForm)
 				pMainForm->addAudioFile(pAudioClip->filename());
@@ -351,15 +373,22 @@ bool qtractorClipCommand::execute ( bool bRedo )
 
 	pSession->lock();
 
-	QListIterator<qtractorAddTrackCommand *> track(m_trackCommands);
+	QListIterator<qtractorTrackCommand *> track(m_trackCommands);
 	while (track.hasNext()) {
-	    qtractorAddTrackCommand *pTrackCommand = track.next();
+	    qtractorTrackCommand *pTrackCommand = track.next();
 		if (bRedo)
 			pTrackCommand->redo();
 		else
 			pTrackCommand->undo();
 	}
 
+	// Pre-close needed clips once...
+	QHash<qtractorClip *, bool>::ConstIterator clip;
+	for (clip = m_clips.constBegin(); clip != m_clips.constEnd(); ++clip) {
+		if (clip.value()) // Scrap peak file (audio).
+			clip.key()->close();
+	}
+
 	QListIterator<Item *> iter(m_items);
 	while (iter.hasNext()) {
 		Item *pItem = iter.next();
@@ -396,8 +425,8 @@ bool qtractorClipCommand::execute ( bool bRedo )
 				iOldTrackChannel = pMidiClip->trackChannel();
 				pMidiClip->setTrackChannel(pItem->trackChannel);
 			}
-			pClip->close();	// Scrap peak file (audio).
-			pClip->open();
+		//--pClip->close();	// Scrap peak file (audio).
+		//--pClip->open();
 			pItem->filename = sOldFilename;
 			if (pMidiClip)
 				pItem->trackChannel = iOldTrackChannel;
@@ -442,18 +471,22 @@ bool qtractorClipCommand::execute ( bool bRedo )
 			unsigned long iOldFadeIn = pClip->fadeInLength();
 			unsigned long iOldFadeOut = pClip->fadeOutLength();
 			float fOldTimeStretch = 0.0f;
+			float fOldPitchShift  = 0.0f;
 			qtractorAudioClip *pAudioClip = NULL;
 			if (pTrack->trackType() == qtractorTrack::Audio) {
 				pAudioClip = static_cast<qtractorAudioClip *> (pClip);
 				if (pAudioClip) {
-					if (pItem->timeStretch > 0.0f)
+					if (pItem->timeStretch > 0.0f) {
 						fOldTimeStretch = pAudioClip->timeStretch();
-					pAudioClip->close(); // Scrap peak file.
+					//--pAudioClip->close(); // Scrap peak file.
+					}
+					if (pItem->pitchShift > 0.0f)
+						fOldPitchShift = pAudioClip->pitchShift();
 				}
 			}
-			else
-			if (pItem->editCommand == NULL)
-				pClip->close();
+		//--else
+		//--if (pItem->editCommand == NULL)
+		//--	pClip->close();
 			if (iOldStart != pItem->clipStart)
 				pTrack->unlinkClip(pClip);
 			pClip->setClipStart(pItem->clipStart);
@@ -461,9 +494,15 @@ bool qtractorClipCommand::execute ( bool bRedo )
 			pClip->setClipLength(pItem->clipLength);
 			pClip->setFadeInLength(pItem->fadeInLength);
 			pClip->setFadeOutLength(pItem->fadeOutLength);
-			if (pAudioClip && pItem->timeStretch > 0.0f) {
-				pAudioClip->setTimeStretch(pItem->timeStretch);
-				pItem->timeStretch = fOldTimeStretch;
+			if (pAudioClip) {
+				if (pItem->timeStretch > 0.0f) {
+					pAudioClip->setTimeStretch(pItem->timeStretch);
+					pItem->timeStretch = fOldTimeStretch;
+				}
+				if (pItem->pitchShift > 0.0f) {
+					pAudioClip->setPitchShift(pItem->pitchShift);
+					pItem->pitchShift = fOldPitchShift;
+				}
 			}
 			if (pItem->editCommand) {
 				if (bRedo)
@@ -471,7 +510,7 @@ bool qtractorClipCommand::execute ( bool bRedo )
 				else
 					(pItem->editCommand)->undo();
 			}
-			else pClip->open();
+		//--else pClip->open();
 			if (iOldStart != pItem->clipStart)
 				pTrack->insertClip(pClip);
 			pItem->clipStart  = iOldStart;
@@ -513,9 +552,9 @@ bool qtractorClipCommand::execute ( bool bRedo )
 			if (pAudioClip) {
 				float fOldTimeStretch = pAudioClip->timeStretch();
 				pAudioClip->setTimeStretch(pItem->timeStretch);
-				pAudioClip->close();			// Scrap peak file.
+			//--pAudioClip->close();			// Scrap peak file.
 				pAudioClip->updateClipTime();	// Care of tempo change.
-				pAudioClip->open();
+			//--pAudioClip->open();
 				pItem->timeStretch = fOldTimeStretch;
 			}
 			break;
@@ -527,7 +566,7 @@ bool qtractorClipCommand::execute ( bool bRedo )
 			if (pAudioClip) {
 				float fOldPitchShift = pAudioClip->pitchShift();
 				pAudioClip->setPitchShift(pItem->pitchShift);
-				pAudioClip->open();
+			//--pAudioClip->open();
 				pItem->pitchShift = fOldPitchShift;
 			}
 			break;
@@ -537,7 +576,7 @@ bool qtractorClipCommand::execute ( bool bRedo )
 			unsigned long iOldFadeOut = pClip->fadeOutLength();
 			pClip->setClipLength(pItem->clipLength);
 			pClip->setFadeOutLength(pItem->fadeOutLength);
-			pClip->open();
+		//--pClip->open();
 			pItem->clipLength = iOldLength;
 			pItem->fadeOutLength = iOldFadeOut;
 			pSession->updateTrack(pTrack);
@@ -548,6 +587,10 @@ bool qtractorClipCommand::execute ( bool bRedo )
 		}
 	}
 
+	// Re-open needed clips, just once...
+	for (clip = m_clips.constBegin(); clip != m_clips.constEnd(); ++clip)
+		clip.key()->open();
+
 	pSession->unlock();
 
 	return true;
