@@ -1,7 +1,7 @@
 // qtractorClipCommand.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2010, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2011, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -29,7 +29,7 @@
 #include <QList>
 
 // Forward declarations.
-class qtractorAddTrackCommand;
+class qtractorTrackCommand;
 class qtractorMidiEditCommand;
 class qtractorMidiClip;
 
@@ -60,7 +60,7 @@ public:
 		unsigned long iClipLength);
 	void resizeClip(qtractorClip *pClip, unsigned long iClipStart,
 		unsigned long iClipOffset, unsigned long iClipLength,
-		float fTimeStretch = 0.0f);
+		float fTimeStretch = 0.0f, float fPitchShift = 0.0f);
 	void gainClip(qtractorClip *pClip, float fGain);
 	void fadeInClip(qtractorClip *pClip, unsigned long iFadeInLength,
 		qtractorClip::FadeType fadeInType);
@@ -70,8 +70,10 @@ public:
 	void pitchShiftClip(qtractorClip *pClip, float fPitchShift);
 	void resetClip(qtractorClip *pClip, unsigned long iClipLength);
 
+	void reopenClip(qtractorClip *pClip, bool bClose = false);
+
 	// Special clip record method.
-	bool addClipRecord(qtractorTrack *pTrack);
+	bool addClipRecord(qtractorTrack *pTrack, unsigned long iClipEnd);
 
 	// When new tracks are needed.
 	void addTrack(qtractorTrack *pTrack);
@@ -140,7 +142,10 @@ private:
 	QList<Item *> m_items;
 
 	// When new tracks are needed.
-	QList<qtractorAddTrackCommand *> m_trackCommands;
+	QList<qtractorTrackCommand *> m_trackCommands;
+
+	// When clips need to reopem.
+	QHash<qtractorClip *, bool> m_clips;
 };
 
 
