@@ -35,8 +35,10 @@
 #include "qtractorMixer.h"
 
 #include "qtractorDocument.h"
+#include "qtractorCurveFile.h"
 
 #include <QDomDocument>
+#include <QFileInfo>
 
 
 //----------------------------------------------------------------------
@@ -327,33 +329,6 @@ int qtractorEngine::updateConnects ( qtractorBus *pBus )
 }
 
 
-//------------------------------------------------------------------------
-// qtractorBus::MonitorObserver -- Local bus state observer.
-
-class qtractorBus::MonitorObserver : public qtractorMidiControlObserver
-{
-public:
-
-	// Constructor.
-	MonitorObserver(qtractorBus *pBus, qtractorSubject *pSubject)
-		: qtractorMidiControlObserver(pSubject), m_pBus(pBus) {}
-
-protected:
-
-	// Update feedback.
-	void update()
-	{
-		m_pBus->monitorChangeNotify(value() > 0.0f);
-		qtractorMidiControlObserver::update();
-	}
-
-private:
-
-	// Members.
-	qtractorBus *m_pBus;
-};
-
-
 //----------------------------------------------------------------------
 // class qtractorBus -- Managed ALSA sequencer port set
 //
@@ -366,8 +341,11 @@ qtractorBus::qtractorBus ( qtractorEngine *pEngine,
 	m_sBusName = sBusName;
 	m_busMode  = busMode;
 
-	m_pMonitorSubject = new qtractorSubject(bMonitor ? 1.0f : 0.0f);
-	m_pMonitorObserver = new MonitorObserver(this, m_pMonitorSubject);
+	m_pMonitorSubject = new qtractorSubject();
+	m_pMonitorSubject->setToggled(true);
+
+	m_pMonitorObserver = new qtractorMidiControlObserver(m_pMonitorSubject);
+	m_pMonitorObserver->setValue(bMonitor ? 1.0f : 0.0f);
 }
 
 // Destructor.
@@ -448,7 +426,7 @@ qtractorSubject *qtractorBus::monitorSubject (void) const
 
 qtractorMidiControlObserver *qtractorBus::monitorObserver (void) const
 {
-	return static_cast<qtractorMidiControlObserver *> (m_pMonitorObserver);
+	return m_pMonitorObserver;
 }
 
 
@@ -467,7 +445,7 @@ void qtractorBus::monitorChangeNotify ( bool bOn )
 }
 
 
-// Load track state (record, mute, solo) controllers (MIDI).
+// Load bus (monitor, gain, pan) controllers (MIDI).
 void qtractorBus::loadControllers ( QDomElement *pElement, BusMode busMode )
 {
 	if (busMode & Input)
@@ -477,7 +455,7 @@ void qtractorBus::loadControllers ( QDomElement *pElement, BusMode busMode )
 }
 
 
-// Save track state (record, mute, solo) controllers (MIDI).
+// Save bus (monitor, gain, pan) controllers (MIDI).
 void qtractorBus::saveControllers (
 	qtractorDocument *pDocument, QDomElement *pElement, BusMode busMode ) const
 {
@@ -513,12 +491,14 @@ void qtractorBus::saveControllers (
 	if (pMidiControl->isMidiObserverMapped(m_pMonitorObserver)) {
 		qtractorMidiControl::Controller *pController
 			= new qtractorMidiControl::Controller;
+		pController->name = m_pMonitorObserver->subject()->name();
 		pController->index = 0; // 0=MonitorObserver
 		pController->ctype = m_pMonitorObserver->type();
 		pController->channel = m_pMonitorObserver->channel();
 		pController->param = m_pMonitorObserver->param();
 		pController->logarithmic = m_pMonitorObserver->isLogarithmic();
 		pController->feedback = m_pMonitorObserver->isFeedback();
+		pController->invert = m_pMonitorObserver->isInvert();
 		controllers.append(pController);
 	}
 
@@ -527,12 +507,14 @@ void qtractorBus::saveControllers (
 	if (pMidiControl->isMidiObserverMapped(pPanObserver)) {
 		qtractorMidiControl::Controller *pController
 			= new qtractorMidiControl::Controller;
+		pController->name = m_pMonitorObserver->subject()->name();
 		pController->index = 1; // 1=PanObserver
 		pController->ctype = pPanObserver->type();
 		pController->channel = pPanObserver->channel();
 		pController->param = pPanObserver->param();
 		pController->logarithmic = pPanObserver->isLogarithmic();
 		pController->feedback = pPanObserver->isFeedback();
+		pController->invert = pPanObserver->isInvert();
 		controllers.append(pController);
 	}
 
@@ -541,12 +523,14 @@ void qtractorBus::saveControllers (
 	if (pMidiControl->isMidiObserverMapped(pGainObserver)) {
 		qtractorMidiControl::Controller *pController
 			= new qtractorMidiControl::Controller;
+		pController->name = m_pMonitorObserver->subject()->name();
 		pController->index = 2; // 2=GainObserver
 		pController->ctype = pGainObserver->type();
 		pController->channel = pGainObserver->channel();
 		pController->param = pGainObserver->param();
 		pController->logarithmic = pGainObserver->isLogarithmic();
 		pController->feedback = pGainObserver->isFeedback();
+		pController->invert = pGainObserver->isInvert();
 		controllers.append(pController);
 	}
 
@@ -557,7 +541,7 @@ void qtractorBus::saveControllers (
 }
 
 
-// Map track state (record, mute, solo) controllers (MIDI).
+// Map bus (monitor, gain, pan) controllers (MIDI).
 void qtractorBus::mapControllers ( BusMode busMode )
 {
 	qtractorMidiControl *pMidiControl = qtractorMidiControl::getInstance();
@@ -581,7 +565,7 @@ void qtractorBus::mapControllers ( BusMode busMode )
 		pMonitor = monitor_out();
 		pMixerRack = pMixer->outputRack();
 	}
-	
+
 	qtractorMixerStrip *pMixerStrip	= pMixerRack->findStrip(pMonitor);
 	if (pMixerStrip == NULL)
 		return;
@@ -609,6 +593,7 @@ void qtractorBus::mapControllers ( BusMode busMode )
 			pObserver->setParam(pController->param);
 			pObserver->setLogarithmic(pController->logarithmic);
 			pObserver->setFeedback(pController->feedback);
+			pObserver->setInvert(pController->invert);
 			pMidiControl->mapMidiObserver(pObserver);
 		}
 	}
@@ -618,6 +603,185 @@ void qtractorBus::mapControllers ( BusMode busMode )
 }
 
 
+// Load bus automation curves (monitor, gain, pan).
+void qtractorBus::loadCurveFile (
+	QDomElement *pElement, BusMode /*busMode*/, qtractorCurveFile *pCurveFile )
+{
+	if (pCurveFile) pCurveFile->load(pElement);
+}
+
+
+// Save bus automation curves (monitor, gain, pan).
+void qtractorBus::saveCurveFile ( qtractorDocument *pDocument,
+	QDomElement *pElement, BusMode busMode, qtractorCurveFile *pCurveFile ) const
+{
+	if (pCurveFile == NULL)
+		return;
+
+	qtractorCurveList *pCurveList = pCurveFile->list();
+	if (pCurveList == NULL)
+		return;
+
+	qtractorSession *pSession = m_pEngine->session();
+	if (pSession == NULL)
+		return;
+
+	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+	if (pMainForm == NULL)
+		return;
+
+	qtractorMixer *pMixer = pMainForm->mixer();
+	if (pMixer == NULL)
+		return;
+
+	QString sBusName(busName());
+	qtractorMonitor *pMonitor = NULL;
+	qtractorMixerRack *pMixerRack = NULL;
+	if (busMode & Input) {
+		pMonitor = monitor_in();
+		pMixerRack = pMixer->inputRack();
+		sBusName += "_in";
+	} else {
+		pMonitor = monitor_out();
+		pMixerRack = pMixer->outputRack();
+		sBusName += "_out";
+	}
+	
+	qtractorMixerStrip *pMixerStrip	= pMixerRack->findStrip(pMonitor);
+	if (pMixerStrip == NULL)
+		return;
+
+	pCurveFile->clear();
+	pCurveFile->setBaseDir(pSession->sessionDir());
+
+	qtractorCurve *pCurve;
+
+	if (busMode & Input) { // It suffices for Duplex...
+	pCurve = monitorSubject()->curve();
+	if (pCurve) {
+		qtractorCurveFile::Item *pCurveItem = new qtractorCurveFile::Item;
+		pCurveItem->name = pCurve->subject()->name();
+		pCurveItem->index = 0;	// 0=MonitorSubject
+		pCurveItem->ctype = qtractorMidiEvent::CONTROLLER;
+		pCurveItem->channel = 0;
+		pCurveItem->param = 80;	// 80=General Purpose Button 1 (on/off)
+		pCurveItem->mode = pCurve->mode();
+		pCurveItem->process = pCurve->isProcess();
+		pCurveItem->capture = pCurve->isCapture();
+		pCurveItem->locked = pCurve->isLocked();
+		pCurveItem->logarithmic = pCurve->isLogarithmic();
+		pCurveItem->color = pCurve->color();
+		pCurveItem->subject = pCurve->subject();
+		pCurveFile->addItem(pCurveItem);
+	}}
+
+	pCurve = pMixerStrip->meter()->panningSubject()->curve();
+	if (pCurve) {
+		qtractorCurveFile::Item *pCurveItem = new qtractorCurveFile::Item;
+		pCurveItem->name = pCurve->subject()->name();
+		pCurveItem->index = 1;	// 1=PanningSubject
+		pCurveItem->ctype = qtractorMidiEvent::CONTROLLER;
+		pCurveItem->channel = 0;
+		pCurveItem->param = 10;	// 10=Pan Position (coarse)
+		pCurveItem->mode = pCurve->mode();
+		pCurveItem->process = pCurve->isProcess();
+		pCurveItem->capture = pCurve->isCapture();
+		pCurveItem->locked = pCurve->isLocked();
+		pCurveItem->logarithmic = pCurve->isLogarithmic();
+		pCurveItem->color = pCurve->color();
+		pCurveItem->subject = pCurve->subject();
+		pCurveFile->addItem(pCurveItem);
+	}
+
+	pCurve = pMixerStrip->meter()->gainSubject()->curve();
+	if (pCurve) {
+		qtractorCurveFile::Item *pCurveItem = new qtractorCurveFile::Item;
+		pCurveItem->name = pCurve->subject()->name();
+		pCurveItem->index = 2; // 2=GainSubject
+		pCurveItem->ctype = qtractorMidiEvent::CONTROLLER;
+		pCurveItem->channel = 0;
+		pCurveItem->param = 7;	// 7=Volume (coarse)
+		pCurveItem->mode = pCurve->mode();
+		pCurveItem->process = pCurve->isProcess();
+		pCurveItem->capture = pCurve->isCapture();
+		pCurveItem->logarithmic = pCurve->isLogarithmic();
+		pCurveItem->locked = pCurve->isLocked();
+		pCurveItem->color = pCurve->color();
+		pCurveItem->subject = pCurve->subject();
+		pCurveFile->addItem(pCurveItem);
+	}
+
+	if (pCurveFile->isEmpty())
+		return;
+
+	const QString sBaseName(sBusName + "_curve");
+	int iClipNo = (pCurveFile->filename().isEmpty() ? 0 : 1);
+	pCurveFile->setFilename(pSession->createFilePath(sBaseName, "mid", iClipNo));
+
+	pCurveFile->save(pDocument, pElement, pSession->timeScale());
+}
+
+
+// Apply bus automation curves (monitor, gain, pan).
+void qtractorBus::applyCurveFile ( BusMode busMode, qtractorCurveFile *pCurveFile ) const
+{
+	if (pCurveFile == NULL)
+		return;
+	if (pCurveFile->items().isEmpty())
+		return;
+
+	qtractorCurveList *pCurveList = pCurveFile->list();
+	if (pCurveList == NULL)
+		return;
+
+	qtractorSession *pSession = m_pEngine->session();
+	if (pSession == NULL)
+		return;
+
+	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+	if (pMainForm == NULL)
+		return;
+
+	qtractorMixer *pMixer = pMainForm->mixer();
+	if (pMixer == NULL)
+		return;
+
+	qtractorMonitor *pMonitor = NULL;
+	qtractorMixerRack *pMixerRack = NULL;
+	if (busMode & Input) {
+		pMonitor = monitor_in();
+		pMixerRack = pMixer->inputRack();
+	} else {
+		pMonitor = monitor_out();
+		pMixerRack = pMixer->outputRack();
+	}
+	
+	qtractorMixerStrip *pMixerStrip = pMixerRack->findStrip(pMonitor);
+	if (pMixerStrip == NULL)
+		return;
+
+	pCurveFile->setBaseDir(pSession->sessionDir());
+
+	QListIterator<qtractorCurveFile::Item *> iter(pCurveFile->items());
+	while (iter.hasNext()) {
+		qtractorCurveFile::Item *pCurveItem = iter.next();
+		switch (pCurveItem->index) {
+		case 0: // 0=MonitorSubject
+			pCurveItem->subject = monitorSubject();
+			break;
+		case 1: // 1=PanSubject
+			pCurveItem->subject = pMixerStrip->meter()->panningSubject();
+			break;
+		case 2: // 2=GainSubject
+			pCurveItem->subject = pMixerStrip->meter()->gainSubject();
+			break;
+		}
+	}
+
+	pCurveFile->apply(pSession->timeScale());
+}
+
+
 // Document element methods.
 bool qtractorBus::loadConnects ( ConnectList& connects,
 	qtractorDocument * /*pDocument*/, QDomElement *pElement )
