@@ -27,6 +27,7 @@
 // Forward declarations.
 class qtractorBus;
 class qtractorSessionCursor;
+class qtractorCurveFile;
 
 class QDomElement;
 
@@ -177,15 +178,22 @@ public:
 	// State (monitor) notifier (proto-slot).
 	void monitorChangeNotify(bool bOn);
 
-	// Load/save track state (record, mute, soloe) controllers (MIDI).
+	// Load/save bus (monitor, gain, pan) controllers (MIDI).
 	void loadControllers(
 		QDomElement *pElement, BusMode busMode);
 	void saveControllers(qtractorDocument *pDocument,
 		QDomElement *pElement, BusMode busMode) const;
 
-	// Map track state (record, mute, solo) controllers (MIDI).
+	// Map bus (monitor, gain, pan) controllers (MIDI).
 	void mapControllers(BusMode busMode);
 
+	// Bus automation curve serialization methods.
+	static void loadCurveFile(
+		QDomElement *pElement, BusMode busMode, qtractorCurveFile *pCurveFile);
+	void saveCurveFile(qtractorDocument *pDocument,
+		QDomElement *pElement, BusMode busMode, qtractorCurveFile *pCurveFile) const;
+	void applyCurveFile(BusMode busMode, qtractorCurveFile *pCurveFile) const;
+
 	// Connection list stuff.
 	struct ConnectItem
 	{
@@ -281,10 +289,10 @@ private:
 	ConnectList m_outputs;
 
 	// State (monitor) observer stuff.
-	class MonitorObserver;
-	MonitorObserver *m_pMonitorObserver;
 	qtractorSubject *m_pMonitorSubject;
 
+	qtractorMidiControlObserver *m_pMonitorObserver;
+
 	qtractorMidiControl::Controllers m_controllers_in;
 	qtractorMidiControl::Controllers m_controllers_out;
 };
