@@ -492,6 +492,7 @@ void qtractorFileListView::openFile (void)
 		return;
 
 	// Find a proper group parent group item...
+	qtractorFileListItem *pFileItem = NULL;
 	qtractorFileGroupItem *pParentItem = currentGroupItem();
 	// Pick each one of the selected files...
 	int iUpdate = 0;
@@ -499,15 +500,19 @@ void qtractorFileListView::openFile (void)
 	while (iter.hasNext()) {
 		const QString& sPath = iter.next();
 		// Add the new file item...
-		qtractorFileListItem *pFileItem	= addFileItem(sPath, pParentItem);
+		pFileItem = addFileItem(sPath, pParentItem);
 		// Make all this new open and visible.
 		if (pFileItem) {
-			iUpdate++;
+			++iUpdate;
 			if (pParentItem)
 				pParentItem->setOpen(true);
 		}
 	}
 
+	// Make the last one current...
+	if (pFileItem)
+		QTreeWidget::setCurrentItem(pFileItem);
+		
 	// Make proper notifications...
 	if (iUpdate > 0)
 		emit contentsChanged();
@@ -545,7 +550,7 @@ void qtractorFileListView::copyItem ( bool bCut )
 				urls.append(QUrl::fromLocalFile(pFileItem->path()));
 				if (bCut) {
 					delete pFileItem;
-					iUpdate++;
+					++iUpdate;
 				}
 			}
 		}
@@ -582,7 +587,7 @@ void qtractorFileListView::pasteItem (void)
 			qtractorFileListItem *pFileItem	= addFileItem(sPath, pParentItem);
 			// Make all this new open and visible.
 			if (pFileItem) {
-				iUpdate++;
+				++iUpdate;
 				if (pParentItem)
 					pParentItem->setOpen(true);
 			}
@@ -630,7 +635,7 @@ void qtractorFileListView::deleteItem (void)
 		while (iter.hasNext()) {
 			QTreeWidgetItem *pItem = iter.next();
 			delete pItem;
-			iUpdate++;
+			++iUpdate;
 		}
 	} else {
 		QTreeWidgetItem *pItem = QTreeWidget::currentItem();
@@ -650,7 +655,7 @@ void qtractorFileListView::deleteItem (void)
 			}
 			// Definite single-delete...
 			delete pItem;
-			iUpdate++;
+			++iUpdate;
 		}
 	}
 
@@ -1151,9 +1156,9 @@ void qtractorFileListView::dropEvent ( QDropEvent *pDropEvent )
 			// Is it one from ourselves (file item) ?...
 			if (m_pDragItem && m_pDragItem->type() == FileItem) {
 				if (dropItem(pDropItem, findItem(sPath, FileItem), bOutdent))
-					iUpdate++;
+					++iUpdate;
 			} else if (addFileItem(sPath, pParentItem))
-				iUpdate++;
+				++iUpdate;
 		}
 	} else if (pMimeData->hasText()) {
 		// Maybe its just a new convenience group...
@@ -1161,9 +1166,9 @@ void qtractorFileListView::dropEvent ( QDropEvent *pDropEvent )
 		// Is it one from ourselves (group item) ?...
 		if (m_pDragItem && m_pDragItem->type() == GroupItem) {
 			if (dropItem(pDropItem, findItem(sText, GroupItem), bOutdent))
-				iUpdate++;
+				++iUpdate;
 		} else if (addGroupItem(sText, pParentItem))
-			iUpdate++;
+			++iUpdate;
 	}
 
 	// Teke care of change modification...
