@@ -1,7 +1,7 @@
 // qtractorFiles.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2010, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2011, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -313,10 +313,10 @@ void qtractorFiles::selectMidiFile ( const QString& sFilename,
 // Audition/pre-listening player methods.
 void qtractorFiles::setPlayState ( bool bOn )
 {
-	m_iPlayUpdate++;
+	++m_iPlayUpdate;
 	m_pPlayItemAction->setChecked(bOn);
 	m_pPlayButton->setChecked(bOn);
-	m_iPlayUpdate--;
+	--m_iPlayUpdate;
 }
 
 bool qtractorFiles::isPlayState (void) const
