@@ -1,7 +1,7 @@
 // qtractorInstrument.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2011, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -194,7 +194,7 @@ bool qtractorInstrumentList::load ( const QString& sFilename )
 	while (!ts.atEnd()) {
 
 		// Read the line.
-		iLine++;
+		++iLine;
 		const QString& sLine = ts.readLine().simplified();
 		// If not empty, nor a comment, call the server...
 		if (sLine.isEmpty() || sLine[0] == ';')
@@ -627,7 +627,7 @@ void qtractorInstrumentList::loadSoundFontPresets ( QFile *pFile, int iSize )
 					patch[int(prog)] = QString(name).simplified();
 					if (bank == 128 && iDrums == 0) {
 						instr.setDrum(128, -1, true); // Usual SF2 standard ;)
-						iDrums++;
+						++iDrums;
 					}
 				}
 			}
