@@ -1,7 +1,7 @@
 // qtractorInstrument.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2011, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -93,7 +93,7 @@ protected:
 
 	// Copy/clone method.
 	void attach(const qtractorInstrumentData& data)
-		{  m_pData = data.m_pData; m_pData->refCount++; }
+		{  m_pData = data.m_pData; ++(m_pData->refCount); }
 
 	// Destroy method.
 	void detach()
@@ -234,7 +234,7 @@ protected:
 
 	// Copy/clone method.
 	void attach(const qtractorInstrument& instr)
-		{  m_pData = instr.m_pData; m_pData->refCount++; }
+		{  m_pData = instr.m_pData; ++(m_pData->refCount); }
 
 	// Destroy method.
 	void detach()
