@@ -49,6 +49,9 @@ class qtractorTempoCursor;
 class qtractorMidiEditorForm;
 class qtractorMidiEditor;
 
+class qtractorMidiControlObserver;
+class qtractorSubject;
+
 class QLabel;
 class QComboBox;
 class QProgressBar;
@@ -97,6 +100,10 @@ public:
 		{ return m_ui.editMenu; }
 	QMenu *trackMenu() const
 		{ return m_ui.trackMenu; }
+	QMenu *trackCurveMenu() const
+		{ return m_ui.trackCurveMenu; }
+	QMenu *trackCurveModeMenu() const
+		{ return m_ui.trackCurveModeMenu; }
 
 	QProgressBar *progressBar() const
 		{ return m_pProgressBar; }
@@ -127,26 +134,13 @@ public slots:
 	void editSelectModeClip();
 	void editSelectModeRange();
 	void editSelectModeRect();
+	void editSelectModeCurve(bool bOn);
+	void editSelectAll();
 	void editSelectNone();
-	void editSelectRange();
+	void editSelectInvert();
 	void editSelectTrack();
 	void editSelectTrackRange();
-	void editSelectAll();
-	void editClipNew();
-	void editClipEdit();
-	void editClipSplit();
-	void editClipMerge();
-	void editClipNormalize();
-	void editClipToolsQuantize();
-	void editClipToolsTranspose();
-	void editClipToolsNormalize();
-	void editClipToolsRandomize();
-	void editClipToolsResize();
-	void editClipToolsRescale();
-	void editClipToolsTimeshift();
-	void editClipTempo();
-	void editClipImport();
-	void editClipExport();
+	void editSelectRange();
 
 	void trackAdd();
 	void trackRemove();
@@ -166,11 +160,43 @@ public slots:
 	void trackMoveUp();
 	void trackMoveDown();
 	void trackMoveBottom();
+	void trackHeightUp();
+	void trackHeightDown();
+	void trackHeightReset();
 	void trackAutoMonitor(bool bOn);
 	void trackImportAudio();
 	void trackImportMidi();
 	void trackExportAudio();
 	void trackExportMidi();
+	void trackCurveSelect(bool On);
+	void trackCurveSelect(QAction *pAction, bool bOn = true);
+	void trackCurveMode(QAction *pAction);
+	void trackCurveProcess(bool bOn);
+	void trackCurveCapture(bool bOn);
+	void trackCurveLocked(bool bOn);
+	void trackCurveLogarithmic(bool bOn);
+	void trackCurveColor();
+	void trackCurveClear();
+	void trackCurveProcessAll(bool bOn);
+	void trackCurveCaptureAll(bool bOn);
+	void trackCurveLockedAll(bool bOn);
+	void trackCurveClearAll();
+
+	void clipNew();
+	void clipEdit();
+	void clipSplit();
+	void clipMerge();
+	void clipNormalize();
+	void clipToolsQuantize();
+	void clipToolsTranspose();
+	void clipToolsNormalize();
+	void clipToolsRandomize();
+	void clipToolsResize();
+	void clipToolsRescale();
+	void clipToolsTimeshift();
+	void clipTempo();
+	void clipImport();
+	void clipExport();
 
 	void viewMenubar(bool bOn);
 	void viewStatusbar(bool bOn);
@@ -242,6 +268,10 @@ public slots:
 	void midiClkNotify(float fTempo);
 
 	void updateRecentFilesMenu();
+	void updateTrackMenu();
+	void updateCurveMenu();
+	void updateCurveModeMenu();
+	void updateClipMenu();
 	void updateExportMenu();
 	void updateZoomMenu();
 	void updateSnapMenu();
@@ -260,6 +290,7 @@ public slots:
 	void selectionNotifySlot(qtractorMidiEditor *pMidiEditor);
 	void changeNotifySlot(qtractorMidiEditor *pMidiEditor);
 	void updateNotifySlot(bool bRefresh);
+	void dirtyNotifySlot();
 
 	void transportTimeChanged(unsigned long iPlayHead);
 	void transportTimeFinished();
@@ -289,8 +320,13 @@ protected:
 	bool editSession();
 	bool closeSession();
 
-	bool loadSessionFile(const QString& sFilename, bool bTemplate = false);
-	bool saveSessionFile(const QString& sFilename, bool bTemplate = false);
+	bool loadSessionFileEx(
+		const QString& sFilename, bool bTemplate, bool bUpdate = false);
+	bool loadSessionFile(const QString& sFilename);
+
+	bool saveSessionFileEx(
+	    const QString& sFilename, bool bTemplate, bool bUpdate = false);
+	bool saveSessionFile(const QString& sFilename);
 
 	bool startSession();
 	bool checkRestartSession();
@@ -326,6 +362,12 @@ protected:
 
 	void updateContents(qtractorMidiEditor *pMidiEditor, bool bRefresh);
 
+	void trackCurveSelectMenuAction(QMenu *pMenu,
+		qtractorMidiControlObserver *pObserver, qtractorSubject *pCurrentSubject) const;
+
+	bool trackCurveSelectMenuReset(QMenu *pMenu) const;
+	bool trackCurveModeMenuReset(QMenu *pMenu) const;
+
 private:
 
 	// The Qt-designer UI struct...
