@@ -184,7 +184,7 @@ bool qtractorMidiClip::openMidiFile ( qtractorMidiFile *pFile,
 			// SMF format 1 (2 tracks, 1 channel)
 			iTrackChannel = 1;
 			iFormat = 1;
-			iTracks++;
+			++iTracks;
 		}
 		// That's it.
 		setFormat(iFormat);
@@ -204,8 +204,7 @@ bool qtractorMidiClip::openMidiFile ( qtractorMidiFile *pFile,
 		// On read mode, SMF format is properly given by open file.
 		setFormat(pFile->format());
 		// Read the event sequence in...
-		if (!pFile->readTrack(m_pSeq, iTrackChannel))
-			return false;
+		pFile->readTrack(m_pSeq, iTrackChannel);
 		// FIXME: On demand, set session time properties from MIDI file...
 		if (m_bSessionFlag) {
 		#if 0
@@ -324,7 +323,7 @@ QString qtractorMidiClip::createFilePathRevision ( bool bForce )
 	if (m_iRevision == 0 || bForce) {
 		qtractorSession *pSession = qtractorSession::getInstance();
 		if (pSession && track())
-			sFilename = pSession->createFilePath(track()->trackName(), 0, "mid");
+			sFilename = pSession->createFilePath(track()->trackName(), "mid");
 		sFilename = qtractorMidiFile::createFilePathRevision(sFilename);
 	#ifdef CONFIG_DEBUG
 		qDebug("qtractorMidiClip::createFilePathRevision(\"%s\")",
@@ -332,8 +331,7 @@ QString qtractorMidiClip::createFilePathRevision ( bool bForce )
 	#endif
 	}
 
-	if (!bForce)
-		m_iRevision++;
+	if (!bForce) ++m_iRevision;
 
 	return sFilename;
 }
@@ -432,11 +430,10 @@ void qtractorMidiClip::close (void)
 	m_pSeq->close();
 
 	// Commit the final clip length...
-	if (m_pSeq->events().count() < 1)
-		iClipLength = 0;
-	else if (iClipLength < 1)
+	if (iClipLength < 1) {
 		iClipLength = pSession->frameFromTick(m_pSeq->duration());
-	setClipLength(iClipLength);
+		setClipLength(iClipLength);
+	}
 
 	// Now's time to write the whole thing...
 	bool bNewFile = (m_pFile && m_pFile->mode() == qtractorMidiFile::Write);
