@@ -214,7 +214,8 @@ void qtractorMidiEditEvent::setEventType (
 {
 	m_eventType = eventType;
 
-	m_pEditor->updateContents();
+	m_pEditor->selectAll(false);
+//	m_pEditor->updateContents();
 }
 
 qtractorMidiEvent::EventType qtractorMidiEditEvent::eventType (void) const
@@ -227,7 +228,8 @@ void qtractorMidiEditEvent::setController ( unsigned char controller )
 {
 	m_controller = controller;
 
-	m_pEditor->updateContents();
+	m_pEditor->selectAll(false);
+//	m_pEditor->updateContents();
 }
 
 unsigned char qtractorMidiEditEvent::controller (void) const
@@ -434,16 +436,52 @@ void qtractorMidiEditEvent::updatePixmap ( int cx, int /*cy*/ )
 		pEvent = pEvent->next();
 	}
 
-	// Draw loop boundaries, if applicable...
 	qtractorSession *pSession = qtractorSession::getInstance();
-	if (pSession && pSession->isLooping()) {
+	if (pSession == NULL)
+		return;
+
+	// Draw loop boundaries, if applicable...
+	if (pSession->isLooping()) {
+		const QBrush shade(QColor(0, 0, 0, 60));
 		p.setPen(Qt::darkCyan);
 		x = pTimeScale->pixelFromFrame(pSession->loopStart()) - dx;
-		if (x >= 0 && x < w)
+		if (x >= w)
+			p.fillRect(QRect(0, 0, w, h), shade);
+		else
+		if (x >= 0) {
+			p.fillRect(QRect(0, 0, x, h), shade);
 			p.drawLine(x, 0, x, h);
+		}
 		x = pTimeScale->pixelFromFrame(pSession->loopEnd()) - dx;
-		if (x >= 0 && x < w)
+		if (x < 0)
+			p.fillRect(QRect(0, 0, w, h), shade);
+		else
+		if (x < w) {
+			p.fillRect(QRect(x, 0, w - x, h), shade);
 			p.drawLine(x, 0, x, h);
+		}
+	}
+
+	// Draw punch boundaries, if applicable...
+	if (pSession->isPunching()) {
+		const QBrush shade(QColor(0, 0, 0, 60));
+		p.setPen(Qt::darkMagenta);
+		x = pTimeScale->pixelFromFrame(pSession->punchIn()) - dx;
+		if (x >= w)
+			p.fillRect(QRect(0, 0, w, h), shade);
+		else
+		if (x >= 0) {
+			p.fillRect(QRect(0, 0, x, h), shade);
+			p.drawLine(x, 0, x, h);
+		}
+		x = pTimeScale->pixelFromFrame(pSession->punchOut()) - dx;
+		if (x < 0)
+			p.fillRect(QRect(0, 0, w, h), shade);
+		else
+		if (x < w) {
+			p.fillRect(QRect(x, 0, w - x, h), shade);
+			p.drawLine(x, 0, x, h);
+		}
 	}
 }
 
@@ -534,6 +572,20 @@ void qtractorMidiEditEvent::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 }
 
 
+// Handle zoom with mouse wheel.
+void qtractorMidiEditEvent::wheelEvent ( QWheelEvent *pWheelEvent )
+{
+	if (pWheelEvent->modifiers() & Qt::ControlModifier) {
+		int delta = pWheelEvent->delta();
+		if (delta > 0)
+			m_pEditor->zoomIn();
+		else
+			m_pEditor->zoomOut();
+	}
+	else qtractorScrollView::wheelEvent(pWheelEvent);
+}
+
+
 // Trap for help/tool-tip and leave events.
 bool qtractorMidiEditEvent::eventFilter ( QObject *pObject, QEvent *pEvent )
 {
