@@ -1,7 +1,7 @@
 // qtractorMidiEditView.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2010, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2011, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -298,7 +298,7 @@ void qtractorMidiEditView::updatePixmap ( int cx, int cy )
 			p.fillRect(0, y + 1, w, h1, rgbSharp); 
 		p.drawLine(0, y, w, y);
 		y += h1;
-		n--;
+		--n;
 	}
 
 	// Account for the editing offset:
