@@ -85,10 +85,6 @@ qtractorMidiEditorForm::qtractorMidiEditorForm (
 	// Snap-per-beat combo-box.
 	m_pSnapPerBeatComboBox = new QComboBox(m_ui.viewToolbar);
 	m_pSnapPerBeatComboBox->setEditable(false);
-	m_pSnapPerBeatComboBox->insertItems(0, qtractorTimeScale::snapItems());
-	m_pSnapPerBeatComboBox->setToolTip(tr("Snap/beat"));
-	m_ui.viewToolbar->addSeparator();
-	m_ui.viewToolbar->addWidget(m_pSnapPerBeatComboBox);
 
 	// Event type selection widgets...
 	m_pViewTypeComboBox = new QComboBox(m_ui.editViewToolbar);
@@ -99,37 +95,69 @@ qtractorMidiEditorForm::qtractorMidiEditorForm (
 	m_pControllerComboBox->setEditable(false);
 	m_pControllerComboBox->setMinimumWidth(220);
 
+	// Snap-to-scale/quantize selection widgets...
+	m_pSnapToScaleKeyComboBox = new QComboBox(m_ui.snapToScaleToolbar);
+	m_pSnapToScaleKeyComboBox->setEditable(false);
+	m_pSnapToScaleKeyComboBox->setMinimumWidth(86);
+	m_pSnapToScaleTypeComboBox = new QComboBox(m_ui.snapToScaleToolbar);
+	m_pSnapToScaleKeyComboBox->setEditable(false);
+
 	// Pre-fill the combo-boxes...
-	const QIcon iconViewType(":/images/itemProperty.png");
-	m_pViewTypeComboBox->addItem(iconViewType,
+	m_pSnapPerBeatComboBox->insertItems(0, qtractorTimeScale::snapItems());
+
+	const QIcon icon(":/images/itemProperty.png");
+
+	m_pViewTypeComboBox->addItem(icon,
 		tr("Note On"), int(qtractorMidiEvent::NOTEON));
-	m_pViewTypeComboBox->addItem(iconViewType,
+	m_pViewTypeComboBox->addItem(icon,
 		tr("Key Press"), int(qtractorMidiEvent::KEYPRESS));
 
-	const QIcon iconEventType(":/images/itemProperty.png");
-	m_pEventTypeComboBox->addItem(iconEventType,
+	m_pEventTypeComboBox->addItem(icon,
 		tr("Note Velocity"), int(qtractorMidiEvent::NOTEON));
-	m_pEventTypeComboBox->addItem(iconEventType,
+	m_pEventTypeComboBox->addItem(icon,
 		tr("Key Press"), int(qtractorMidiEvent::KEYPRESS));
-	m_pEventTypeComboBox->addItem(iconEventType,
+	m_pEventTypeComboBox->addItem(icon,
 		tr("Controller"), int(qtractorMidiEvent::CONTROLLER));
-	m_pEventTypeComboBox->addItem(iconEventType,
+	m_pEventTypeComboBox->addItem(icon,
 		tr("Pgm Change"), int(qtractorMidiEvent::PGMCHANGE));
-	m_pEventTypeComboBox->addItem(iconEventType,
+	m_pEventTypeComboBox->addItem(icon,
 		tr("Chan Press"), int(qtractorMidiEvent::CHANPRESS));
-	m_pEventTypeComboBox->addItem(iconEventType,
+	m_pEventTypeComboBox->addItem(icon,
 		tr("Pitch Bend"), int(qtractorMidiEvent::PITCHBEND));
-	m_pEventTypeComboBox->addItem(iconEventType,
+	m_pEventTypeComboBox->addItem(icon,
 		tr("Sys Ex"), int(qtractorMidiEvent::SYSEX));
 
+	// Snap-to-scale/quantize selection widgets...
+	QStringListIterator iter(qtractorMidiEditor::scaleKeyNames());
+	while (iter.hasNext())
+		m_pSnapToScaleKeyComboBox->addItem(icon, iter.next());
+	m_pSnapToScaleTypeComboBox->insertItems(0,
+		qtractorMidiEditor::scaleTypeNames());
+
 //	updateInstrumentNames();
 
+	// Set combo-boxes tooltips...
+	m_pSnapPerBeatComboBox->setToolTip(tr("Snap/beat"));
+	m_pViewTypeComboBox->setToolTip(tr("Note type"));
+	m_pEventTypeComboBox->setToolTip(tr("Value type"));
+	m_pControllerComboBox->setToolTip(tr("Controller type"));
+	m_pSnapToScaleKeyComboBox->setToolTip(tr("Scale key"));
+	m_pSnapToScaleTypeComboBox->setToolTip(tr("Scale type"));
+
 	// Add combo-boxes to toolbars...
+	m_ui.viewToolbar->addSeparator();
+	m_ui.viewToolbar->addWidget(m_pSnapPerBeatComboBox);
+
 	m_ui.editViewToolbar->addWidget(m_pViewTypeComboBox);
+
 	m_ui.editEventToolbar->addWidget(m_pEventTypeComboBox);
 	m_ui.editEventToolbar->addSeparator();
 	m_ui.editEventToolbar->addWidget(m_pControllerComboBox);
 
+	m_ui.snapToScaleToolbar->addWidget(m_pSnapToScaleKeyComboBox);
+	m_ui.snapToScaleToolbar->addSeparator();
+	m_ui.snapToScaleToolbar->addWidget(m_pSnapToScaleTypeComboBox);
+
 	QStatusBar *pStatusBar = statusBar();
 
 	const QSize pad(4, 0);
@@ -177,7 +205,6 @@ qtractorMidiEditorForm::qtractorMidiEditorForm (
 	addAction(m_ui.viewMenubarAction);
 
 	// Special integration ones.
-	addAction(m_ui.editSelectRangeAction);
 	addAction(m_ui.transportBackwardAction);
 	addAction(m_ui.transportLoopAction);
 	addAction(m_ui.transportLoopSetAction);
@@ -247,15 +274,18 @@ qtractorMidiEditorForm::qtractorMidiEditorForm (
 	QObject::connect(m_ui.editDeleteAction,
 		SIGNAL(triggered(bool)),
 		SLOT(editDelete()));
+	QObject::connect(m_ui.editSelectAllAction,
+		SIGNAL(triggered(bool)),
+		SLOT(editSelectAll()));
 	QObject::connect(m_ui.editSelectNoneAction,
 		SIGNAL(triggered(bool)),
 		SLOT(editSelectNone()));
 	QObject::connect(m_ui.editSelectInvertAction,
 		SIGNAL(triggered(bool)),
 		SLOT(editSelectInvert()));
-	QObject::connect(m_ui.editSelectAllAction,
+	QObject::connect(m_ui.editSelectRangeAction,
 		SIGNAL(triggered(bool)),
-		SLOT(editSelectAll()));
+		SLOT(editSelectRange()));
 
 	QObject::connect(m_ui.toolsQuantizeAction,
 		SIGNAL(triggered(bool)),
@@ -297,6 +327,9 @@ qtractorMidiEditorForm::qtractorMidiEditorForm (
 	QObject::connect(m_ui.viewToolbarTransportAction,
 		SIGNAL(triggered(bool)),
 		SLOT(viewToolbarTransport(bool)));
+	QObject::connect(m_ui.viewToolbarScaleAction,
+		SIGNAL(triggered(bool)),
+		SLOT(viewToolbarScale(bool)));
 	QObject::connect(m_ui.viewNoteDurationAction,
 		SIGNAL(triggered(bool)),
 		SLOT(viewNoteDuration(bool)));
@@ -359,6 +392,9 @@ qtractorMidiEditorForm::qtractorMidiEditorForm (
 	QObject::connect(m_ui.viewSnapMenu,
 		SIGNAL(aboutToShow()),
 		SLOT(updateSnapMenu()));
+	QObject::connect(m_ui.viewScaleMenu,
+		SIGNAL(aboutToShow()),
+		SLOT(updateScaleMenu()));
 
 	QObject::connect(m_pSnapPerBeatComboBox,
 		SIGNAL(activated(int)),
@@ -375,6 +411,13 @@ qtractorMidiEditorForm::qtractorMidiEditorForm (
 		SIGNAL(activated(int)),
 		SLOT(controllerChanged(int)));
 
+	QObject::connect(m_pSnapToScaleKeyComboBox,
+		SIGNAL(activated(int)),
+		SLOT(snapToScaleKeyChanged(int)));
+	QObject::connect(m_pSnapToScaleTypeComboBox,
+		SIGNAL(activated(int)),
+		SLOT(snapToScaleTypeChanged(int)));
+
 	QObject::connect(m_pMidiEditor,
 		SIGNAL(selectNotifySignal(qtractorMidiEditor *)),
 		SLOT(selectionChanged(qtractorMidiEditor *)));
@@ -396,6 +439,7 @@ qtractorMidiEditorForm::qtractorMidiEditorForm (
 		m_ui.viewToolbarEditAction->setChecked(pOptions->bMidiEditToolbar);
 		m_ui.viewToolbarViewAction->setChecked(pOptions->bMidiViewToolbar);
 		m_ui.viewToolbarTransportAction->setChecked(pOptions->bMidiTransportToolbar);
+		m_ui.viewToolbarScaleAction->setChecked(pOptions->bMidiScaleToolbar);
 		m_ui.viewNoteDurationAction->setChecked(pOptions->bMidiNoteDuration);
 		m_ui.viewNoteColorAction->setChecked(pOptions->bMidiNoteColor);
 		m_ui.viewValueColorAction->setChecked(pOptions->bMidiValueColor);
@@ -415,6 +459,7 @@ qtractorMidiEditorForm::qtractorMidiEditorForm (
 		viewToolbarEdit(pOptions->bMidiEditToolbar);
 		viewToolbarView(pOptions->bMidiViewToolbar);
 		viewToolbarTransport(pOptions->bMidiTransportToolbar);
+		viewToolbarScale(pOptions->bMidiScaleToolbar);
 		m_pMidiEditor->setZoomMode(pOptions->iMidiZoomMode);
 		m_pMidiEditor->setHorizontalZoom(pOptions->iMidiHorizontalZoom);
 		m_pMidiEditor->setVerticalZoom(pOptions->iMidiVerticalZoom);
@@ -427,8 +472,13 @@ qtractorMidiEditorForm::qtractorMidiEditorForm (
 		m_pMidiEditor->setNoteDuration(pOptions->bMidiNoteDuration);
 		m_pMidiEditor->setSendNotes(pOptions->bMidiPreview);
 		m_pMidiEditor->setSyncView(pOptions->bMidiFollow);
+		m_pMidiEditor->setSnapToScaleKey(pOptions->iMidiSnapToScaleKey);
+		m_pMidiEditor->setSnapToScaleType(pOptions->iMidiSnapToScaleType);
 		// Default snap-per-beat setting...
 		m_pSnapPerBeatComboBox->setCurrentIndex(pOptions->iMidiSnapPerBeat);
+		// Default snap-to-scale settings...
+		m_pSnapToScaleKeyComboBox->setCurrentIndex(pOptions->iMidiSnapToScaleKey);
+		m_pSnapToScaleTypeComboBox->setCurrentIndex(pOptions->iMidiSnapToScaleType);
 		// Restore whole dock windows state.
 		QByteArray aDockables = pOptions->settings().value(
 			"/MidiEditor/Layout/DockWindows").toByteArray();
@@ -448,9 +498,6 @@ qtractorMidiEditorForm::qtractorMidiEditorForm (
 	// Make last-but-not-least conections....
 	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
 	if (pMainForm) {
-		QObject::connect(m_ui.editSelectRangeAction,
-			SIGNAL(triggered(bool)),
-			pMainForm, SLOT(editSelectRange()));
 		QObject::connect(m_ui.transportBackwardAction,
 			SIGNAL(triggered(bool)),
 			pMainForm, SLOT(transportBackward()));
@@ -562,6 +609,7 @@ void qtractorMidiEditorForm::closeEvent ( QCloseEvent *pCloseEvent )
 		pOptions->bMidiEditToolbar = m_ui.editToolbar->isVisible();
 		pOptions->bMidiViewToolbar = m_ui.viewToolbar->isVisible();
 		pOptions->bMidiTransportToolbar = m_ui.transportToolbar->isVisible();
+		pOptions->bMidiScaleToolbar = m_ui.snapToScaleToolbar->isVisible();
 		pOptions->iMidiZoomMode = m_pMidiEditor->zoomMode();
 		pOptions->iMidiHorizontalZoom = m_pMidiEditor->horizontalZoom();
 		pOptions->iMidiVerticalZoom = m_pMidiEditor->verticalZoom();
@@ -576,6 +624,9 @@ void qtractorMidiEditorForm::closeEvent ( QCloseEvent *pCloseEvent )
 		pOptions->bMidiFollow  = m_ui.viewFollowAction->isChecked();
 		// Save snap-per-beat setting...
 		pOptions->iMidiSnapPerBeat = m_pSnapPerBeatComboBox->currentIndex();
+		// Save snap-to-scale settings...
+		pOptions->iMidiSnapToScaleKey = m_pSnapToScaleKeyComboBox->currentIndex();
+		pOptions->iMidiSnapToScaleType = m_pSnapToScaleTypeComboBox->currentIndex();
 		// Close floating dock windows...
 		if (m_pMidiEventList->isFloating())
 			m_pMidiEventList->close();
@@ -770,7 +821,7 @@ void qtractorMidiEditorForm::resetDirtyCount (void)
 	// MIDI clip might be dirty already.
 	qtractorMidiClip *pMidiClip = midiClip();
 	if (pMidiClip && pMidiClip->isDirty())
-		m_iDirtyCount++;
+		++m_iDirtyCount;
 }
 
 
@@ -1062,6 +1113,13 @@ void qtractorMidiEditorForm::editSelectAll (void)
 }
 
 
+// Select contents range.
+void qtractorMidiEditorForm::editSelectRange (void)
+{
+	m_pMidiEditor->selectRange(true, true);
+}
+
+
 // Quantize tool.
 void qtractorMidiEditorForm::toolsQuantize (void)
 {
@@ -1198,6 +1256,20 @@ void qtractorMidiEditorForm::viewToolbarTransport ( bool bOn )
 }
 
 
+// Show/hide the scale-toolbar.
+void qtractorMidiEditorForm::viewToolbarScale ( bool bOn )
+{
+#if 0
+	if (bOn)
+		m_ui.snapToScaleToolbar->show();
+	else
+		m_ui.snapToScaleToolbar->hide();
+#else
+	m_ui.snapToScaleToolbar->setVisible(bOn);
+#endif
+}
+
+
 // View note (pitch) coloring.
 void qtractorMidiEditorForm::viewNoteColor ( bool bOn )
 {
@@ -1317,6 +1389,36 @@ void qtractorMidiEditorForm::viewSnap (void)
 }
 
 
+// Change snap-to-scale key setting via menu.
+void qtractorMidiEditorForm::viewScaleKey (void)
+{
+	// Retrieve snap-to-scale key index from from action data...
+	QAction *pAction = qobject_cast<QAction *> (sender());
+	if (pAction) {
+		// Commit the change as usual...
+		snapToScaleKeyChanged(pAction->data().toInt());
+		// Update the other toolbar control...
+		m_pSnapToScaleKeyComboBox->setCurrentIndex(
+			m_pMidiEditor->snapToScaleKey());
+	}
+}
+
+
+// Change snap-to-scale typesetting via menu.
+void qtractorMidiEditorForm::viewScaleType (void)
+{
+	// Retrieve snap-to-scale type index from from action data...
+	QAction *pAction = qobject_cast<QAction *> (sender());
+	if (pAction) {
+		// Commit the change as usual...
+		snapToScaleTypeChanged(pAction->data().toInt());
+		// Update the other toolbar control...
+		m_pSnapToScaleTypeComboBox->setCurrentIndex(
+			m_pMidiEditor->snapToScaleType());
+	}
+}
+
+
 // Refresh view display.
 void qtractorMidiEditorForm::viewRefresh (void)
 {
@@ -1439,7 +1541,7 @@ void qtractorMidiEditorForm::stabilizeForm (void)
 	int k = 0;
 	if (format() == 0) {
 		sTrackChannel = tr("Channel %1");
-		k++;
+		++k;
 	} else {
 		sTrackChannel = tr("Track %1");
 	}
@@ -1568,6 +1670,37 @@ void qtractorMidiEditorForm::updateSnapMenu (void)
 }
 
 
+// Snap-to-scale view menu builder.
+void qtractorMidiEditorForm::updateScaleMenu (void)
+{
+	m_ui.viewScaleMenu->clear();
+
+	int iSnapToScaleKey = m_pMidiEditor->snapToScaleKey();
+	int iScaleKey = 0;
+	QStringListIterator iter_key(qtractorMidiEditor::scaleKeyNames());
+	while (iter_key.hasNext()) {
+		QAction *pAction = m_ui.viewScaleMenu->addAction(
+			iter_key.next(), this, SLOT(viewScaleKey()));
+		pAction->setCheckable(true);
+		pAction->setChecked(iScaleKey == iSnapToScaleKey);
+		pAction->setData(iScaleKey++);
+	}
+
+	m_ui.viewScaleMenu->addSeparator();
+
+	int iSnapToScaleType = m_pMidiEditor->snapToScaleType();
+	int iScaleType = 0;
+	QStringListIterator iter_type(qtractorMidiEditor::scaleTypeNames());
+	while (iter_type.hasNext()) {
+		QAction *pAction = m_ui.viewScaleMenu->addAction(
+			iter_type.next(), this, SLOT(viewScaleType()));
+		pAction->setCheckable(true);
+		pAction->setChecked(iScaleType == iSnapToScaleType);
+		pAction->setData(iScaleType++);
+	}
+}
+
+
 // Snap-per-beat spin-box change slot.
 void qtractorMidiEditorForm::snapPerBeatChanged ( int iSnap )
 {
@@ -1591,6 +1724,19 @@ void qtractorMidiEditorForm::snapPerBeatChanged ( int iSnap )
 }
 
 
+// Snap-to-scale/quantize combo-box change slots.
+void qtractorMidiEditorForm::snapToScaleKeyChanged ( int iSnapToScaleKey )
+{
+	m_pMidiEditor->setSnapToScaleKey(iSnapToScaleKey);
+}
+
+
+void qtractorMidiEditorForm::snapToScaleTypeChanged ( int iSnapToScaleType )
+{
+	m_pMidiEditor->setSnapToScaleType(iSnapToScaleType);
+}
+
+
 // Tool selection handlers.
 void qtractorMidiEditorForm::viewTypeChanged ( int iIndex )
 {
@@ -1645,7 +1791,7 @@ void qtractorMidiEditorForm::selectionChanged ( qtractorMidiEditor *pMidiEditor
 
 void qtractorMidiEditorForm::contentsChanged ( qtractorMidiEditor *pMidiEditor )
 {
-	m_iDirtyCount++;
+	++m_iDirtyCount;
 
 	selectionChanged(pMidiEditor);
 }
