@@ -113,9 +113,10 @@ protected slots:
 	void editPasteRepeat();
 	void editDelete();
 
-	void editSelectNone();
 	void editSelectAll();
+	void editSelectNone();
 	void editSelectInvert();
+	void editSelectRange();
 
 	void toolsQuantize();
 	void toolsTranspose();
@@ -131,6 +132,7 @@ protected slots:
 	void viewToolbarEdit(bool bOn);
 	void viewToolbarView(bool bOn);
 	void viewToolbarTransport(bool bOn);
+	void viewToolbarScale(bool bOn);
 	void viewNoteDuration(bool bOn);
 	void viewNoteColor(bool bOn);
 	void viewValueColor(bool bOn);
@@ -144,6 +146,8 @@ protected slots:
 	void viewZoomVertical();
 	void viewZoomAll();
 	void viewSnap();
+	void viewScaleKey();
+	void viewScaleType();
 	void viewSnapGrid(bool bOn);
 	void viewToolTips(bool bOn);
 	void viewRefresh();
@@ -154,6 +158,7 @@ protected slots:
 
 	void updateZoomMenu();
 	void updateSnapMenu();
+	void updateScaleMenu();
 
 	void sendNote(int iNote, int iVelocity);
 
@@ -165,7 +170,10 @@ protected slots:
 	void eventTypeChanged(int);
 	void controllerChanged(int);
 
-	void snapPerBeatChanged(int iSnap);
+	void snapToScaleKeyChanged(int iSnapToScaleKey);
+	void snapToScaleTypeChanged(int iSnapToScaleType);
+
+	void snapPerBeatChanged(int iSnapPerBeat);
 
 protected:
 
@@ -203,6 +211,10 @@ private:
 	QComboBox *m_pEventTypeComboBox;
 	QComboBox *m_pControllerComboBox;
 
+	// Snap-to-scale/quantize selection widgets...
+	QComboBox *m_pSnapToScaleKeyComboBox;
+	QComboBox *m_pSnapToScaleTypeComboBox;
+
 	// Status items.
 	QLabel *m_pTrackNameLabel;
 	QLabel *m_pFileNameLabel;
