@@ -40,6 +40,7 @@ class qtractorMidiOutputThread;
 class qtractorMidiMonitor;
 class qtractorMidiSysexList;
 class qtractorPluginList;
+class qtractorCurveList;
 
 class QSocketNotifier;
 
@@ -214,10 +215,8 @@ public:
 	unsigned short captureQuantize() const;
 
 	// ALSA device queue timer.
-	void setAlsaTimer(int iAlsaTimer)
-		{ m_iAlsaTimer = iAlsaTimer; }
-	int alsaTimer() const
-		{ return m_iAlsaTimer; }
+	void setAlsaTimer(int iAlsaTimer);
+	int alsaTimer() const;
 
 	// MMC device-id accessors.
 	void setMmcDevice(unsigned char mmcDevice);
@@ -241,7 +240,7 @@ public:
 protected:
 
 	// Concrete device (de)activation methods.
-	bool init(void);
+	bool init();
 	bool activate();
 	bool start();
 	void stop();
@@ -286,7 +285,9 @@ private:
 	// The delta-time when playback started .
 	long m_iTimeStart;
 	long m_iTimeDrift;
-	
+
+	long m_iFrameStart;
+
 	// The assigned control buses.
 	bool             m_bControlBus;
 	qtractorMidiBus *m_pIControlBus;
@@ -426,6 +427,14 @@ public:
 	qtractorPluginList *pluginList_in()  const;
 	qtractorPluginList *pluginList_out() const;
 
+	// Automation curve list accessors.
+	qtractorCurveList *curveList_in()  const;
+	qtractorCurveList *curveList_out() const;
+
+	// Automation curve serializer accessors.
+	qtractorCurveFile *curveFile_in()  const;
+	qtractorCurveFile *curveFile_out() const;
+
 	// Retrieve/restore client:port connections.
 	// return the effective number of connection attempts.
 	int updateConnects(BusMode busMode,
@@ -485,10 +494,14 @@ private:
 	qtractorPluginList *m_pIPluginList;
 	qtractorPluginList *m_pOPluginList;
 
+	// Automation curve serializer instances.
+	qtractorCurveFile  *m_pICurveFile;
+	qtractorCurveFile  *m_pOCurveFile;
+
 	// SysEx setup list.
 	qtractorMidiSysexList *m_pSysexList;
 
-	// Deafult instrument name.
+	// Default instrument name.
 	QString m_sInstrumentName;
 
 	// Channel patch mapper.
