@@ -1,7 +1,7 @@
 // qtractorMidiMonitor.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2011, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -76,7 +76,7 @@ void qtractorMidiMonitor::enqueue ( qtractorMidiEvent::EventType type,
 		if (item.value < val && type == qtractorMidiEvent::NOTEON)
 			item.value = val;
 		// Increment enqueued count.
-		item.count++;
+		++(item.count);
 		// Done enqueueing.
 	} else {
 		// Alternative is sending it directly
@@ -84,7 +84,7 @@ void qtractorMidiMonitor::enqueue ( qtractorMidiEvent::EventType type,
 		if (m_item.value < val && type == qtractorMidiEvent::NOTEON)
 			m_item.value = val;
 		// Increment direct count.
-		m_item.count++;
+		++(m_item.count);
 		// Done direct.
 	}
 }
