@@ -1,7 +1,7 @@
 // qtractorMidiToolsForm.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2010, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2011, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -64,6 +64,9 @@ protected slots:
     void presetSave();
     void presetDelete();
 
+	void timeshiftSpinBoxChanged(double p);
+	void timeshiftSliderChanged(int i);
+
 	void formatChanged(int);
 	void changed();
 	void accept();
@@ -89,6 +92,8 @@ private:
 
 	int m_iDirtyCount;
 	int m_iUpdate;
+
+	class TimeshiftCurve *m_pTimeshiftCurve;
 };
 
 
