@@ -40,6 +40,9 @@
 #include "qtractorAudioEngine.h"
 #include "qtractorMidiEngine.h"
 
+#include "qtractorMidiControlObserver.h"
+
+#include "qtractorCurve.h"
 
 #include "qtractorMainForm.h"
 #include "qtractorBusForm.h"
@@ -144,9 +147,9 @@ qtractorBus *qtractorMonitorButton::bus (void) const
 // Visitors overload.
 void qtractorMonitorButton::updateValue ( float fValue )
 {
-	m_iUpdate++;
+	++m_iUpdate;
 	QPushButton::setChecked(fValue > 0.0f);
-	m_iUpdate--;
+	--m_iUpdate;
 }
 
 
@@ -171,7 +174,12 @@ void qtractorMonitorButton::updateMonitor (void)
 {
 	if (m_pTrack) {
 		setSubject(m_pTrack->monitorSubject());
-		addMidiControlAction(m_pTrack->monitorObserver());
+		qtractorMidiControlObserver *pMidiObserver
+			= m_pTrack->monitorObserver();
+		if (pMidiObserver) {
+			pMidiObserver->setCurveList(m_pTrack->curveList());
+			addMidiControlAction(pMidiObserver);
+		}
 	}
 	else
 	if (m_pBus) {
@@ -299,7 +307,7 @@ void qtractorMixerStrip::initMixerStrip (void)
 
 	m_pPluginListView = new qtractorPluginListView(/*this*/);
 	m_pPluginListView->setFont(font6);
-	m_pPluginListView->setFixedHeight(fm.lineSpacing() << 2);
+	m_pPluginListView->setFixedHeight((fm.lineSpacing() + 2) << 2);
 	m_pPluginListView->setTinyScrollBar(true);
 	m_pLayout->addWidget(m_pPluginListView);
 
@@ -357,7 +365,7 @@ void qtractorMixerStrip::initMixerStrip (void)
 
 	m_pLayout->addWidget(m_pMonitorButton);
 	m_pLayout->addLayout(m_pButtonLayout);
-
+	
 	// Now, there's whether we are Audio or MIDI related...
 	m_pMeter = NULL;
 	m_pMidiLabel = NULL;
@@ -453,12 +461,29 @@ void qtractorMixerStrip::initMixerStrip (void)
 
 	// Eventually the right one...
 	if (m_pMeter) {
+		// Set MIDI controller & automation hooks...
+		qtractorMidiControlObserver *pMidiObserver;
+		pMidiObserver = m_pMeter->panningObserver();
+		if (m_pTrack)
+			pMidiObserver->setCurveList(m_pTrack->curveList());
+		m_pMeter->addMidiControlAction(m_pMeter->panSlider(), pMidiObserver);
+		pMidiObserver = m_pMeter->gainObserver();
+		if (m_pTrack)
+			pMidiObserver->setCurveList(m_pTrack->curveList());
+		m_pMeter->addMidiControlAction(m_pMeter->gainSlider(), pMidiObserver);
+		// Finally, add to layout...	
 		m_pLayout->addWidget(m_pMeter);
-		QObject::connect(m_pMeter,
-			SIGNAL(panningChangedSignal(float)),
+		QObject::connect(m_pMeter->panSlider(),
+			SIGNAL(valueChanged(float)),
 			SLOT(panningChangedSlot(float)));
-		QObject::connect(m_pMeter,
-			SIGNAL(gainChangedSignal(float)),
+		QObject::connect(m_pMeter->panSpinBox(),
+			SIGNAL(valueChanged(float)),
+			SLOT(panningChangedSlot(float)));
+		QObject::connect(m_pMeter->gainSlider(),
+			SIGNAL(valueChanged(float)),
+			SLOT(gainChangedSlot(float)));
+		QObject::connect(m_pMeter->gainSpinBox(),
+			SIGNAL(valueChanged(float)),
 			SLOT(gainChangedSlot(float)));
 	}
 
@@ -535,12 +560,14 @@ void qtractorMixerStrip::updateName (void)
 
 	m_pLabel->setText(sName);
 
-	qtractorMonitor *pMonitor = m_pMeter->monitor();
-	if (pMonitor) {
-		pMonitor->panningSubject()->setName(sName + ' ' + tr("Pan"));
-		pMonitor->gainSubject()->setName(sName + ' ' + sGain);
+	if (m_pMeter) {
+		qtractorMonitor *pMonitor = m_pMeter->monitor();
+		if (pMonitor) {
+			pMonitor->panningSubject()->setName(sName + ' ' + tr("Pan"));
+			pMonitor->gainSubject()->setName(sName + ' ' + sGain);
+		}
 	}
-
+	
 	QFrame::setToolTip(sName + ' ' + sType);
 }
 
@@ -1367,6 +1394,33 @@ void qtractorMixer::updateBusStrip ( qtractorMixerRack *pRack,
 	}
 
 	pBus->mapControllers(busMode);
+#if 0
+	qtractorAudioBus *pAudioBus;
+	qtractorMidiBus  *pMidiBus;
+
+	switch (pBus->busType()) {
+	case qtractorTrack::Audio:
+		pAudioBus = static_cast<qtractorAudioBus *> (pBus);
+		if (pAudioBus) {
+			pAudioBus->applyCurveFile(busMode,
+				(busMode == qtractorBus::Input
+			       ? pAudioBus->curveFile_in()
+			       : pAudioBus->curveFile_out()));
+		}
+		break;
+	case qtractorTrack::Midi:
+		pMidiBus = static_cast<qtractorMidiBus *> (pBus);
+		if (pMidiBus) {
+			pMidiBus->applyCurveFile(busMode,
+				(busMode == qtractorBus::Input
+			         ? pMidiBus->curveFile_in()
+			         : pMidiBus->curveFile_out()));
+		}
+		break;
+	default:
+		break;
+	}
+#endif
 }
 
 
@@ -1382,16 +1436,31 @@ void qtractorMixer::updateTrackStrip ( qtractorTrack *pTrack, bool bReset )
 	}
 
 	pTrack->mapControllers();
+	pTrack->applyCurveFile(pTrack->curveFile());
+
+	qtractorCurveList *pCurveList = pTrack->curveList();
+	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+	if (pCurveList && pMainForm && pMainForm->tracks()) {
+		pMainForm->tracks()->updateTrackList();
+		QObject::connect(
+			pCurveList->proxy(), SIGNAL(update()),
+			pMainForm->tracks(), SLOT(updateTrackView()));
+	}
 }
 
 
 // Update buses'racks.
-void qtractorMixer::updateBuses (void)
+void qtractorMixer::updateBuses ( bool bReset )
 {
 	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return;
 
+	if (bReset) {
+		m_pInputRack->clear();
+		m_pOutputRack->clear();		
+	}
+
 	m_pInputRack->markStrips(1);
 	m_pOutputRack->markStrips(1);
 
@@ -1415,16 +1484,40 @@ void qtractorMixer::updateBuses (void)
 
 	m_pOutputRack->cleanStrips(1);
 	m_pInputRack->cleanStrips(1);
+
+	// Autonomic resizing of bus splitter sizes...
+	int ws, iUpdate = 0;
+	QList<int> sizes = m_pSplitter->sizes();
+	int& w0 = sizes[0];
+	int& w1 = sizes[1];
+	int& w2 = sizes[2];
+	ws = m_pInputRack->workspace()->width() + 4;
+	if (w0 > ws) {
+		w1 += (w0 - ws);
+		w0  = ws;
+		++iUpdate;
+	}
+	ws = m_pOutputRack->workspace()->width() + 4;
+	if (w2 > ws) {
+		w1 += (w2 - ws);
+		w2  = ws;
+		++iUpdate;
+	}
+	if (iUpdate > 0)
+		m_pSplitter->setSizes(sizes);
 }
 
 
 // Update tracks'rack.
-void qtractorMixer::updateTracks (void)
+void qtractorMixer::updateTracks ( bool bReset )
 {
 	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return;
 
+	if (bReset)
+		m_pTrackRack->clear();
+
 	m_pTrackRack->markStrips(1);
 
 	for (qtractorTrack *pTrack = pSession->tracks().first();
