@@ -1,7 +1,7 @@
 // qtractorMmcEvent.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2010, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2011, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -104,7 +104,7 @@ int qtractorMmcEvent::track (void) const
 			int iMask = (1 << i);
 			if (data[2] & iMask)
 				break;
-			iTrack++;
+			++iTrack;
 		}
 	}
 
