@@ -253,6 +253,7 @@ void qtractorOptions::loadOptions (void)
 	bTrackViewDropSpan   = m_settings.value("/TrackViewDropSpan", false).toBool();
 	bTrackViewSnapGrid   = m_settings.value("/TrackViewSnapGrid", true).toBool();
 	bTrackViewToolTips   = m_settings.value("/TrackViewToolTips", true).toBool();
+	bTrackViewCurveEdit  = m_settings.value("/TrackViewCurveEdit", false).toBool();
 	m_settings.endGroup();
 
 	// MIDI options group.
@@ -265,6 +266,7 @@ void qtractorOptions::loadOptions (void)
 	bMidiEditToolbar = m_settings.value("/EditToolbar", true).toBool();
 	bMidiViewToolbar = m_settings.value("/ViewToolbar", true).toBool();
 	bMidiTransportToolbar = m_settings.value("/TransportToolbar", false).toBool();
+	bMidiScaleToolbar = m_settings.value("/ScaleToolbar", false).toBool();
 	bMidiNoteDuration = m_settings.value("/NoteDuration", true).toBool();
 	bMidiNoteColor   = m_settings.value("/NoteColor", false).toBool();
 	bMidiValueColor  = m_settings.value("/ValueColor", false).toBool();
@@ -278,6 +280,8 @@ void qtractorOptions::loadOptions (void)
 	iMidiSnapPerBeat = m_settings.value("/SnapPerBeat", 4).toInt();
 	bMidiSnapGrid    = m_settings.value("/SnapGrid", false).toBool();
 	bMidiToolTips    = m_settings.value("/ToolTips", true).toBool();
+	iMidiSnapToScaleKey = m_settings.value("/SnapToScaleKey", 0).toInt();
+	iMidiSnapToScaleType = m_settings.value("/SnapToScaleType", 0).toInt();
 	m_settings.endGroup();
 
 	// Meter colors.
@@ -464,6 +468,7 @@ void qtractorOptions::saveOptions (void)
 	m_settings.setValue("/TrackViewDropSpan", bTrackViewDropSpan);
 	m_settings.setValue("/TrackViewSnapGrid", bTrackViewSnapGrid);
 	m_settings.setValue("/TrackViewToolTips", bTrackViewToolTips);
+	m_settings.setValue("/TrackViewCurveEdit", bTrackViewCurveEdit);
 	m_settings.endGroup();
 
 	// MIDI Editor options group.
@@ -476,6 +481,7 @@ void qtractorOptions::saveOptions (void)
 	m_settings.setValue("/EditToolbar", bMidiEditToolbar);
 	m_settings.setValue("/ViewToolbar", bMidiViewToolbar);
 	m_settings.setValue("/TransportToolbar", bMidiTransportToolbar);
+	m_settings.setValue("/ScaleToolbar", bMidiScaleToolbar);
 	m_settings.setValue("/NoteDuration", bMidiNoteDuration);
 	m_settings.setValue("/NoteColor", bMidiNoteColor);
 	m_settings.setValue("/ValueColor", bMidiValueColor);
@@ -489,6 +495,8 @@ void qtractorOptions::saveOptions (void)
 	m_settings.setValue("/SnapPerBeat", iMidiSnapPerBeat);
 	m_settings.setValue("/SnapGrid", bMidiSnapGrid);
 	m_settings.setValue("/ToolTips", bMidiToolTips);
+	m_settings.setValue("/SnapToScaleKey", iMidiSnapToScaleKey);
+	m_settings.setValue("/SnapToScaleType", iMidiSnapToScaleType);
 	m_settings.endGroup();
 
 	// Meter colors.
@@ -553,7 +561,7 @@ bool qtractorOptions::parse_args ( const QStringList& args )
 		if (iCmdArgs > 0) {
 			sSessionFile += ' ';
 			sSessionFile += args.at(i);
-			iCmdArgs++;
+			++iCmdArgs;
 			continue;
 		}
 
@@ -575,7 +583,7 @@ bool qtractorOptions::parse_args ( const QStringList& args )
 			}
 			sSessionId = sVal;
 			if (iEqual < 0)
-				i++;
+				++i;
 		}
 		else
 	#endif
@@ -592,7 +600,7 @@ bool qtractorOptions::parse_args ( const QStringList& args )
 			// If we don't have one by now,
 			// this will be the startup session file...
 			sSessionFile += sArg;
-			iCmdArgs++;
+			++iCmdArgs;
 		}
 	}
 
@@ -663,7 +671,7 @@ void qtractorOptions::loadComboBoxHistory ( QComboBox *pComboBox, int iLimit )
 		pComboBox->setUpdatesEnabled(false);
 		pComboBox->setDuplicatesEnabled(false);
 		pComboBox->clear();
-		for (int i = 0; i < iLimit; i++) {
+		for (int i = 0; i < iLimit; ++i) {
 			const QString& sText = m_settings.value(
 				"/Item" + QString::number(i + 1)).toString();
 			if (sText.isEmpty())
@@ -687,10 +695,10 @@ void qtractorOptions::saveComboBoxHistory ( QComboBox *pComboBox, int iLimit )
 		int i = pComboBox->findText(sCurrentText);
 		if (i >= 0) {
 			pComboBox->removeItem(i);
-			iCount--;
+			--iCount;
 		}
 		pComboBox->insertItem(0, sCurrentText);
-		iCount++;
+		++iCount;
 	}
 
 	// Take care of item limit...
@@ -699,7 +707,7 @@ void qtractorOptions::saveComboBoxHistory ( QComboBox *pComboBox, int iLimit )
 
 	// Save combobox list to configuration settings file...
 	m_settings.beginGroup("/History/" + pComboBox->objectName());
-	for (int i = 0; i < iCount; i++) {
+	for (int i = 0; i < iCount; ++i) {
 		const QString& sText = pComboBox->itemText(i);
 		if (sText.isEmpty())
 			continue;
