@@ -192,6 +192,7 @@ public:
 	bool bTrackViewDropSpan;
 	bool bTrackViewSnapGrid;
 	bool bTrackViewToolTips;
+	bool bTrackViewCurveEdit;
 
 	// MIDI Editor options...
 	bool bMidiMenubar;
@@ -200,6 +201,7 @@ public:
 	bool bMidiEditToolbar;
 	bool bMidiViewToolbar;
 	bool bMidiTransportToolbar;
+	bool bMidiScaleToolbar;
 	bool bMidiNoteDuration;
 	bool bMidiNoteColor;
 	bool bMidiValueColor;
@@ -213,6 +215,8 @@ public:
 	int  iMidiSnapPerBeat;
 	bool bMidiSnapGrid;
 	bool bMidiToolTips;
+	int  iMidiSnapToScaleKey;
+	int  iMidiSnapToScaleType;
 
 	// Meter colors.
 	QStringList audioMeterColors;
