@@ -32,6 +32,7 @@
 
 #include "qtractorSession.h"
 #include "qtractorDocument.h"
+#include "qtractorCurveFile.h"
 
 #ifdef CONFIG_LADSPA
 #include "qtractorLadspaPlugin.h"
@@ -252,7 +253,7 @@ bool qtractorPluginFile::getTypes ( qtractorPluginPath& path,
 			if (pType->open()) {
 				path.addType(pType);
 				pType->close();
-				iIndex++;
+				++iIndex;
 			} else {
 				delete pType;
 				break;
@@ -273,7 +274,7 @@ bool qtractorPluginFile::getTypes ( qtractorPluginPath& path,
 			if (pType->open()) {
 				path.addType(pType);
 				pType->close();
-				iIndex++;
+				++iIndex;
 			} else {
 				delete pType;
 				break;
@@ -299,7 +300,7 @@ bool qtractorPluginFile::getTypes ( qtractorPluginPath& path,
 			if (pType->open()) {
 				path.addType(pType);
 				pType->close();
-				iIndex++;
+				++iIndex;
 			} else {
 				delete pType;
 			}
@@ -820,7 +821,7 @@ bool qtractorPlugin::loadPreset ( const QString& sFilename )
 		// Check for preset item...
 		if (eChild.tagName() == "configs" || eChild.tagName() == "configure") {
 			// Parse for config entries...
-			qtractorPlugin::loadConfigs(&eChild, m_configs);
+			qtractorPlugin::loadConfigs(&eChild, m_configs, m_ctypes);
 		}
 		else
 		if (eChild.tagName() == "params") {
@@ -947,7 +948,8 @@ void qtractorPlugin::realizeValues (void)
 
 
 // Load plugin configuration stuff (CLOB).
-void qtractorPlugin::loadConfigs ( QDomElement *pElement, Configs& configs )
+void qtractorPlugin::loadConfigs (
+	QDomElement *pElement, Configs& configs, ConfigTypes& ctypes )
 {
 	for (QDomNode nConfig = pElement->firstChild();
 			!nConfig.isNull();
@@ -956,8 +958,15 @@ void qtractorPlugin::loadConfigs ( QDomElement *pElement, Configs& configs )
 		QDomElement eConfig = nConfig.toElement();
 		if (eConfig.isNull())
 			continue;
-		if (eConfig.tagName() == "config")
-			configs[eConfig.attribute("key")] = eConfig.text();
+		if (eConfig.tagName() == "config") {
+			const QString& sKey = eConfig.attribute("key");
+			if (!sKey.isEmpty()) {
+				configs[sKey] = eConfig.text();
+				const QString& sType = eConfig.attribute("type");
+				if (!sType.isEmpty())
+					ctypes[sKey] = sType;
+			}
+		}
 	}
 }
 
@@ -989,6 +998,9 @@ void qtractorPlugin::saveConfigs (
 	for (; iter != m_configs.constEnd(); ++iter) {
 		QDomElement eConfig = pDocument->createElement("config");
 		eConfig.setAttribute("key", iter.key());
+		ConfigTypes::ConstIterator ctype = m_ctypes.find(iter.key());
+		if (ctype != m_ctypes.constEnd())
+			eConfig.setAttribute("type", ctype.value());
 		eConfig.appendChild(
 			pDocument->createTextNode(iter.value()));
 		pElement->appendChild(eConfig);
@@ -1030,6 +1042,8 @@ qtractorPluginList::qtractorPluginList ( unsigned short iChannels,
 	m_pppBuffers[0] = NULL;
 	m_pppBuffers[1] = NULL;
 
+	m_pCurveList = new qtractorCurveList();
+
 	setBuffer(iChannels, iBufferSize, iSampleRate, iFlags);
 }
 
@@ -1041,6 +1055,8 @@ qtractorPluginList::~qtractorPluginList (void)
 
 	// Clear out all dependables...
 	m_views.clear();
+
+	delete m_pCurveList;
 }
 
 
@@ -1068,7 +1084,7 @@ void qtractorPluginList::setBuffer ( unsigned short iChannels,
 
 	// Delete old interim buffer...
 	if (m_pppBuffers[1]) {
-		for (i = 0; i < m_iChannels; i++)
+		for (i = 0; i < m_iChannels; ++i)
 			delete [] m_pppBuffers[1][i];
 		delete [] m_pppBuffers[1];
 		m_pppBuffers[1] = NULL;
@@ -1076,6 +1092,7 @@ void qtractorPluginList::setBuffer ( unsigned short iChannels,
 
 	// Destroy any MIDI manager still there...
 	if (m_pMidiManager) {
+		m_bAudioOutputBus = m_pMidiManager->isAudioOutputBus();
 		qtractorMidiManager::deleteMidiManager(m_pMidiManager);
 		m_pMidiManager = NULL;
 	}
@@ -1095,7 +1112,7 @@ void qtractorPluginList::setBuffer ( unsigned short iChannels,
 	// Allocate new interim buffer...
 	if (m_iChannels > 0 && m_iBufferSize > 0) {
 		m_pppBuffers[1] = new float * [m_iChannels];
-		for (i = 0; i < m_iChannels; i++) {
+		for (i = 0; i < m_iChannels; ++i) {
 			m_pppBuffers[1][i] = new float [m_iBufferSize];
 			::memset(m_pppBuffers[1][i], 0, m_iBufferSize * sizeof(float));
 		}
@@ -1145,7 +1162,7 @@ void qtractorPluginList::resetBuffer (void)
 #endif
 	// Reset interim buffer, if any...
 	if (m_pppBuffers[1]) {
-		for (unsigned short i = 0; i < m_iChannels; i++)
+		for (unsigned short i = 0; i < m_iChannels; ++i)
 			::memset(m_pppBuffers[1][i], 0, m_iBufferSize * sizeof(float));
 	}
 #if 0
@@ -1172,9 +1189,9 @@ bool qtractorPluginList::isActivatedAll (void) const
 void qtractorPluginList::updateActivated ( bool bActivated )
 {
 	if (bActivated) {
-		m_iActivated++;
+		++m_iActivated;
 	} else  {
-		m_iActivated--;
+		--m_iActivated;
 	}
 }
 
@@ -1199,9 +1216,6 @@ void qtractorPluginList::insertPlugin ( qtractorPlugin *pPlugin,
 	else
 		append(pPlugin);
 
-	if (pPlugin->isActivated())
-		updateActivated(true);
-
 	// Now update each observer list-view...
 	QListIterator<qtractorPluginListView *> iter(m_views);
 	while (iter.hasNext()) {
@@ -1306,6 +1320,7 @@ qtractorPlugin *qtractorPluginList::copyPlugin ( qtractorPlugin *pPlugin )
 	if (pNewPlugin) {
 		pNewPlugin->setPreset(pPlugin->preset());
 		pNewPlugin->setConfigs(pPlugin->configs());
+		pNewPlugin->setConfigTypes(pPlugin->configTypes());
 		pNewPlugin->setValues(pPlugin->values());
 		pNewPlugin->realizeConfigs();
 		pNewPlugin->realizeValues();
@@ -1408,8 +1423,10 @@ bool qtractorPluginList::loadElement (
 			QStringList vlist;
 			bool bActivated = false;
 			qtractorPlugin::Configs configs;
+			qtractorPlugin::ConfigTypes ctypes;
 			qtractorPlugin::Values values;
 			qtractorMidiControl::Controllers controllers;
+			qtractorCurveFile cfile(qtractorPluginList::curveList());
 			qtractorPluginType::Hint typeHint
 				= qtractorPluginType::hintFromText(
 					ePlugin.attribute("type"));
@@ -1438,9 +1455,9 @@ bool qtractorPluginList::loadElement (
 				if (eParam.tagName() == "activated")
 					bActivated = qtractorDocument::boolFromText(eParam.text());
 				else
-				if (eParam.tagName() == "configs" || eParam.tagName() == "configure") {
+				if (eParam.tagName() == "configs") {
 					// Load plugin configuration stuff (CLOB)...
-					qtractorPlugin::loadConfigs(&eParam, configs);
+					qtractorPlugin::loadConfigs(&eParam, configs, ctypes);
 				}
 				else
 				if (eParam.tagName() == "params") {
@@ -1452,6 +1469,11 @@ bool qtractorPluginList::loadElement (
 					// Load plugin parameter controllers...
 					qtractorPlugin::loadControllers(&eParam, controllers);
 				}
+				else
+				if (eParam.tagName() == "curve-file") {
+					// Load plugin automation curves...
+					qtractorPlugin::loadCurveFile(&eParam, &cfile);
+				}
 			}
 			qtractorPlugin *pPlugin
 				= qtractorPluginFile::createPlugin(this,
@@ -1467,12 +1489,14 @@ bool qtractorPluginList::loadElement (
 			if (pPlugin) {
 				pPlugin->setPreset(sPreset);
 				pPlugin->setConfigs(configs);
+				pPlugin->setConfigTypes(ctypes);
 				if (!vlist.isEmpty())
 					pPlugin->setValueList(vlist);
 				if (!values.isEmpty())
 					pPlugin->setValues(values);
 				append(pPlugin);
 				pPlugin->mapControllers(controllers);
+				pPlugin->applyCurveFile(&cfile);
 				pPlugin->setActivated(bActivated); // Later's better!
 			}
 			// Cleanup.
@@ -1507,7 +1531,7 @@ bool qtractorPluginList::saveElement ( qtractorDocument *pDocument,
 			QString::number(m_pMidiManager->currentProg()), pElement);
 
 	// Save plugins...
-	for (qtractorPlugin *pPlugin = first();
+	for (qtractorPlugin *pPlugin = qtractorPluginList::first();
 			pPlugin; pPlugin = pPlugin->next()) {
 
 		// Do freeze plugin state...
@@ -1542,10 +1566,21 @@ bool qtractorPluginList::saveElement ( qtractorDocument *pDocument,
 		QDomElement eParams = pDocument->document()->createElement("params");
 		pPlugin->saveValues(pDocument->document(), &eParams);
 		ePlugin.appendChild(eParams);
+		// Plugin paramneter controllers...
 		QDomElement eControllers
 			= pDocument->document()->createElement("controllers");
 		pPlugin->saveControllers(pDocument, &eControllers);
 		ePlugin.appendChild(eControllers);
+		// Save plugin automation...
+		qtractorCurveList *pCurveList = qtractorPluginList::curveList();
+		if (pCurveList && !pCurveList->isEmpty()) {
+			qtractorCurveFile cfile(pCurveList);
+			QDomElement eCurveFile
+				= pDocument->document()->createElement("curve-file");
+			pPlugin->saveCurveFile(pDocument, &eCurveFile, &cfile);
+			ePlugin.appendChild(eCurveFile);
+		}
+	
 		// Add this plugin...
 		pElement->appendChild(ePlugin);
 
@@ -1599,10 +1634,11 @@ void qtractorPlugin::saveControllers (
 	Params::ConstIterator param = m_params.constBegin();
 	for ( ; param != m_params.constEnd(); ++param) {
 		qtractorPluginParam *pParam = param.value();
-		qtractorPluginParam::Observer *pObserver = pParam->observer();
+		qtractorMidiControlObserver *pObserver = pParam->observer();
 		if (pMidiControl->isMidiObserverMapped(pObserver)) {
 			qtractorMidiControl::Controller *pController
 				= new qtractorMidiControl::Controller;
+			pController->name = pParam->name();
 			pController->index = pParam->index();
 			pController->ctype = pObserver->type();
 			pController->channel = pObserver->channel();
@@ -1631,7 +1667,7 @@ void qtractorPlugin::mapControllers (
 			qtractorPluginParam *pParam = findParam(pController->index);
 			if (pParam == NULL)
 				continue;
-			qtractorPluginParam::Observer *pObserver = pParam->observer();
+			qtractorMidiControlObserver *pObserver = pParam->observer();
 			pObserver->setType(pController->ctype);
 			pObserver->setChannel(pController->channel);
 			pObserver->setParam(pController->param);
@@ -1643,6 +1679,105 @@ void qtractorPlugin::mapControllers (
 }
 
 
+// Load plugin automation curves (monitor, gain, pan, record, mute, solo).
+void qtractorPlugin::loadCurveFile (
+	QDomElement *pElement, qtractorCurveFile *pCurveFile )
+{
+	if (pCurveFile) pCurveFile->load(pElement);
+}
+
+
+// Save plugin automation curves (monitor, gain, pan, record, mute, solo).
+void qtractorPlugin::saveCurveFile ( qtractorDocument *pDocument,
+	QDomElement *pElement, qtractorCurveFile *pCurveFile ) const
+{
+	if (pCurveFile == NULL)
+		return;
+
+	qtractorCurveList *pCurveList = pCurveFile->list();
+	if (pCurveList == NULL)
+		return;
+
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return;
+
+	pCurveFile->clear();
+	pCurveFile->setBaseDir(pSession->sessionDir());
+	
+	unsigned short iParam = 0;
+	Params::ConstIterator param = m_params.constBegin();
+	for ( ; param != m_params.constEnd(); ++param) {
+		qtractorPluginParam *pParam = param.value();
+		qtractorCurve *pCurve = pParam->subject()->curve();
+		if (pCurve) {
+			unsigned short controller = (iParam % 0x7f);
+			if (controller == 0x00 || controller == 0x20)
+				++iParam; // Avoid bank-select controllers, please.
+			qtractorCurveFile::Item *pCurveItem = new qtractorCurveFile::Item;
+			pCurveItem->name = pParam->name();
+			pCurveItem->index = pParam->index();
+			pCurveItem->ctype = qtractorMidiEvent::CONTROLLER;
+			pCurveItem->channel = ((iParam / 0x7f) % 16);
+			pCurveItem->param = (iParam % 0x7f);
+			pCurveItem->mode = pCurve->mode();
+			pCurveItem->process = pCurve->isProcess();
+			pCurveItem->capture = pCurve->isCapture();
+			pCurveItem->locked = pCurve->isLocked();
+			pCurveItem->logarithmic = pCurve->isLogarithmic();
+			pCurveItem->color = pCurve->color();
+			pCurveItem->subject = pCurve->subject();
+			pCurveFile->addItem(pCurveItem);
+			++iParam;
+		}
+	}
+
+	if (pCurveFile->isEmpty())
+		return;
+
+	QString sBaseName(list()->name());
+	sBaseName += '_';
+	sBaseName += type()->label();
+	sBaseName += '_';
+	sBaseName += QString::number(type()->uniqueID(), 16);
+	sBaseName += "_curve";
+//	int iClipNo = (pCurveFile->filename().isEmpty() ? 0 : 1);
+	pCurveFile->setFilename(pSession->createFilePath(sBaseName, "mid", 1));
+
+	pCurveFile->save(pDocument, pElement, pSession->timeScale());
+}
+
+
+// Apply plugin automation curves (monitor, gain, pan, record, mute, solo).
+void qtractorPlugin::applyCurveFile ( qtractorCurveFile *pCurveFile ) const
+{
+	if (pCurveFile == NULL)
+		return;
+	if (pCurveFile->items().isEmpty())
+		return;
+
+	qtractorCurveList *pCurveList = pCurveFile->list();
+	if (pCurveList == NULL)
+		return;
+
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return;
+
+	pCurveFile->setBaseDir(pSession->sessionDir());
+
+	QListIterator<qtractorCurveFile::Item *> iter(pCurveFile->items());
+	while (iter.hasNext()) {
+		qtractorCurveFile::Item *pCurveItem = iter.next();
+		qtractorPluginParam *pParam = findParam(pCurveItem->index);
+		if (pParam)
+			pCurveItem->subject = pParam->subject();
+	}
+
+	pCurveFile->apply(pSession->timeScale());
+}
+
+
 //----------------------------------------------------------------------------
 // qtractorPluginParam -- Plugin parameter (control input port) instance.
 //
