@@ -47,6 +47,8 @@ class qtractorPluginListItem;
 class qtractorMidiManager;
 
 class qtractorDocument;
+class qtractorCurveList;
+class qtractorCurveFile;
 
 class QDomDocument;
 class QDomElement;
@@ -267,7 +269,7 @@ public:
 	// Constructor.
 	qtractorPluginParam(qtractorPlugin *pPlugin, unsigned long iIndex)
 		: m_pPlugin(pPlugin), m_iIndex(iIndex),
-			m_subject(0.0f), m_observer(&m_subject, this) {}
+			m_subject(0.0f), m_observer(&m_subject) {}
 
 	// Main properties accessors.
 	qtractorPlugin *plugin() const { return m_pPlugin; }
@@ -310,32 +312,6 @@ public:
 	float defaultValue() const
 		{ return m_subject.defaultValue(); }
 	
-	//------------------------------------------------------------------------
-	// Observer -- Local dedicated observer.
-	
-	class Observer : public qtractorMidiControlObserver
-	{
-	public:
-	
-		// Constructor.
-		Observer(qtractorSubject *pSubject, qtractorPluginParam *pParam)
-			: qtractorMidiControlObserver(pSubject), m_pParam(pParam) {}
-	
-	protected:
-	
-		// Update feedback.
-		void update()
-		{
-			m_pParam->updateValue(value(), true);
-			qtractorMidiControlObserver::update();
-		}
-	
-	private:
-	
-		// Members.
-		qtractorPluginParam *m_pParam;
-	};
-
 	// Current parameter value.
 	void setValue(float fValue, bool bUpdate);
 	float value() const
@@ -353,7 +329,7 @@ public:
 	qtractorSubject *subject() { return &m_subject; }
 
 	// Specialized observer value.
-	Observer *observer() { return &m_observer; }
+	qtractorMidiControlObserver *observer() { return &m_observer; }
 
 private:
 
@@ -365,7 +341,7 @@ private:
 	qtractorSubject m_subject;
 
 	// Port observer manager.
-	Observer m_observer;
+	qtractorMidiControlObserver m_observer;
 };
 
 
@@ -396,7 +372,7 @@ public:
 	unsigned int bufferSize() const;
 	unsigned short channels() const;
 	bool isMidi() const;
-	
+
 	// Activation methods.
 	void setActivated(bool bActivated);
 	bool isActivated() const
@@ -528,6 +504,19 @@ public:
 	const QString& config(const QString& sKey)
 		{ return m_configs[sKey]; }
 
+	// Plugin configuration (types) stuff.
+	typedef QHash<QString, QString> ConfigTypes;
+
+	void setConfigTypes(const ConfigTypes& ctypes)
+		{ m_ctypes = ctypes; }
+	const ConfigTypes& configTypes() const
+		{ return m_ctypes; }
+
+	void setConfigType(const QString& sKey, const QString& sType)
+		{ m_ctypes[sKey] = sType; }
+	const QString& configType(const QString& sKey)
+		{ return m_ctypes[sKey]; }
+
 	// Plugin parameter values stuff.
 	typedef QHash<unsigned long, float> Values;
 
@@ -550,12 +539,13 @@ public:
 
 	// Plugin configure clearance.
 	void clearConfigs()
-		{ m_configs.clear(); }
+		{ m_configs.clear(); m_ctypes.clear(); }
 	void clearValues()
 		{ m_values.clear(); }
 
 	// Load plugin configuration/parameter values stuff.
-	static void loadConfigs(QDomElement *pElement, Configs& configs);
+	static void loadConfigs(
+		QDomElement *pElement, Configs& configs, ConfigTypes& ctypes);
 	static void loadValues(QDomElement *pElement, Values& values);
 
 	// Save plugin configuration/parameter values stuff.
@@ -571,6 +561,13 @@ public:
 	// Map/realize plugin parameter controllers (MIDI).
 	void mapControllers(const qtractorMidiControl::Controllers& controllers);
 
+	// Plugin automation curve serialization methods.
+	static void loadCurveFile(
+	    QDomElement *pElement, qtractorCurveFile *pCurveFile);
+	void saveCurveFile(qtractorDocument *pDocument,
+		QDomElement *pElement, qtractorCurveFile *pCurveFile) const;
+	void applyCurveFile (qtractorCurveFile *pCurveFile) const;
+
 protected:
 
 	// Instance number settler.
@@ -602,6 +599,9 @@ private:
 	// Plugin configuration (CLOB) stuff.
 	Configs m_configs;
 
+	// Plugin configuration (type) stuff.
+	ConfigTypes m_ctypes;
+
 	// Plugin parameter values (part of configuration).
 	Values m_values;
 };
@@ -663,6 +663,10 @@ public:
 	unsigned int bufferSize() const { return m_iBufferSize; }
 	unsigned int flags()      const { return m_iFlags; }
 
+	// Specific automation curve list accessor.
+	qtractorCurveList *curveList() const
+		{ return m_pCurveList; }
+
 	// Specific MIDI manager accessor.
 	qtractorMidiManager *midiManager() const
 		{ return m_pMidiManager; }
@@ -731,6 +735,9 @@ private:
 	// An accessible list of observers.
 	QList<qtractorPluginListView *> m_views;
 
+	// Specific automation curve list accessor.
+	qtractorCurveList *m_pCurveList;
+
 	// Specific MIDI manager.
 	qtractorMidiManager *m_pMidiManager;
 
