@@ -61,7 +61,7 @@ bool qtractorPluginCommand::addPlugins (void)
 	if (pSession == NULL)
 		return false;
 
-	pSession->lock();
+//	pSession->lock();
 
 	// Add all listed plugins, in order...
 	QListIterator<qtractorPlugin *> iter(m_plugins);
@@ -74,7 +74,7 @@ bool qtractorPluginCommand::addPlugins (void)
 	// Avoid the disposal of the plugin reference(s).
 	setAutoDelete(false);
 
-	pSession->unlock();
+//	pSession->unlock();
 
 	return true;
 }
@@ -87,7 +87,7 @@ bool qtractorPluginCommand::removePlugins (void)
 	if (pSession == NULL)
 		return false;
 
-	pSession->lock();
+//	pSession->lock();
 
 	// Unlink all listed plugins, in order...
 	QListIterator<qtractorPlugin *> iter(m_plugins);
@@ -101,7 +101,7 @@ bool qtractorPluginCommand::removePlugins (void)
 	// Allow the disposal of the plugin reference(s).
 	setAutoDelete(true);
 
-	pSession->unlock();
+//	pSession->unlock();
 
 	return true;
 }
@@ -207,7 +207,7 @@ bool qtractorInsertPluginCommand::redo (void)
 	if (pPluginList == NULL)
 		return false;
 
-	pSession->lock();
+//	pSession->lock();
 
 	qtractorPlugin *pNextPlugin = pPlugin->next();
 
@@ -220,7 +220,7 @@ bool qtractorInsertPluginCommand::redo (void)
 	// Whether to allow the disposal of the plugin reference.
 	setAutoDelete(false);
 
-	pSession->unlock();
+//	pSession->unlock();
 
 	return true;
 }
@@ -240,7 +240,7 @@ bool qtractorInsertPluginCommand::undo (void)
 	if (pPluginList == NULL)
 		return false;
 
-	pSession->lock();
+//	pSession->lock();
 
 	qtractorPlugin *pNextPlugin = pPlugin->next();
 
@@ -253,7 +253,7 @@ bool qtractorInsertPluginCommand::undo (void)
 	// Whether to allow the disposal of the plugin reference.
 	setAutoDelete(true);
 
-	pSession->unlock();
+//	pSession->unlock();
 
 	return true;
 }
@@ -288,7 +288,7 @@ bool qtractorMovePluginCommand::redo (void)
 	if (pSession == NULL)
 		return false;
 
-	pSession->lock();
+//	pSession->lock();
 
 	// Save the previous track alright...
 	qtractorPlugin *pNextPlugin = pPlugin->next();
@@ -301,7 +301,7 @@ bool qtractorMovePluginCommand::redo (void)
 	m_pPluginList = pPluginList;
 	setNextPlugin(pNextPlugin);
 
-	pSession->unlock();
+//	pSession->unlock();
 
 	return true;
 }
