@@ -182,7 +182,7 @@ void qtractorPluginForm::setPlugin ( qtractorPlugin *pPlugin )
 		while (iRows > 12 && iCols < 4)
 			iRows = (params.count() / ++iCols);
 		if (params.count() % iCols)	// Adjust to balance.
-			iRows++;
+			++iRows;
 		int iRow = 0;
 		int iCol = 0;
 		qtractorPlugin::Params::ConstIterator param = params.constBegin();
@@ -192,10 +192,14 @@ void qtractorPluginForm::setPlugin ( qtractorPlugin *pPlugin )
 				= new qtractorPluginParamWidget(pParam, this);
 			m_paramWidgets.insert(pParam->index(), pParamWidget);
 			m_pGridLayout->addWidget(pParamWidget, iRow, iCol);
-			addMidiControlAction(pParamWidget, pParam->observer());
+			qtractorMidiControlObserver *pMidiObserver = pParam->observer();
+			if (pMidiObserver) {
+				pMidiObserver->setCurveList(pPlugin->list()->curveList());
+				addMidiControlAction(pParamWidget, pMidiObserver);
+			}
 			if (++iRow >= iRows) {
 				iRow = 0;
-				iCol++;
+				++iCol;
 			}
 		}
 	}
@@ -203,8 +207,8 @@ void qtractorPluginForm::setPlugin ( qtractorPlugin *pPlugin )
 
 	// Show editor button if available?
 	m_ui.EditToolButton->setVisible(bEditor);
-	//if (bEditor)
-	//	toggleEditor(m_pPlugin->isEditorVisible());
+	if (bEditor)
+		toggleEditor(m_pPlugin->isEditorVisible());
 
 	// Show insert tool options...
 	bool bInsertPlugin
@@ -260,12 +264,12 @@ void qtractorPluginForm::toggleEditor ( bool bOn )
 	if (m_iUpdate > 0)
 		return;
 
-	m_iUpdate++;
+	++m_iUpdate;
 
 	// Set the toggle button anyway...
 	m_ui.EditToolButton->setChecked(bOn);
 
-	m_iUpdate--;
+	--m_iUpdate;
 }
 
 
@@ -284,9 +288,9 @@ void qtractorPluginForm::setPreset ( const QString& sPreset )
 	if (sEditText.isEmpty())
 		sEditText = g_sDefPreset;
 
-	m_iUpdate++;
+	++m_iUpdate;
 	m_ui.PresetComboBox->setEditText(sEditText);
-	m_iUpdate--;
+	--m_iUpdate;
 }
 
 QString qtractorPluginForm::preset (void) const
@@ -327,9 +331,9 @@ void qtractorPluginForm::updateActivated (void)
 	if (m_iUpdate > 0)
 		return;
 
-	m_iUpdate++;
+	++m_iUpdate;
 	m_ui.ActivateToolButton->setChecked(m_pPlugin->isActivated());
-	m_iUpdate--;
+	--m_iUpdate;
 }
 
 
@@ -378,7 +382,7 @@ void qtractorPluginForm::changeParamSlot ( unsigned long iIndex )
 #endif
 
 	// Sure is dirty...
-	m_iDirtyCount++;
+	++m_iDirtyCount;
 	stabilize();
 }
 
@@ -390,7 +394,7 @@ void qtractorPluginForm::changePresetSlot ( const QString& sPreset )
 		return;
 
 	if (!sPreset.isEmpty() && m_ui.PresetComboBox->findText(sPreset) >= 0)
-		m_iDirtyCount++;
+		++m_iDirtyCount;
 
 	stabilize();
 }
@@ -636,7 +640,7 @@ void qtractorPluginForm::paramsSlot ( bool bOn )
 	if (m_iUpdate > 0)
 		return;
 
-	m_iUpdate++;
+	++m_iUpdate;
 
 	m_ui.ParamsGridWidget->setVisible(bOn);
 	if (bOn)
@@ -649,7 +653,7 @@ void qtractorPluginForm::paramsSlot ( bool bOn )
 	resize(width() + 1, height() + 1);
 	adjustSize();
 
-	m_iUpdate--;
+	--m_iUpdate;
 }
 
 
@@ -662,14 +666,14 @@ void qtractorPluginForm::editSlot ( bool bOn )
 	if (m_iUpdate > 0)
 		return;
 
-	m_iUpdate++;
+	++m_iUpdate;
 
 	if (bOn)
 		m_pPlugin->openEditor(this);
 	else
 		m_pPlugin->closeEditor();
 
-	m_iUpdate--;
+	--m_iUpdate;
 }
 
 
@@ -696,7 +700,7 @@ void qtractorPluginForm::activateSlot ( bool bOn )
 	if (m_iUpdate > 0)
 		return;
 
-	m_iUpdate++;
+	++m_iUpdate;
 
 	// Make it a undoable command...
 	qtractorSession *pSession = qtractorSession::getInstance();
@@ -704,7 +708,7 @@ void qtractorPluginForm::activateSlot ( bool bOn )
 		pSession->execute(
 			new qtractorActivatePluginCommand(m_pPlugin, bOn));
 
-	m_iUpdate--;
+	--m_iUpdate;
 }
 
 
@@ -720,7 +724,8 @@ void qtractorPluginForm::refresh (void)
 #ifdef CONFIG_DEBUG
 	qDebug("qtractorPluginForm[%p]::refresh()", this);
 #endif
-	m_iUpdate++;
+
+	++m_iUpdate;
 
 	qtractorSubject::flushQueue();
 
@@ -745,7 +750,7 @@ void qtractorPluginForm::refresh (void)
 	qtractorSubject::resetQueue();
 
 	m_iDirtyCount = 0;
-	m_iUpdate--;
+	--m_iUpdate;
 }
 
 
@@ -808,40 +813,25 @@ void qtractorPluginForm::keyPressEvent ( QKeyEvent *pKeyEvent )
 
 
 // MIDI controller/observer attachment (context menu)
-//
-Q_DECLARE_METATYPE(qtractorMidiControlObserver *);
-
 void qtractorPluginForm::addMidiControlAction (
 	QWidget *pWidget, qtractorMidiControlObserver *pMidiObserver )
 {
-	QAction *pAction = new QAction(
-		QIcon(":/images/itemControllers.png"),
-		tr("MIDI Controller..."), pWidget);
-
-	pAction->setData(
-		qVariantFromValue<qtractorMidiControlObserver *> (pMidiObserver));
-
-	QObject::connect(pAction,
-		SIGNAL(triggered(bool)),
-		SLOT(midiControlActionSlot()));
-
-	pWidget->addAction(pAction);
-	pWidget->setContextMenuPolicy(Qt::ActionsContextMenu);
+	qtractorMidiControlObserverForm::addMidiControlAction(
+		this, pWidget, pMidiObserver);
 }
 
 
 void qtractorPluginForm::midiControlActionSlot (void)
 {
-	QAction *pAction = qobject_cast<QAction *> (sender());
-	if (pAction) {
-		qtractorMidiControlObserver *pMidiObserver
-			= qVariantValue<qtractorMidiControlObserver *> (pAction->data());
-		if (pMidiObserver) {
-			qtractorMidiControlObserverForm form(this);
-			form.setMidiObserver(pMidiObserver);
-			form.exec();
-		}
-	}
+	qtractorMidiControlObserverForm::midiControlAction(
+		this, qobject_cast<QAction *> (sender()));
+}
+
+
+void qtractorPluginForm::midiControlMenuSlot ( const QPoint& pos )
+{
+	qtractorMidiControlObserverForm::midiControlMenu(
+		qobject_cast<QWidget *> (sender()), pos);
 }
 
 
@@ -981,6 +971,24 @@ qtractorPluginParamWidget::qtractorPluginParamWidget (
 		}
 	}
 
+	if (m_pCheckBox) {
+		QObject::connect(m_pCheckBox,
+			SIGNAL(valueChanged(float)),
+			SLOT(updateValue(float)));
+	}
+
+	if (m_pSpinBox) {
+		QObject::connect(m_pSpinBox,
+			SIGNAL(valueChanged(float)),
+			SLOT(updateValue(float)));
+	}
+
+	if (m_pSlider) {
+		QObject::connect(m_pSlider,
+			SIGNAL(valueChanged(float)),
+			SLOT(updateValue(float)));
+	}
+
 	QFrame::setLayout(m_pGridLayout);
 //	QFrame::setFrameShape(QFrame::StyledPanel);
 //	QFrame::setFrameShadow(QFrame::Raised);
@@ -1029,4 +1037,11 @@ int qtractorPluginParamWidget::paramDecimals (void) const
 }
 
 
+// Parameter value change slot.
+void qtractorPluginParamWidget::updateValue ( float fValue )
+{
+	m_pParam->updateValue(fValue, true);
+}
+
+
 // end of qtractorPluginForm.cpp
