@@ -1,7 +1,7 @@
 // qtractorPluginListView.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2010, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2011, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -243,7 +243,7 @@ qtractorPluginListView::~qtractorPluginListView (void)
 	setPluginList(NULL);
 
 	if (--g_iItemRefCount == 0) {
-		for (int i = 0; i < 2; i++) {
+		for (int i = 0; i < 2; ++i) {
 			delete g_pItemIcons[i];
 			g_pItemIcons[i] = NULL;
 		}
@@ -525,6 +525,8 @@ void qtractorPluginListView::activatePlugin (void)
 
 	pSession->execute(
 		new qtractorActivatePluginCommand(pPlugin, !pPlugin->isActivated()));
+
+	emit contentsChanged();
 }
 
 
@@ -554,6 +556,8 @@ void qtractorPluginListView::activateAllPlugins (void)
 	}
 
 	pSession->execute(pActivateAllCommand);
+
+	emit contentsChanged();
 }
 
 
@@ -583,6 +587,8 @@ void qtractorPluginListView::deactivateAllPlugins (void)
 	}
 
 	pSession->execute(pDeactivateAllCommand);
+
+	emit contentsChanged();
 }
 
 
@@ -665,7 +671,7 @@ void qtractorPluginListView::moveDownPlugin (void)
 
 
 // Show/hide an existing plugin form slot.
-void qtractorPluginListView::editPlugin (void)
+void qtractorPluginListView::propertiesPlugin (void)
 {
 	if (m_pPluginList == NULL)
 		return;
@@ -686,6 +692,28 @@ void qtractorPluginListView::editPlugin (void)
 }
 
 
+// Show/hide an existing plugin editor (GUI) slot.
+void qtractorPluginListView::editPlugin (void)
+{
+	if (m_pPluginList == NULL)
+		return;
+
+	qtractorPluginListItem *pItem
+		= static_cast<qtractorPluginListItem *> (QListWidget::currentItem());
+	if (pItem == NULL)
+		return;
+
+	qtractorPlugin *pPlugin = pItem->plugin();
+	if (pPlugin == NULL)
+		return;
+
+	if (pPlugin->isEditorVisible())
+		pPlugin->closeEditor();
+	else
+		pPlugin->openEditor(pPlugin->form());
+}
+
+
 // Insert specific slots.
 void qtractorPluginListView::insertPluginOutputs (void)
 {
@@ -760,6 +788,8 @@ void qtractorPluginListView::audioOutputBus (void)
 	pSession->execute(
 		new qtractorAudioOutputBusCommand(pMidiManager,
 			!pMidiManager->isAudioOutputBus()));
+
+	emit contentsChanged();
 }
 
 
@@ -875,6 +905,7 @@ void qtractorPluginListView::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 				pSession->execute(
 					new qtractorActivatePluginCommand(pPlugin,
 						!pPlugin->isActivated()));
+				emit contentsChanged();
 			}
 		}
 	}
@@ -1184,12 +1215,19 @@ void qtractorPluginListView::contextMenuEvent (
 	menu.addSeparator();
 
 	pAction = menu.addAction(
-		QIcon(":/images/formEdit.png"),
-		tr("&Edit Plugin..."), this, SLOT(editPlugin()));
+		QIcon(":/images/trackProperties.png"),
+		tr("&Properties..."), this, SLOT(propertiesPlugin()));
 	pAction->setCheckable(true);
 	pAction->setChecked(pPlugin && pPlugin->isFormVisible());
 	pAction->setEnabled(pItem != NULL);
 
+	pAction = menu.addAction(
+		QIcon(":/images/formEdit.png"),
+		tr("&Edit"), this, SLOT(editPlugin()));
+	pAction->setCheckable(true);
+	pAction->setChecked(pPlugin && pPlugin->isEditorVisible());
+	pAction->setEnabled(pPlugin && pPlugin->type()->isEditor());
+
 	qtractorMidiManager *pMidiManager = m_pPluginList->midiManager();
 	if (pMidiManager) {
 		menu.addSeparator();
