@@ -29,6 +29,7 @@
 #include "qtractorOptions.h"
 
 #include <QHeaderView>
+#include <QPushButton>
 #include <QLineEdit>
 #include <QPainter>
 #include <QRegExp>
@@ -95,7 +96,7 @@ qtractorPluginSelectForm::qtractorPluginSelectForm (
 	m_ui.PluginListView->resizeColumnToContents(1);		// Audio.
 	m_ui.PluginListView->resizeColumnToContents(2);		// MIDI.
 	m_ui.PluginListView->resizeColumnToContents(3);		// Controls.
-	m_ui.PluginListView->resizeColumnToContents(4);		// Mode.
+	m_ui.PluginListView->resizeColumnToContents(4);		// Modes.
 	pHeader->resizeSection(5, 120);						// Path.
 	m_ui.PluginListView->resizeColumnToContents(6);		// Index
 	m_ui.PluginListView->resizeColumnToContents(7);		// Instances
@@ -141,21 +142,21 @@ qtractorPluginSelectForm::qtractorPluginSelectForm (
 	QObject::connect(m_ui.PluginListView,
 		SIGNAL(itemSelectionChanged()),
 		SLOT(stabilize()));
-	QObject::connect(m_ui.PluginListView,
-		SIGNAL(itemActivated(QTreeWidgetItem *, int)),
-		SLOT(accept()));
+//	QObject::connect(m_ui.PluginListView,
+//		SIGNAL(itemActivated(QTreeWidgetItem *, int)),
+//		SLOT(accept()));
 	QObject::connect(m_ui.PluginListView,
 		SIGNAL(itemDoubleClicked(QTreeWidgetItem *, int)),
 		SLOT(accept()));
 	QObject::connect(m_ui.PluginActivateCheckBox,
 		SIGNAL(clicked()),
 		SLOT(stabilize()));
-	QObject::connect(m_ui.OkPushButton,
-		SIGNAL(clicked()),
+	QObject::connect(m_ui.DialogButtonBox,
+		SIGNAL(accepted()),
 		SLOT(accept()));
-	QObject::connect(m_ui.CancelPushButton,
-		SIGNAL(clicked()),
-		SLOT(close()));
+	QObject::connect(m_ui.DialogButtonBox,
+		SIGNAL(rejected()),
+		SLOT(reject()));
 }
 
 
@@ -335,10 +336,17 @@ void qtractorPluginSelectForm::refresh (void)
 			cols << QString("%1:%2").arg(iAudioIns).arg(iAudioOuts);
 			cols << QString("%1:%2").arg(iMidiIns).arg(iMidiOuts);
 			cols << QString("%1:%2").arg(iControlIns).arg(iControlOuts);
+			QStringList modes;
+			if (pType->isEditor())
+				modes << tr("GUI");
+			if (pType->isConfigure())
+				modes << tr("EXT");
 			if (pType->isRealtime())
-				cols << tr("RT");
-			else
+				modes << tr("RT");
+			if (modes.isEmpty())
 				cols << "-";
+			else
+				cols << modes.join(",");
 			cols << sFilename;
 			cols << QString::number(pType->index());
 			cols << QString::number(iInstances);
@@ -355,7 +363,7 @@ void qtractorPluginSelectForm::refresh (void)
 			pItem->setTextAlignment(1, Qt::AlignHCenter);	// Audio
 			pItem->setTextAlignment(2, Qt::AlignHCenter);	// MIDI
 			pItem->setTextAlignment(3, Qt::AlignHCenter);	// Controls
-			pItem->setTextAlignment(4, Qt::AlignHCenter);	// Mode
+			pItem->setTextAlignment(4, Qt::AlignHCenter);	// Modes
 			items.append(pItem);
 		}
 	}
@@ -377,7 +385,7 @@ void qtractorPluginSelectForm::refresh (void)
 // Stabilize slot.
 void qtractorPluginSelectForm::stabilize (void)
 {
-	m_ui.OkPushButton->setEnabled(
+	m_ui.DialogButtonBox->button(QDialogButtonBox::Ok)->setEnabled(
 		!m_ui.PluginListView->selectedItems().isEmpty());
 }
 
