@@ -35,6 +35,7 @@
 #include "qtractorMidiBuffer.h"
 
 #include "qtractorPlugin.h"
+#include "qtractorCurve.h"
 
 #include "qtractorInstrument.h"
 #include "qtractorCommand.h"
@@ -216,6 +217,8 @@ void qtractorSession::clear (void)
 
 	ATOMIC_SET(&m_busy, 0);
 
+	m_pAudioPeakFactory->sync();
+
 	m_pCurrentTrack = NULL;
 
 	m_tracks.clear();
@@ -589,6 +592,20 @@ unsigned long qtractorSession::tickFromFrame ( unsigned long iFrame )
 }
 
 
+// Tick/Frame range conversion (delta conversion).
+unsigned long qtractorSession::frameFromTickRange (
+    unsigned long iTickStart, unsigned long iTickEnd )
+{
+	return m_props.timeScale.frameFromTickRange(iTickStart, iTickEnd);
+}
+
+unsigned long qtractorSession::tickFromFrameRange (
+    unsigned long iFrameStart, unsigned long iFrameEnd )
+{
+	return m_props.timeScale.tickFromFrameRange(iFrameStart, iFrameEnd);
+}
+
+
 // Beat/frame snap filters.
 unsigned long qtractorSession::tickSnap ( unsigned long iTick )
 {
@@ -798,6 +815,9 @@ void qtractorSession::insertTrack ( qtractorTrack *pTrack,
 		setSoloTracks(true);
 #endif
 
+	if (pTrack->curveList())
+		m_curves.insert(pTrack->curveList(), pTrack);
+
 	qtractorSessionCursor *pSessionCursor = m_cursors.first();
 	while (pSessionCursor) {
 		pSessionCursor->addTrack(pTrack);
@@ -868,6 +888,9 @@ void qtractorSession::unlinkTrack ( qtractorTrack *pTrack )
 	if (pTrack->isSolo())
 		setSoloTracks(false);
 
+	if (pTrack->curveList())
+		m_curves.remove(pTrack->curveList());
+
 	if (pTrack->trackType() == qtractorTrack::Midi)
 		releaseMidiTag(pTrack);
 
@@ -887,9 +910,9 @@ qtractorTrack *qtractorSession::trackAt ( int iTrack ) const
 void qtractorSession::setRecordTracks ( bool bRecord )
 {
 	if (bRecord) {
-		m_iRecordTracks++;
+		++m_iRecordTracks;
 	} else if (m_iRecordTracks > 0) {
-		m_iRecordTracks--;
+		--m_iRecordTracks;
 	}
 }
 
@@ -903,9 +926,9 @@ unsigned int qtractorSession::recordTracks (void) const
 void qtractorSession::setMuteTracks ( bool bMute )
 {
 	if (bMute) {
-		m_iMuteTracks++;
+		++m_iMuteTracks;
 	} else if (m_iMuteTracks > 0) {
-		m_iMuteTracks--;
+		--m_iMuteTracks;
 	}
 }
 
@@ -919,9 +942,9 @@ unsigned int qtractorSession::muteTracks (void) const
 void qtractorSession::setSoloTracks ( bool bSolo )
 {
 	if (bSolo) {
-		m_iSoloTracks++;
+		++m_iSoloTracks;
 	} else if (m_iSoloTracks > 0) {
-		m_iSoloTracks--;
+		--m_iSoloTracks;
 	}
 }
 
@@ -1192,6 +1215,10 @@ void qtractorSession::setPlayHead ( unsigned long iFrame )
 
 	seek(iFrame, true);
 
+	// Sync all track automation...
+	if (!bPlaying)
+		process_curve(iFrame);
+
 	setPlaying(bPlaying);
 	unlock();
 }
@@ -1309,37 +1336,37 @@ unsigned long qtractorSession::punchOutTime (void) const
 
 unsigned long qtractorSession::frameTime (void) const
 {
-	return	m_pAudioEngine->sessionCursor()->frameTime();
+	return m_pAudioEngine->sessionCursor()->frameTime();
 }
 
 unsigned long qtractorSession::framePos (void) const
 {
-	return	m_pAudioEngine->sessionCursor()->framePos();
+	return m_pAudioEngine->sessionCursor()->framePos();
 }
 
 
 // Sanitize a given name.
 QString qtractorSession::sanitize ( const QString& s )
 {
-	return s.simplified().replace(QRegExp("[\\s|\\.|\\-]+"), "_");
+	return s.simplified().replace(QRegExp("[\\s|\\.|\\-|/]+"), "_");
 }
 
 
 // Create a brand new filename (absolute file path).
-QString qtractorSession::createFilePath ( const QString& sTrackName,
-	int iClipNo, const QString& sExt )
+QString qtractorSession::createFilePath (
+	const QString& sTrackName, const QString& sExt, int iClipNo )
 {
 	QString sFilename = qtractorSession::sanitize(m_props.sessionName);
 	if (!sFilename.isEmpty())
 		sFilename += '-';
 	sFilename += qtractorSession::sanitize(sTrackName) + "-%1." + sExt;
 
-	if (iClipNo < 1)
-		iClipNo++;
-
-	QFileInfo fi(m_props.sessionDir, sFilename.arg(iClipNo));
-	while (fi.exists())
+	QFileInfo fi;
+	if (iClipNo > 0) {
+		fi.setFile(m_props.sessionDir, sFilename.arg(iClipNo));
+	} else do {
 		fi.setFile(m_props.sessionDir, sFilename.arg(++iClipNo));
+	} while (fi.exists());
 
 #ifdef CONFIG_DEBUG
 	qDebug("qtractorSession::createFilePath(\"%s\")",
@@ -1356,25 +1383,34 @@ void qtractorSession::setRecording ( bool bRecording )
 	m_bRecording = bRecording;
 
 	// For all armed tracks...
+	unsigned long iClipStart = playHead();
+
+	if (isPunching()) {
+		unsigned long iPunchIn = punchIn();
+		if (iClipStart < iPunchIn)
+			iClipStart = iPunchIn;
+	}
+
 	for (qtractorTrack *pTrack = m_tracks.first();
 			pTrack; pTrack = pTrack->next()) {
 		if (pTrack->isRecord())
-			trackRecord(pTrack, bRecording);
+			trackRecord(pTrack, bRecording, iClipStart);
 	}
 }
 
-bool qtractorSession::isRecording() const
+bool qtractorSession::isRecording (void) const
 {
 	return m_bRecording;
 }
 
 
 // Immediate track record-arming.
-void qtractorSession::trackRecord ( qtractorTrack *pTrack, bool bRecord )
+void qtractorSession::trackRecord (
+	qtractorTrack *pTrack, bool bRecord, unsigned long iClipStart )
 {
 #ifdef CONFIG_DEBUG
-	qDebug("qtractorSession::trackRecord(\"%s\", %d)",
-		pTrack->trackName().toUtf8().constData(), (int) bRecord);
+	qDebug("qtractorSession::trackRecord(\"%s\", %d, %lu)",
+		pTrack->trackName().toUtf8().constData(), int(bRecord), iClipStart);
 #endif
 
 	// Just ditch the in-record clip...
@@ -1386,10 +1422,10 @@ void qtractorSession::trackRecord ( qtractorTrack *pTrack, bool bRecord )
 		// One-down current tracks in record mode.
 		switch (pTrack->trackType()) {
 		case qtractorTrack::Audio:
-			m_iAudioRecord--;
+			--m_iAudioRecord;
 			break;
 		case qtractorTrack::Midi:
-			m_iMidiRecord--;
+			--m_iMidiRecord;
 			break;
 		default:
 			break;
@@ -1403,27 +1439,18 @@ void qtractorSession::trackRecord ( qtractorTrack *pTrack, bool bRecord )
 		return;
 	}
 
-	// Here's the place to create and set the capture clip...
-	unsigned long iClipStart = playHead();
-
-	if (isPunching()) {
-		unsigned long iPunchIn = punchIn();
-		if (iClipStart < iPunchIn)
-			iClipStart = iPunchIn;
-	}
-
 	switch (pTrack->trackType()) {
 	case qtractorTrack::Audio:
 	{
 		qtractorAudioClip *pAudioClip = new qtractorAudioClip(pTrack);
 		pAudioClip->setClipStart(iClipStart);
 		pAudioClip->openAudioFile(
-			createFilePath(pTrack->trackName(), 0,
+			createFilePath(pTrack->trackName(),
 				qtractorAudioFileFactory::defaultExt()),
 			qtractorAudioFile::Write);
 		pTrack->setClipRecord(pAudioClip);
 		// One-up audio tracks in record mode.
-		m_iAudioRecord++;
+		++m_iAudioRecord;
 		break;
 	}
 	case qtractorTrack::Midi:
@@ -1431,7 +1458,7 @@ void qtractorSession::trackRecord ( qtractorTrack *pTrack, bool bRecord )
 		qtractorMidiClip *pMidiClip = new qtractorMidiClip(pTrack);
 		pMidiClip->setClipStart(iClipStart);
 		pMidiClip->openMidiFile(
-			createFilePath(pTrack->trackName(), 0, "mid"),
+			createFilePath(pTrack->trackName(), "mid"),
 			qtractorMidiClip::defaultFormat(),
 			qtractorMidiFile::Write);
 		pTrack->setClipRecord(pMidiClip);
@@ -1444,7 +1471,7 @@ void qtractorSession::trackRecord ( qtractorTrack *pTrack, bool bRecord )
 				pMidiClip->sequence()->setTimeOffset(iTime - iTimeStart);
 		}
 		// One-up MIDI tracks in record mode.
-		m_iMidiRecord++;
+		++m_iMidiRecord;
 		break;
 	}
 	default:
@@ -1589,16 +1616,24 @@ bool qtractorSession::isAutoTimeStretch (void) const
 void qtractorSession::process ( qtractorSessionCursor *pSessionCursor,
 	unsigned long iFrameStart, unsigned long iFrameEnd )
 {
-	// Now, for every Audio track...
+	const qtractorTrack::TrackType syncType = pSessionCursor->syncType();
+
+	// Now, for every track...
 	int iTrack = 0;
 	qtractorTrack *pTrack = m_tracks.first();
 	while (pTrack) {
-		if (pTrack->trackType() == pSessionCursor->syncType()) {
+		// Track automation processing...
+		if (syncType == qtractorTrack::Audio) {
+			qtractorCurveList *pCurveList = pTrack->curveList();
+			if (pCurveList && pCurveList->isProcess())
+				pCurveList->process(iFrameStart);
+		}
+		if (syncType == pTrack->trackType()) {
 			pTrack->process(pSessionCursor->clip(iTrack),
 				iFrameStart, iFrameEnd);
 		}
 		pTrack = pTrack->next();
-		iTrack++;
+		++iTrack;
 	}
 }
 
@@ -1616,6 +1651,25 @@ void qtractorSession::process_record (
 }
 
 
+// Session special process automation executive.
+void qtractorSession::process_curve ( unsigned long iFrame )
+{
+	// Now, for every track...
+	qtractorTrack *pTrack = m_tracks.first();
+	while (pTrack) {
+		pTrack->process_curve(iFrame);
+		pTrack = pTrack->next();
+	}
+}
+
+
+// Find track of specific curve-list.
+qtractorTrack *qtractorSession::findTrack ( qtractorCurveList *pCurveList ) const
+{
+	return m_curves.value(pCurveList, NULL);
+}
+
+
 // Document element methods.
 bool qtractorSession::loadElement (
 	qtractorSessionDocument *pDocument, QDomElement *pElement )
