@@ -140,6 +140,12 @@ public:
 	unsigned long frameFromTick(unsigned long iTick);
 	unsigned long tickFromFrame(unsigned long iFrame);
 
+	// Tick/Frame range conversion (delta conversion).
+	unsigned long frameFromTickRange(
+	    unsigned long iTickStart, unsigned long iTickEnd);
+	unsigned long tickFromFrameRange(
+	    unsigned long iFrameStart, unsigned long iFrameEnd);
+
 	// Beat/frame snap filters.
 	unsigned long tickSnap(unsigned long iTick);
 	unsigned long frameSnap(unsigned long iFrame);
@@ -266,8 +272,8 @@ public:
 	static QString sanitize(const QString& s); 
 
 	// Create a brand new filename (absolute file path).
-	QString createFilePath(const QString& sTrackName,
-		int iClipNo, const QString& sExt); 
+	QString createFilePath(
+		const QString& sTrackName, const QString& sExt, int iClipNo = 0);
 
 	// Consolidated session record state.
 	void setRecording(bool bRecording);
@@ -278,7 +284,8 @@ public:
 	unsigned short midiRecord() const;
 
 	// Special track-immediate methods.
-	void trackRecord(qtractorTrack *pTrack, bool bRecord);
+	void trackRecord(
+		qtractorTrack *pTrack, bool bRecord, unsigned long iClipStart);
 	void trackMute(qtractorTrack *pTrack, bool bMute);
 	void trackSolo(qtractorTrack *pTrack, bool bSolo);
 
@@ -311,6 +318,9 @@ public:
 	void process_record(
 		unsigned long iFrameStart, unsigned long iFrameEnd);
 
+	// Session special process automation executive.
+	void process_curve(unsigned long iFrame);
+
 	// Document element methods.
 	bool loadElement(qtractorSessionDocument *pDocument, QDomElement *pElement);
 	bool saveElement(qtractorSessionDocument *pDocument, QDomElement *pElement);
@@ -351,6 +361,9 @@ public:
 	// Instrument names mapping.
 	qtractorInstrumentList *instruments() const;
 
+	// Find track of specific curve-list.
+	qtractorTrack *findTrack(qtractorCurveList *pCurveList) const;
+
 	// Singleton instance accessor.
 	static qtractorSession *getInstance();
 
@@ -441,6 +454,9 @@ private:
 	// Session command executive (undo/redo)
 	qtractorCommandList *m_pCommands;
 
+	// Curve-to-track mapping.
+	QHash<qtractorCurveList *, qtractorTrack *> m_curves;
+
 	// The singleton instance.
 	static qtractorSession *g_pSession;
 };
