@@ -1,7 +1,7 @@
 // qtractorTimeScale.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2010, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2011, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -457,9 +457,9 @@ unsigned long qtractorTimeScale::frameFromText (
 					bars += pNode->bar;
 			} else {
 				if (bars > 0)
-					bars--;
+					--bars;
 				if (beats > 0)
-					beats--;
+					--beats;
 			}
 			pNode = m_cursor.seekBar(bars);
 			if (pNode) {
@@ -528,8 +528,8 @@ QString qtractorTimeScale::textFromFrame (
 					bars += pNode->bar;
 			}
 			if (!bDelta) {
-				bars++;
-				beats++;
+				++bars;
+				++beats;
 			}
 			sText.sprintf("%u.%02u.%03lu", bars, beats, ticks);
 			break;
@@ -626,13 +626,13 @@ QStringList qtractorTimeScale::snapItems ( int iSnap )
 
 	if (iSnap == 0) {
 		items.append(QObject::tr("None"));
-		iSnap++;
+		++iSnap;
 	}
 
 	QString sPrefix = QObject::tr("Beat");
 	if (iSnap == 1) {
 		items.append(sPrefix);
-		iSnap++;
+		++iSnap;
 	}
 
 	sPrefix += "/%1";
@@ -643,4 +643,27 @@ QStringList qtractorTimeScale::snapItems ( int iSnap )
 }
 
 
+// Tick/Frame range conversion (delta conversion).
+unsigned long qtractorTimeScale::frameFromTickRange (
+	unsigned long iTickStart, unsigned long iTickEnd )
+{
+	Node *pNode = m_cursor.seekTick(iTickStart);
+	unsigned long iFrameStart = (pNode ? pNode->frameFromTick(iTickStart) : 0);
+	pNode = m_cursor.seekTick(iTickEnd);
+	unsigned long iFrameEnd = (pNode ? pNode->frameFromTick(iTickEnd) : 0);
+	return (iFrameEnd > iFrameStart ? iFrameEnd - iFrameStart : 0);
+}
+
+
+unsigned long qtractorTimeScale::tickFromFrameRange (
+	unsigned long iFrameStart, unsigned long iFrameEnd )
+{
+	Node *pNode = m_cursor.seekFrame(iFrameStart);
+	unsigned long iTickStart = (pNode ? pNode->tickFromFrame(iFrameStart) : 0);
+	pNode = m_cursor.seekFrame(iFrameEnd);
+	unsigned long iTickEnd = (pNode ? pNode->tickFromFrame(iFrameEnd) : 0);
+	return (iTickEnd > iTickStart ? iTickEnd - iTickStart : 0);
+}
+
+
 // end of qtractorTimeScale.cpp
