@@ -1,7 +1,7 @@
 // qtractorTimeScale.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2010, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2011, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -35,8 +35,11 @@ class qtractorTimeScale
 {
 public:
 
+	// Available display-formats.
+	enum DisplayFormat { Frames = 0, Time, BBT };
+
 	// Default constructor.
-	qtractorTimeScale() : m_cursor(this)
+	qtractorTimeScale() : m_displayFormat(Frames), m_cursor(this)
 		{ clear(); }
 
 	// Copy constructor.
@@ -176,10 +179,10 @@ public:
 
 		// Beat/pixel convertors.
 		unsigned int beatFromPixel(int x) const
-			{ return beat + uroundf(
+			{ return beat + (unsigned long) (
 				(beatRate * (x - pixel)) / ts->pixelRate()); }
 		int pixelFromBeat(unsigned int iBeat) const
-			{ return pixel + uroundf(
+			{ return pixel + (unsigned long) (
 				(ts->pixelRate() * (iBeat - beat)) / beatRate); }
 
 		// Pixel/beat rate convertor.
@@ -188,10 +191,10 @@ public:
 
 		// Bar/pixel convertors.
 		unsigned short barFromPixel(int x) const
-			{ return bar + uroundf(
+			{ return bar + (unsigned long) (
 				(beatRate * (x - pixel)) / (ts->pixelRate() * beatsPerBar)); }
 		int pixelFromBar(unsigned short iBar) const
-			{ return pixel + uroundf(
+			{ return pixel + (unsigned long) (
 				(ts->pixelRate() * beatsPerBar * (iBar - bar)) / beatRate); }
 
 		// Bar/beat convertors.
@@ -383,9 +386,6 @@ public:
 		return (pNode ? pNode->pixelSnap(x) : x);
 	}
 
-	// Available display-formats.
-	enum DisplayFormat { Frames, Time, BBT };
-
 	// Display-format accessors.
 	void setDisplayFormat(DisplayFormat displayFormat)
 		{ m_displayFormat = displayFormat; }
@@ -469,6 +469,12 @@ public:
 		return (pNode ? pNode->beatDivisor : 2);
 	}
 
+	// Tick/Frame range conversion (delta conversion).
+	unsigned long frameFromTickRange(
+	    unsigned long iTickStart, unsigned long iTickEnd);
+	unsigned long tickFromFrameRange(
+	    unsigned long iFrameStart, unsigned long iFrameEnd);
+
 protected:
 
 	// Tempo-map independent coefficients.
