@@ -30,6 +30,7 @@
 #include "qtractorDocument.h"
 #include "qtractorAudioEngine.h"
 #include "qtractorAudioMonitor.h"
+#include "qtractorAudioBuffer.h"
 #include "qtractorMidiEngine.h"
 #include "qtractorMidiMonitor.h"
 #include "qtractorMidiBuffer.h"
@@ -37,41 +38,17 @@
 #include "qtractorPlugin.h"
 #include "qtractorMixer.h"
 #include "qtractorMeter.h"
+#include "qtractorCurveFile.h"
 
 #include "qtractorTrackCommand.h"
 
 #include "qtractorMainForm.h"
+#include "qtractorTracks.h"
 
 #include <QPainter>
 
 #include <QDomDocument>
-
-
-//------------------------------------------------------------------------
-// qtractorTrack::MonitorObserver -- Local track state observer.
-
-class qtractorTrack::MonitorObserver : public qtractorMidiControlObserver
-{
-public:
-
-	// Constructor.
-	MonitorObserver(qtractorTrack *pTrack, qtractorSubject *pSubject)
-		: qtractorMidiControlObserver(pSubject), m_pTrack(pTrack) {}
-
-protected:
-
-	// Update feedback.
-	void update()
-	{
-		m_pTrack->monitorChangeNotify(value() > 0.0f);
-		qtractorMidiControlObserver::update();
-	}
-
-private:
-
-	// Members.
-	qtractorTrack *m_pTrack;
-};
+#include <QFileInfo>
 
 
 //------------------------------------------------------------------------
@@ -91,8 +68,23 @@ protected:
 	// Update feedback.
 	void update()
 	{
-		m_pTrack->stateChangeNotify(m_toolType, value() > 0.0f);
+		bool bOn = (value() > 0.0f);
+		switch (m_toolType) {
+		case qtractorTrack::Record:
+			m_pTrack->setRecord(bOn);
+			break;
+		case qtractorTrack::Mute:
+			m_pTrack->setMute(bOn);
+			break;
+		case qtractorTrack::Solo:
+			m_pTrack->setSolo(bOn);
+			break;
+		}
 		qtractorMidiControlObserver::update();
+
+		qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+		if (pMainForm)
+			pMainForm->tracks()->updateContents(true);
 	}
 
 private:
@@ -103,6 +95,78 @@ private:
 };
 
 
+//----------------------------------------------------------------------------
+// qtractorTrack::MidiVolumeObserver -- Local dedicated observer.
+
+class qtractorTrack::MidiVolumeObserver : public qtractorObserver
+{
+public:
+
+	// Constructor.
+	MidiVolumeObserver(qtractorTrack *pTrack, qtractorSubject *pSubject)
+		: qtractorObserver(pSubject), m_pTrack(pTrack), m_volume(0) {}
+
+protected:
+
+	// Update feedback.
+	void update()
+	{
+		const float fVolume = value();
+		const unsigned char vol = int(127.0f * fVolume) & 0x7f;
+		if (m_volume != vol) {
+			qtractorMidiBus *pMidiBus
+				= static_cast<qtractorMidiBus *> (m_pTrack->outputBus());
+			if (pMidiBus) {
+				pMidiBus->setVolume(m_pTrack, fVolume);
+				m_volume = vol;
+			}
+		}
+	}
+
+private:
+
+	// Members.
+	qtractorTrack *m_pTrack;
+	unsigned char  m_volume;
+};
+
+
+//----------------------------------------------------------------------------
+// qtractorTrack::MidiPanningObserver -- Local dedicated observer.
+
+class qtractorTrack::MidiPanningObserver : public qtractorObserver
+{
+public:
+
+	// Constructor.
+	MidiPanningObserver(qtractorTrack *pTrack, qtractorSubject *pSubject)
+		: qtractorObserver(pSubject), m_pTrack(pTrack), m_panning(0) {}
+
+protected:
+
+	// Update feedback.
+	void update()
+	{
+		const float fPanning = value();
+		const unsigned char pan = (0x40 + int(63.0f * fPanning)) & 0x7f;
+		if (m_panning != pan) {
+			qtractorMidiBus *pMidiBus
+				= static_cast<qtractorMidiBus *> (m_pTrack->outputBus());
+			if (pMidiBus) {
+				pMidiBus->setPanning(m_pTrack, fPanning);
+				m_panning = pan;
+			}
+		}
+	}
+
+private:
+
+	// Members.
+	qtractorTrack *m_pTrack;
+	unsigned char  m_panning;
+};
+
+
 //-------------------------------------------------------------------------
 // qtractorTrack::Properties -- Track properties structure.
 
@@ -174,13 +238,23 @@ qtractorTrack::qtractorTrack ( qtractorSession *pSession, TrackType trackType )
 
 	m_clips.setAutoDelete(true);
 
+	m_pSyncThread = NULL;
+
+	m_pMidiVolumeObserver  = NULL;
+	m_pMidiPanningObserver = NULL;
+
 	m_pMonitorSubject = new qtractorSubject();
+	m_pMonitorSubject->setToggled(true);
 
 	m_pRecordSubject  = new qtractorSubject();
 	m_pMuteSubject    = new qtractorSubject();
 	m_pSoloSubject    = new qtractorSubject();
 
-	m_pMonitorObserver = new MonitorObserver(this, m_pMonitorSubject);
+	m_pRecordSubject->setToggled(true);
+	m_pMuteSubject->setToggled(true);
+	m_pSoloSubject->setToggled(true);
+
+	m_pMonitorObserver = new qtractorMidiControlObserver(m_pMonitorSubject);
 
 	m_pRecordObserver = new StateObserver(this, Record, m_pRecordSubject);
 	m_pMuteObserver   = new StateObserver(this, Mute,   m_pMuteSubject);
@@ -192,6 +266,8 @@ qtractorTrack::qtractorTrack ( qtractorSession *pSession, TrackType trackType )
 
 	m_pPluginList = new qtractorPluginList(0, 0, pSession->sampleRate(), iFlags);
 
+	m_pCurveFile = new qtractorCurveFile(m_pPluginList->curveList());
+
 	setHeight(HeightBase);	// Default track height.
 	clear();
 }
@@ -223,6 +299,8 @@ qtractorTrack::~qtractorTrack (void)
 	qDeleteAll(m_controllers);
 	m_controllers.clear();
 
+	if (m_pCurveFile)
+		delete m_pCurveFile;
 	if (m_pPluginList)
 		delete m_pPluginList;
 	if (m_pMonitor)
@@ -247,6 +325,16 @@ void qtractorTrack::clear (void)
 	m_props.solo    = false;
 	m_props.gain    = 1.0f;
 	m_props.panning = 0.0f;
+
+	if (m_pSyncThread) {
+		if (m_pSyncThread->isRunning()) do {
+			m_pSyncThread->setRunState(false);
+		//	m_pSyncThread->terminate();
+			m_pSyncThread->sync();
+		} while (!m_pSyncThread->wait(100));
+		delete m_pSyncThread;
+		m_pSyncThread = NULL;
+	}
 }
 
 
@@ -339,6 +427,10 @@ bool qtractorTrack::open (void)
 		if (pMidiBus) {
 			m_pMonitor = new qtractorMidiMonitor(
 				m_props.gain, m_props.panning);
+			m_pMidiVolumeObserver = new MidiVolumeObserver(
+				this, m_pMonitor->gainSubject());
+			m_pMidiPanningObserver = new MidiPanningObserver(
+				this, m_pMonitor->panningSubject());
 		}
 		// Get audio bus as for the plugin list...
 		qtractorAudioBus *pAudioBus = NULL;
@@ -382,6 +474,22 @@ bool qtractorTrack::open (void)
 					pStrip->setTrack(this);
 			}
 		}
+		// Update gain and panning curve new subjects...
+		qtractorCurveList *pCurveList = m_pPluginList->curveList();
+		if (pCurveList) {
+			qtractorCurve *pCurve = pCurveList->first();
+			while (pCurve) {
+				qtractorSubject *pSubject = pCurve->subject();
+				if (pSubject) {
+					if (pSubject == pMonitor->gainSubject())
+						pCurve->setSubject(m_pMonitor->gainSubject());
+					else
+					if (pSubject == pMonitor->panningSubject())
+						pCurve->setSubject(m_pMonitor->panningSubject());
+				}
+				pCurve = pCurve->next();
+			}
+		}		
 		// That's it...
 		delete pMonitor;
 	}
@@ -394,6 +502,16 @@ bool qtractorTrack::open (void)
 // Track close method.
 void qtractorTrack::close (void)
 {
+	if (m_pMidiVolumeObserver) {
+		delete m_pMidiVolumeObserver;
+		m_pMidiVolumeObserver = NULL;
+	}
+
+	if (m_pMidiPanningObserver) {
+		delete m_pMidiPanningObserver;
+		m_pMidiPanningObserver = NULL;
+	}
+
 #if 0
 	if (m_pMonitor) {
 		delete m_pMonitor;
@@ -480,7 +598,7 @@ void qtractorTrack::setMonitor ( bool bMonitor )
 // Record status accessors.
 void qtractorTrack::setRecord ( bool bRecord )
 {
-	const bool bOldRecord = isRecord();
+	const bool bOldRecord = m_props.record; // isRecord();
 	if ((bOldRecord && !bRecord) || (!bOldRecord && bRecord))
 		m_pSession->setRecordTracks(bRecord);
 
@@ -488,13 +606,20 @@ void qtractorTrack::setRecord ( bool bRecord )
 
 	m_pRecordObserver->setValue(bRecord ? 1.0f : 0.0f);
 
-	if (m_pSession->isRecording())
-		m_pSession->trackRecord(this, bRecord);
+	if (m_pSession->isRecording()) {
+		unsigned long iClipStart = m_pSession->playHead();
+		if (m_pSession->isPunching()) {
+			unsigned long iPunchIn = m_pSession->punchIn();
+			if (iClipStart < iPunchIn)
+				iClipStart = iPunchIn;
+		}
+		m_pSession->trackRecord(this, bRecord, iClipStart);
+	}
 }
 
 bool qtractorTrack::isRecord (void) const
 {
-	return (m_pRecordSubject->value() > 0.0f);
+	return m_props.record; // (m_pRecordSubject->value() > 0.0f);
 }
 
 
@@ -504,7 +629,7 @@ void qtractorTrack::setMute ( bool bMute )
 	if (m_pSession->isPlaying() && bMute)
 		m_pSession->trackMute(this, bMute);
 
-	const bool bOldMute = isMute();
+	const bool bOldMute = m_props.mute; // isMute();
 	if ((bOldMute && !bMute) || (!bOldMute && bMute))
 		m_pSession->setMuteTracks(bMute);
 
@@ -518,7 +643,7 @@ void qtractorTrack::setMute ( bool bMute )
 
 bool qtractorTrack::isMute (void) const
 {
-	return (m_pMuteSubject->value() > 0.0f);
+	return m_props.mute; // (m_pMuteSubject->value() > 0.0f);
 }
 
 
@@ -528,7 +653,7 @@ void qtractorTrack::setSolo ( bool bSolo )
 	if (m_pSession->isPlaying() && bSolo)
 		m_pSession->trackSolo(this, bSolo);
 
-	const bool bOldSolo = isSolo();
+	const bool bOldSolo = m_props.solo; // isSolo();
 	if ((bOldSolo && !bSolo) || (!bOldSolo && bSolo))
 		m_pSession->setSoloTracks(bSolo);
 
@@ -542,7 +667,7 @@ void qtractorTrack::setSolo ( bool bSolo )
 
 bool qtractorTrack::isSolo (void) const
 {
-	return (m_pSoloSubject->value() > 0.0f);
+	return m_props.solo; // (m_pSoloSubject->value() > 0.0f);
 }
 
 
@@ -977,6 +1102,16 @@ void qtractorTrack::process_record (
 }
 
 
+// Track special process automation executive.
+void qtractorTrack::process_curve ( unsigned long iFrame )
+{
+	qtractorCurveList *pCurveList = curveList();
+	if (pCurveList && pCurveList->isProcess())
+		pCurveList->process(iFrame);
+}
+
+
+
 // Track paint method.
 void qtractorTrack::drawTrack ( QPainter *pPainter, const QRect& trackRect,
 	unsigned long iTrackStart, unsigned long iTrackEnd, qtractorClip *pClip )
@@ -1000,15 +1135,16 @@ void qtractorTrack::drawTrack ( QPainter *pPainter, const QRect& trackRect,
 				x += m_pSession->pixelFromFrame(iClipStart - iTrackStart);
 			} else {
 				iClipOffset = iTrackStart - iClipStart;
-				x--;	// Give some clip left-border room.
+				--x;	// Give some clip left-border room.
 			}
 			if (iClipEnd < iTrackEnd) {
 				w -= m_pSession->pixelFromFrame(iTrackEnd - iClipEnd);// + 1;
 			} else {
-				w++;	// Give some clip right-border room.
+				++w;	// Give some clip right-border room.
 			}
 			QRect rect(x, y, w - x, h);
 			pClip->drawClip(pPainter, rect, iClipOffset);
+		#if 0
 			// Draw the clip selection...
 			if (pClip->isClipSelected()) {
 				unsigned long iSelectStart = pClip->clipSelectStart();
@@ -1018,19 +1154,23 @@ void qtractorTrack::drawTrack ( QPainter *pPainter, const QRect& trackRect,
 				if (iSelectStart >= iTrackStart) {
 					x += m_pSession->pixelFromFrame(iSelectStart - iTrackStart);
 				} else {
-					x--;	// Give selection some left-border room.
+					--x;	// Give selection some left-border room.
 				}
 				if (iSelectEnd < iTrackEnd) {
 					w -= m_pSession->pixelFromFrame(iTrackEnd - iSelectEnd);
 				} else {
-					w++;	// Give selection some right-border room.
+					++w;	// Give selection some right-border room.
 				}
 				rect.setRect(x, y, w - x, h);
 				pPainter->fillRect(rect, QColor(0, 0, 255, 120));
 			}
+		#endif
 		}
 		pClip = pClip->next();
 	}
+
+	if (m_props.mute || (!m_props.solo && m_pSession->soloTracks()))
+		pPainter->fillRect(trackRect, QColor(0, 0, 0, 60));
 }
 
 
@@ -1096,6 +1236,18 @@ void qtractorTrack::updateClipEditors (void)
 }
 
 
+// Audio buffer ring-cache (playlist) methods.
+qtractorAudioBufferThread *qtractorTrack::syncThread (void)
+{
+	if (m_pSyncThread == NULL) {
+		m_pSyncThread = new qtractorAudioBufferThread();
+		m_pSyncThread->start(QThread::HighPriority);
+	}
+
+	return m_pSyncThread;
+}
+
+
 // Track state (monitor record, mute, solo) button setup.
 qtractorSubject *qtractorTrack::monitorSubject (void) const
 {
@@ -1120,7 +1272,7 @@ qtractorSubject *qtractorTrack::soloSubject (void) const
 
 qtractorMidiControlObserver *qtractorTrack::monitorObserver (void) const
 {
-	return static_cast<qtractorMidiControlObserver *> (m_pMonitorObserver);
+	return m_pMonitorObserver;
 }
 
 qtractorMidiControlObserver *qtractorTrack::recordObserver (void) const
@@ -1266,6 +1418,11 @@ bool qtractorTrack::loadElement (
 			qtractorTrack::loadControllers(&eChild);
 		}
 		else
+		if (eChild.tagName() == "curve-file") {
+			// Load track automation curves...
+			qtractorTrack::loadCurveFile(&eChild, m_pCurveFile);
+		}
+		else
 		// Load clips...
 		if (eChild.tagName() == "clips" && !pDocument->isTemplate()) {
 			for (QDomNode nClip = eChild.firstChild();
@@ -1371,6 +1528,16 @@ bool qtractorTrack::saveElement (
 	qtractorTrack::saveControllers(pDocument, &eControllers);
 	pElement->appendChild(eControllers);
 
+	// Save track automation...
+	qtractorCurveList *pCurveList = qtractorTrack::curveList();
+	if (pCurveList && !pCurveList->isEmpty()) {
+		qtractorCurveFile cfile(pCurveList);
+		QDomElement eCurveFile
+			= pDocument->document()->createElement("curve-file");
+		qtractorTrack::saveCurveFile(pDocument, &eCurveFile, &cfile);
+		pElement->appendChild(eCurveFile);
+	}
+
 	// Clips are not saved when in template mode...
 	if (!pDocument->isTemplate()) {
 		// Save track clips...
@@ -1460,6 +1627,7 @@ void qtractorTrack::saveControllers (
 	if (pMidiControl->isMidiObserverMapped(m_pMonitorObserver)) {
 		qtractorMidiControl::Controller *pController
 			= new qtractorMidiControl::Controller;
+		pController->name = m_pMonitorObserver->subject()->name();
 		pController->index = 0; // 0=MonitorObserver
 		pController->ctype = m_pMonitorObserver->type();
 		pController->channel = m_pMonitorObserver->channel();
@@ -1474,6 +1642,7 @@ void qtractorTrack::saveControllers (
 	if (pMidiControl->isMidiObserverMapped(pPanObserver)) {
 		qtractorMidiControl::Controller *pController
 			= new qtractorMidiControl::Controller;
+		pController->name = m_pMonitorObserver->subject()->name();
 		pController->index = 1; // 1=PanObserver
 		pController->ctype = pPanObserver->type();
 		pController->channel = pPanObserver->channel();
@@ -1488,6 +1657,7 @@ void qtractorTrack::saveControllers (
 	if (pMidiControl->isMidiObserverMapped(pGainObserver)) {
 		qtractorMidiControl::Controller *pController
 			= new qtractorMidiControl::Controller;
+		pController->name = m_pMonitorObserver->subject()->name();
 		pController->index = 2; // 2=GainObserver
 		pController->ctype = pGainObserver->type();
 		pController->channel = pGainObserver->channel();
@@ -1500,6 +1670,7 @@ void qtractorTrack::saveControllers (
 	if (pMidiControl->isMidiObserverMapped(m_pRecordObserver)) {
 		qtractorMidiControl::Controller *pController
 			= new qtractorMidiControl::Controller;
+		pController->name = m_pMonitorObserver->subject()->name();
 		pController->index = 3; // 3=RecordObserver
 		pController->ctype = m_pRecordObserver->type();
 		pController->channel = m_pRecordObserver->channel();
@@ -1512,6 +1683,7 @@ void qtractorTrack::saveControllers (
 	if (pMidiControl->isMidiObserverMapped(m_pMuteObserver)) {
 		qtractorMidiControl::Controller *pController
 			= new qtractorMidiControl::Controller;
+		pController->name = m_pMonitorObserver->subject()->name();
 		pController->index = 4; // 4=MuteObserver
 		pController->ctype = m_pMuteObserver->type();
 		pController->channel = m_pMuteObserver->channel();
@@ -1524,6 +1696,7 @@ void qtractorTrack::saveControllers (
 	if (pMidiControl->isMidiObserverMapped(m_pSoloObserver)) {
 		qtractorMidiControl::Controller *pController
 			= new qtractorMidiControl::Controller;
+		pController->name = m_pMonitorObserver->subject()->name();
 		pController->index = 5; // 5=SoloObserver
 		pController->ctype = m_pSoloObserver->type();
 		pController->channel = m_pSoloObserver->channel();
@@ -1540,7 +1713,7 @@ void qtractorTrack::saveControllers (
 }
 
 
-// Map track state (record, mute, solo) controllers (MIDI).
+// Map track state (monitor, gain, pan, record, mute, solo) controllers (MIDI).
 void qtractorTrack::mapControllers (void)
 {
 	qtractorMidiControl *pMidiControl = qtractorMidiControl::getInstance();
@@ -1599,4 +1772,246 @@ void qtractorTrack::mapControllers (void)
 }
 
 
+// Track automation curve list accessor.
+qtractorCurveList *qtractorTrack::curveList (void) const
+{
+	return (m_pPluginList ? m_pPluginList->curveList() : NULL);
+}
+
+
+// Track automation curve serializer accessor.
+qtractorCurveFile *qtractorTrack::curveFile (void) const
+{
+	return m_pCurveFile;
+}
+
+
+// Track automation current curve accessor.
+qtractorCurve *qtractorTrack::currentCurve (void) const
+{
+	qtractorCurveList *pCurveList = curveList();
+	return (pCurveList ? pCurveList->currentCurve() : NULL);
+}
+
+
+// Load track automation curves (monitor, gain, pan, record, mute, solo).
+void qtractorTrack::loadCurveFile (
+	QDomElement *pElement, qtractorCurveFile *pCurveFile )
+{
+	if (pCurveFile) pCurveFile->load(pElement);
+}
+
+
+// Save track automation curves (monitor, gain, pan, record, mute, solo).
+void qtractorTrack::saveCurveFile ( qtractorDocument *pDocument,
+	QDomElement *pElement, qtractorCurveFile *pCurveFile ) const
+{
+	if (pCurveFile == NULL)
+		return;
+
+	qtractorCurveList *pCurveList = pCurveFile->list();
+	if (pCurveList == NULL)
+		return;
+
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return;
+
+	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+	if (pMainForm == NULL)
+		return;
+
+	qtractorMixer *pMixer = pMainForm->mixer();
+	if (pMixer == NULL)
+		return;
+
+	qtractorMixerStrip *pMixerStrip
+		= pMixer->trackRack()->findStrip(m_pMonitor);
+	if (pMixerStrip == NULL)
+		return;
+
+	pCurveFile->clear();
+	pCurveFile->setBaseDir(pSession->sessionDir());
+
+	qtractorCurve *pCurve;
+
+	pCurve = monitorSubject()->curve();
+	if (pCurve) {
+		qtractorCurveFile::Item *pCurveItem = new qtractorCurveFile::Item;
+		pCurveItem->name = pCurve->subject()->name();
+		pCurveItem->index = 0;	// 0=MonitorSubject
+		pCurveItem->ctype = qtractorMidiEvent::CONTROLLER;
+		pCurveItem->channel = 0;
+		pCurveItem->param = 80;	// 80=General Purpose Button 1 (on/off)
+		pCurveItem->mode = pCurve->mode();
+		pCurveItem->process = pCurve->isProcess();
+		pCurveItem->capture = pCurve->isCapture();
+		pCurveItem->locked = pCurve->isLocked();
+		pCurveItem->logarithmic = pCurve->isLogarithmic();
+		pCurveItem->color = pCurve->color();
+		pCurveItem->subject = pCurve->subject();
+		pCurveFile->addItem(pCurveItem);
+	}
+
+	pCurve = pMixerStrip->meter()->panningSubject()->curve();
+	if (pCurve) {
+		qtractorCurveFile::Item *pCurveItem = new qtractorCurveFile::Item;
+		pCurveItem->name = pCurve->subject()->name();
+		pCurveItem->index = 1;	// 1=PanningSubject
+		pCurveItem->ctype = qtractorMidiEvent::CONTROLLER;
+		pCurveItem->channel = 0;
+		pCurveItem->param = 10;	// 10=Pan Position (coarse)
+		pCurveItem->mode = pCurve->mode();
+		pCurveItem->process = pCurve->isProcess();
+		pCurveItem->capture = pCurve->isCapture();
+		pCurveItem->locked = pCurve->isLocked();
+		pCurveItem->logarithmic = pCurve->isLogarithmic();
+		pCurveItem->color = pCurve->color();
+		pCurveItem->subject = pCurve->subject();
+		pCurveFile->addItem(pCurveItem);
+	}
+
+	pCurve = pMixerStrip->meter()->gainSubject()->curve();
+	if (pCurve) {
+		qtractorCurveFile::Item *pCurveItem = new qtractorCurveFile::Item;
+		pCurveItem->name = pCurve->subject()->name();
+		pCurveItem->index = 2; // 2=GainSubject
+		pCurveItem->ctype = qtractorMidiEvent::CONTROLLER;
+		pCurveItem->channel = 0;
+		pCurveItem->param = 7;	// 7=Volume (coarse)
+		pCurveItem->mode = pCurve->mode();
+		pCurveItem->process = pCurve->isProcess();
+		pCurveItem->capture = pCurve->isCapture();
+		pCurveItem->locked = pCurve->isLocked();
+		pCurveItem->logarithmic = pCurve->isLogarithmic();
+		pCurveItem->color = pCurve->color();
+		pCurveItem->subject = pCurve->subject();
+		pCurveFile->addItem(pCurveItem);
+	}
+
+	pCurve = recordSubject()->curve();
+	if (pCurve) {
+		qtractorCurveFile::Item *pCurveItem = new qtractorCurveFile::Item;
+		pCurveItem->name = pCurve->subject()->name();
+		pCurveItem->index = 3;	// 3=RecordSubject
+		pCurveItem->ctype = qtractorMidiEvent::CONTROLLER;
+		pCurveItem->channel = 0;
+		pCurveItem->param = 81;	// 81=General Purpose Button 2 (on/off)
+		pCurveItem->mode = pCurve->mode();
+		pCurveItem->process = pCurve->isProcess();
+		pCurveItem->capture = pCurve->isCapture();
+		pCurveItem->locked = pCurve->isLocked();
+		pCurveItem->logarithmic = pCurve->isLogarithmic();
+		pCurveItem->color = pCurve->color();
+		pCurveItem->subject = pCurve->subject();
+		pCurveFile->addItem(pCurveItem);
+	}
+
+	pCurve = muteSubject()->curve();
+	if (pCurve) {
+		qtractorCurveFile::Item *pCurveItem = new qtractorCurveFile::Item;
+		pCurveItem->name = pCurve->subject()->name();
+		pCurveItem->index = 4;	// 4=MuteSubject
+		pCurveItem->ctype = qtractorMidiEvent::CONTROLLER;
+		pCurveItem->channel = 0;
+		pCurveItem->param = 82;	// 82=General Purpose Button 3 (on/off)
+		pCurveItem->mode = pCurve->mode();
+		pCurveItem->process = pCurve->isProcess();
+		pCurveItem->capture = pCurve->isCapture();
+		pCurveItem->locked = pCurve->isLocked();
+		pCurveItem->logarithmic = pCurve->isLogarithmic();
+		pCurveItem->color = pCurve->color();
+		pCurveItem->subject = pCurve->subject();
+		pCurveFile->addItem(pCurveItem);
+	}
+
+	pCurve = soloSubject()->curve();
+	if (pCurve) {
+		qtractorCurveFile::Item *pCurveItem = new qtractorCurveFile::Item;
+		pCurveItem->name = pCurve->subject()->name();
+		pCurveItem->index = 5;	// 5=SoloSubject
+		pCurveItem->ctype = qtractorMidiEvent::CONTROLLER;
+		pCurveItem->channel = 0;
+		pCurveItem->param = 83;	// 83=General Purpose Button 4 (on/off)
+		pCurveItem->mode = pCurve->mode();
+		pCurveItem->process = pCurve->isProcess();
+		pCurveItem->capture = pCurve->isCapture();
+		pCurveItem->locked = pCurve->isLocked();
+		pCurveItem->logarithmic = pCurve->isLogarithmic();
+		pCurveItem->color = pCurve->color();
+		pCurveItem->subject = pCurve->subject();
+		pCurveFile->addItem(pCurveItem);
+	}
+
+	if (pCurveFile->isEmpty())
+		return;
+
+	const QString sBaseName(trackName() + "_curve");
+	int iClipNo = (pCurveFile->filename().isEmpty() ? 0 : 1);
+	pCurveFile->setFilename(pSession->createFilePath(sBaseName, "mid", iClipNo));
+
+	pCurveFile->save(pDocument, pElement, pSession->timeScale());
+}
+
+
+// Apply track automation curves (monitor, gain, pan, record, mute, solo).
+void qtractorTrack::applyCurveFile ( qtractorCurveFile *pCurveFile ) const
+{
+	if (pCurveFile == NULL)
+		return;
+	if (pCurveFile->items().isEmpty())
+		return;
+
+	qtractorCurveList *pCurveList = pCurveFile->list();
+	if (pCurveList == NULL)
+		return;
+
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return;
+
+	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+	if (pMainForm == NULL)
+		return;
+
+	qtractorMixer *pMixer = pMainForm->mixer();
+	if (pMixer == NULL)
+		return;
+
+	qtractorMixerStrip *pMixerStrip
+		= pMixer->trackRack()->findStrip(m_pMonitor);
+	if (pMixerStrip == NULL)
+		return;
+
+	pCurveFile->setBaseDir(pSession->sessionDir());
+	
+	QListIterator<qtractorCurveFile::Item *> iter(pCurveFile->items());
+	while (iter.hasNext()) {
+		qtractorCurveFile::Item *pCurveItem = iter.next();
+		switch (pCurveItem->index) {
+		case 0: // 0=MonitorSubject
+			pCurveItem->subject = monitorSubject();
+			break;
+		case 1: // 1=PanSubject
+			pCurveItem->subject = pMixerStrip->meter()->panningSubject();
+			break;
+		case 2: // 2=GainSubject
+			pCurveItem->subject = pMixerStrip->meter()->gainSubject();
+			break;
+		case 3: // 3=RecordSubject
+			pCurveItem->subject = recordSubject();
+			break;
+		case 4: // 4=MuteSubject
+			pCurveItem->subject = muteSubject();
+			break;
+		case 5: // 5=SoloSubject
+			pCurveItem->subject = soloSubject();
+			break;
+		}
+	}
+
+	pCurveFile->apply(pSession->timeScale());
+}
+
+
 // end of qtractorTrack.h
