@@ -40,6 +40,10 @@ class qtractorBus;
 
 class qtractorSubject;
 class qtractorMidiControlObserver;
+class qtractorAudioBufferThread;
+class qtractorCurveList;
+class qtractorCurveFile;
+class qtractorCurve;
 
 // Special forward declarations.
 class QDomElement;
@@ -192,6 +196,9 @@ public:
 	void process_record(
 		unsigned long iFrameStart, unsigned long iFrameEnd);
 
+	// Track special process automation executive.
+	void process_curve(unsigned long iFrame);
+
 	// Track paint method.
 	void drawTrack(QPainter *pPainter, const QRect& trackRect,
 		unsigned long iTrackStart, unsigned long iTrackEnd,
@@ -206,6 +213,9 @@ public:
 	// Update all clips editors.
 	void updateClipEditors();
 
+	// Audio buffer ring-cache (playlist) methods.
+	qtractorAudioBufferThread *syncThread();
+
 	// Track state (monitor, record, mute, solo) button setup.
 	qtractorSubject *monitorSubject() const;
 	qtractorSubject *recordSubject() const;
@@ -227,13 +237,29 @@ public:
 	bool loadElement(qtractorDocument *pDocument, QDomElement *pElement);
 	bool saveElement(qtractorDocument *pDocument, QDomElement *pElement) const;
 
-	// Load/save track state (record, mute, soloe) controllers (MIDI).
+	// Load/save track state (record, mute, solo) controllers (MIDI).
 	void loadControllers(QDomElement *pElement);
 	void saveControllers(qtractorDocument *pDocument, QDomElement *pElement) const;
 
 	// Map track state (record, mute, solo) controllers (MIDI).
 	void mapControllers();
 
+	// Track automation curve list accessor.
+	qtractorCurveList *curveList() const;
+
+	// Track automation curve serializer accessor.
+	qtractorCurveFile *curveFile() const;
+
+	// Track automation current curve accessor.
+	qtractorCurve *currentCurve() const;
+
+	// Track automation curve serialization methods.
+	static void loadCurveFile(
+		QDomElement *pElement, qtractorCurveFile *pCurveFile);
+	void saveCurveFile(qtractorDocument *pDocument,
+		QDomElement *pElement, qtractorCurveFile *pCurveFile) const;
+	void applyCurveFile(qtractorCurveFile *pCurveFile) const;
+
 	// Track properties structure.
 	struct Properties
 	{
@@ -295,15 +321,25 @@ private:
 
 	qtractorList<qtractorClip> m_clips; // List of clips.
 
-	qtractorClip *m_pClipRecord;    // Current clip on record (capture).
+	qtractorClip *m_pClipRecord;        // Current clip on record (capture).
 
 	qtractorPluginList *m_pPluginList;	// Plugin chain (audio).
 
+	// Audio buffer ring-cache (playlist).
+	qtractorAudioBufferThread *m_pSyncThread;
+
+	// MIDI track/channel (volume, panning) observers.
+	class MidiVolumeObserver;
+	class MidiPanningObserver;
+
+	MidiVolumeObserver  *m_pMidiVolumeObserver;
+	MidiPanningObserver *m_pMidiPanningObserver;
+
 	// State (monitor, record, mute, solo) observer stuff.
-	class MonitorObserver;
+	qtractorMidiControlObserver *m_pMonitorObserver;
+
 	class StateObserver;
 
-	MonitorObserver *m_pMonitorObserver;
 	StateObserver   *m_pRecordObserver;
 	StateObserver   *m_pMuteObserver;
 	StateObserver   *m_pSoloObserver;
@@ -314,6 +350,8 @@ private:
 	qtractorSubject *m_pSoloSubject;
 
 	qtractorMidiControl::Controllers m_controllers;
+
+	qtractorCurveFile *m_pCurveFile;
 };
 
 
