@@ -24,6 +24,9 @@
 
 #include "qtractorMidiControlObserverForm.h"
 
+#include "qtractorMidiControlObserver.h"
+
+
 #if QT_VERSION < 0x040300
 #define lighter(x)	light(x)
 #define darker(x)	dark(x)
@@ -45,43 +48,29 @@ qtractorMidiControlButton::qtractorMidiControlButton ( QWidget *pParent )
 
 
 // MIDI controller/observer attachment (context menu) activator.
-//
-Q_DECLARE_METATYPE(qtractorMidiControlObserver *);
-
 void qtractorMidiControlButton::addMidiControlAction (
 	qtractorMidiControlObserver *pMidiObserver )
 {
 	if (m_pMidiControlAction)
-		QPushButton::removeAction(m_pMidiControlAction);
-		
-	m_pMidiControlAction = new QAction(
-		QIcon(":/images/itemControllers.png"),
-		tr("MIDI Controller..."), this);
+		removeAction(m_pMidiControlAction);
 
-	m_pMidiControlAction->setData(
-		qVariantFromValue<qtractorMidiControlObserver *> (pMidiObserver));
-
-	QObject::connect(m_pMidiControlAction,
-		SIGNAL(triggered(bool)),
-		SLOT(midiControlActionSlot()));
-
-	QPushButton::addAction(m_pMidiControlAction);
-	QPushButton::setContextMenuPolicy(Qt::ActionsContextMenu);
+	m_pMidiControlAction
+		= qtractorMidiControlObserverForm::addMidiControlAction(
+			this, this, pMidiObserver);
 }
 
 
 void qtractorMidiControlButton::midiControlActionSlot (void)
 {
-	QAction *pAction = qobject_cast<QAction *> (sender());
-	if (pAction && pAction == m_pMidiControlAction) {
-		qtractorMidiControlObserver *pMidiObserver
-			= qVariantValue<qtractorMidiControlObserver *> (pAction->data());
-		if (pMidiObserver) {
-			qtractorMidiControlObserverForm form(parentWidget());
-			form.setMidiObserver(pMidiObserver);
-			form.exec();
-		}
-	}
+	qtractorMidiControlObserverForm::midiControlAction(
+		this, qobject_cast<QAction *> (sender()));
+}
+
+
+void qtractorMidiControlButton::midiControlMenuSlot ( const QPoint& pos )
+{
+	qtractorMidiControlObserverForm::midiControlMenu(
+		qobject_cast<QWidget *> (sender()), pos);
 }
 
 
@@ -131,7 +120,7 @@ qtractorTrackButton::qtractorTrackButton ( qtractorTrack *pTrack,
 // Visitors overload.
 void qtractorTrackButton::updateValue ( float fValue )
 {
-	m_iUpdate++;
+	++m_iUpdate;
 
 	bool bOn = (fValue > 0.0f);
 
@@ -142,7 +131,7 @@ void qtractorTrackButton::updateValue ( float fValue )
 
 	QPushButton::setChecked(bOn);
 
-	m_iUpdate--;
+	--m_iUpdate;
 }
 
 
@@ -181,18 +170,32 @@ qtractorTrack::ToolType qtractorTrackButton::toolType (void) const
 // Track state (record, mute, solo) button setup.
 void qtractorTrackButton::updateTrack (void)
 {
+	qtractorMidiControlObserver *pMidiObserver;
+
 	switch (m_toolType) {
 	case qtractorTrack::Record:
 		setSubject(m_pTrack->recordSubject());
-		addMidiControlAction(m_pTrack->recordObserver());
+		pMidiObserver = m_pTrack->recordObserver();
+		if (pMidiObserver) {
+			pMidiObserver->setCurveList(m_pTrack->curveList());
+			addMidiControlAction(pMidiObserver);
+		}
 		break;
 	case qtractorTrack::Mute:
 		setSubject(m_pTrack->muteSubject());
-		addMidiControlAction(m_pTrack->muteObserver());
+		pMidiObserver = m_pTrack->muteObserver();
+		if (pMidiObserver) {
+			pMidiObserver->setCurveList(m_pTrack->curveList());
+			addMidiControlAction(pMidiObserver);
+		}
 		break;
 	case qtractorTrack::Solo:
 		setSubject(m_pTrack->soloSubject());
-		addMidiControlAction(m_pTrack->soloObserver());
+		pMidiObserver = m_pTrack->soloObserver();
+		if (pMidiObserver) {
+			pMidiObserver->setCurveList(m_pTrack->curveList());
+			addMidiControlAction(pMidiObserver);
+		}
 		break;
 	}
 
