@@ -248,6 +248,11 @@ bool qtractorMoveTrackCommand::redo (void)
 	// Swap it nice, finally.
 	m_pNextTrack = pNextTrack;
 
+	// Mixer turn...
+	qtractorMixer *pMixer = pMainForm->mixer();
+	if (pMixer)
+		pMixer->updateTracks(true);
+
 	// ATTN: MIDI controller map feedback.
 	qtractorMidiControl *pMidiControl = qtractorMidiControl::getInstance();
 	if (pMidiControl) {
@@ -350,7 +355,7 @@ bool qtractorImportTrackCommand::redo (void)
 		if (!m_pSaveCommand->redo())
 		    bResult = false;
 	}
-	m_iSaveCount++;
+	++m_iSaveCount;
 
 	QListIterator<qtractorAddTrackCommand *> iter(m_trackCommands);
 	while (iter.hasNext()) {
@@ -412,7 +417,7 @@ bool qtractorEditTrackCommand::redo (void)
 	// Howdy, maybe we're already have a name on recording...
 	bool bRecord = m_pTrack->isRecord();
 	if (bRecord)
-		pSession->trackRecord(m_pTrack, false);
+		pSession->trackRecord(m_pTrack, false, 0);
 
 	// Make the track property change...
 	bool bResult = qtractorPropertyCommand<qtractorTrack::Properties>::redo();
@@ -429,8 +434,15 @@ bool qtractorEditTrackCommand::redo (void)
 				.arg(m_pTrack->outputBusName()));
 	}
 	else // Reassign recording...
-	if (bRecord)
-		pSession->trackRecord(m_pTrack, true);
+	if (bRecord) {
+		unsigned long iClipStart = pSession->playHead();
+		if (pSession->isPunching()) {
+			unsigned long iPunchIn = pSession->punchIn();
+			if (iClipStart < iPunchIn)
+				iClipStart = iPunchIn;
+		}
+		pSession->trackRecord(m_pTrack, true, iClipStart);
+	}
 
 	// Refresh track item, at least the names...
 	pTracks->trackList()->updateTrack(m_pTrack);
@@ -514,7 +526,7 @@ qtractorTrackStateCommand::qtractorTrackStateCommand ( qtractorTrack *pTrack,
 		}
 	}
 
-	setRefresh(false);
+	setRefresh(m_toolType != qtractorTrack::Record);
 }
 
 // Destructor.
@@ -556,7 +568,9 @@ bool qtractorTrackStateCommand::redo (void)
 			&& m_pClipCommand == NULL && m_iRecordCount == 0) {
 			m_pClipCommand = new qtractorClipCommand(QString());
 			// Do all the record stuffing here...
-			if (m_pClipCommand->addClipRecord(pTrack)) {
+			unsigned long iClipEnd = (pSession->isPunching()
+				? pSession->punchOut() : pSession->framePos());
+			if (m_pClipCommand->addClipRecord(pTrack, iClipEnd)) {
 				// Yes, we've recorded something...
 				setRefresh(true);
 			} else {
@@ -568,7 +582,7 @@ bool qtractorTrackStateCommand::redo (void)
 		// Was it before (skip undos)?
 		if (m_pClipCommand && (m_iRecordCount % 2) == 0)
 			m_pClipCommand->redo();
-		m_iRecordCount++;
+		++m_iRecordCount;
 		// Carry on...
 		pTrack->setRecord(m_bOn);
 		break;
@@ -612,8 +626,6 @@ bool qtractorTrackStateCommand::redo (void)
 	// Toggle/update all other?
 	if (!m_tracks.isEmpty()) {
 		// Exclusive mode.
-		qtractorTrackList *pTrackList = pMainForm->tracks()->trackList();
-		qtractorMidiEngine *pMidiEngine = pSession->midiEngine();
 		qtractorMidiControl *pMidiControl = qtractorMidiControl::getInstance();
 		QListIterator<TrackItem *> iter(m_tracks);
 		while (iter.hasNext()) {
@@ -815,7 +827,7 @@ bool qtractorTrackGainCommand::redo (void)
 
 	// Set track gain (respective monitor gets set too...)
 	pTrack->setGain(m_fGain);
-
+#if 0
 	// MIDI tracks are special...
 	if (pTrack->trackType() == qtractorTrack::Midi) {
 		// Gotta make sure we've a proper MIDI bus...
@@ -824,7 +836,7 @@ bool qtractorTrackGainCommand::redo (void)
 		if (pMidiBus)
 			pMidiBus->setVolume(pTrack, m_fGain);
 	}
-
+#endif
 	// Send MIDI controller command(s)...
 	if (midiControlFeedback()) {
 		qtractorMidiControl *pMidiControl = qtractorMidiControl::getInstance();
