@@ -40,6 +40,8 @@
 #include "qtractorMainForm.h"
 #include "qtractorMixer.h"
 
+#include "qtractorCurve.h"
+
 #include <QHeaderView>
 
 #include <QApplication>
@@ -61,58 +63,105 @@
 
 
 //----------------------------------------------------------------------------
-// qtractorTrackListHeaderModel -- Track-list header model.
+// qtractorCurveButton -- Track automation curve menu button.
 
-class qtractorTrackListHeaderModel : public QAbstractListModel
+class qtractorCurveButton : public QPushButton
 {
+
 public:
 
 	// Constructor.
-	qtractorTrackListHeaderModel(QObject *pParent = 0)
-		: QAbstractListModel(pParent)
+	qtractorCurveButton(qtractorTrack *pTrack, QWidget *pParent)
+		: QPushButton(pParent), m_pTrack(pTrack)
+		{ QPushButton::setFocusPolicy(Qt::NoFocus); }
+
+	// Button state updater.
+	void updateTrack()
 	{
-		m_headerText
-			<< tr("Nr")
-			<< tr("Track Name")
-			<< tr("Bus")
-			<< tr("Ch")
-			<< tr("Patch")
-			<< tr("Instrument");
-	};
-
-	int rowCount(const QModelIndex&) const
-		{ return 0; }
-
-	int columnCount(const QModelIndex&) const
-		{ return m_headerText.count(); }
-
-	QVariant headerData(int section, Qt::Orientation orient, int role) const
-	{
-		if (orient == Qt::Horizontal) {
-			switch (role) {
-			case Qt::DisplayRole:
-				return m_headerText.at(section);
-			case Qt::TextAlignmentRole:
-				if (section == qtractorTrackList::Number ||
-					section == qtractorTrackList::Channel)
-					return int(Qt::AlignHCenter | Qt::AlignVCenter);
-				else
-					return int(Qt::AlignLeft | Qt::AlignVCenter);
+		qtractorCurveList *pCurveList = m_pTrack->curveList();
+		if (pCurveList) {
+			QPalette pal;
+			if (pCurveList->isCapture()) {
+				pal.setColor(QPalette::Button, Qt::darkRed);
+				pal.setColor(QPalette::ButtonText, Qt::red);
 			}
-		}	
-		return QVariant();
+			else
+			if (pCurveList->isProcess()) {
+				pal.setColor(QPalette::Button, Qt::darkGreen);
+				pal.setColor(QPalette::ButtonText, Qt::green);
+			}
+			QPushButton::setPalette(pal);
+			QString sToolTip(tr("Automation (%1)"));
+			qtractorSubject *pSubject = NULL;
+			qtractorCurve *pCurrentCurve = pCurveList->currentCurve();
+			if (pCurrentCurve)
+				pSubject = pCurrentCurve->subject();
+			if (pSubject)
+				QPushButton::setToolTip(sToolTip.arg(pSubject->name()));
+			else
+				QPushButton::setToolTip(sToolTip.arg(tr("none")));
+		}
 	}
 
-	QVariant data(const QModelIndex&, int) const
-		{ return QVariant(); }
+protected:
+
+	// Virtual trap to set current track
+	// before showing the automation menu...
+	bool hitButton(const QPoint& pos) const
+	{
+		qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+		if (pMainForm) {
+			qtractorTracks *pTracks = pMainForm->tracks();
+			if (pTracks)
+				pTracks->trackList()->setCurrentTrack(m_pTrack);
+		}
+
+		return QPushButton::hitButton(pos);
+	}
 
 private:
 
-	// Model variables.
-	QStringList m_headerText;
+	// Instance variables.
+	qtractorTrack *m_pTrack;
 };
 
 
+//----------------------------------------------------------------------------
+// qtractorTrackListHeaderModel -- Track-list header model.
+
+// Constructor.
+qtractorTrackListHeaderModel::qtractorTrackListHeaderModel ( QObject *pParent )
+	: QAbstractListModel(pParent)
+{
+	m_headerText
+		<< tr("Nr")
+		<< tr("Track Name")
+		<< tr("Bus")
+		<< tr("Ch")
+		<< tr("Patch")
+		<< tr("Instrument");
+};
+
+QVariant qtractorTrackListHeaderModel::headerData (
+	int section, Qt::Orientation orient, int role ) const
+{
+	if (orient == Qt::Horizontal) {
+		switch (role) {
+		case Qt::DisplayRole:
+			return m_headerText.at(section);
+		case Qt::TextAlignmentRole:
+			if (section == qtractorTrackList::Number ||
+				section == qtractorTrackList::Channel)
+				return int(Qt::AlignHCenter | Qt::AlignVCenter);
+			else
+				return int(Qt::AlignLeft | Qt::AlignVCenter);
+		}
+	}	
+
+	return QVariant();
+}
+
+
 //----------------------------------------------------------------------------
 // qtractorTrackItemWidget -- Track button layout widget.
 
@@ -135,7 +184,18 @@ qtractorTrackItemWidget::qtractorTrackItemWidget (
 	m_pSoloButton   = new qtractorTrackButton(pTrack,
 		qtractorTrack::Solo, buttonSize, this);
 
+	m_pCurveButton = new qtractorCurveButton(pTrack, this);
+	m_pCurveButton->setFixedSize(QSize(32, 16));
+	m_pCurveButton->setFont(m_pRecordButton->font());
+	m_pCurveButton->setText("A");
+	m_pCurveButton->setToolTip(tr("Automation"));
+
+	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+	if (pMainForm)
+		m_pCurveButton->setMenu(pMainForm->trackCurveMenu());
+
 //	pHBoxLayout->addStretch();
+	pHBoxLayout->addWidget(m_pCurveButton);
 	pHBoxLayout->addWidget(m_pRecordButton);
 	pHBoxLayout->addWidget(m_pMuteButton);
 	pHBoxLayout->addWidget(m_pSoloButton);
@@ -160,6 +220,8 @@ qtractorTrackList::qtractorTrackList ( qtractorTracks *pTracks, QWidget *pParent
 
 	m_pRubberBand = NULL;
 
+	m_iUpdateContents = 0;
+
 	m_pPixmap[IconAudio] = new QPixmap(":/images/trackAudio.png");
 	m_pPixmap[IconMidi]  = new QPixmap(":/images/trackMidi.png");
 
@@ -363,6 +425,8 @@ void qtractorTrackList::Item::update ( qtractorTrackList *pTrackList )
 			break;
 		}
 	}
+
+	widget->curveButton()->updateTrack();
 }
 
 
@@ -374,7 +438,7 @@ int qtractorTrackList::trackRow ( qtractorTrack *pTrack ) const
 	while (iter.hasNext()) {
 		if (pTrack == iter.next()->track)
 			return iTrack;
-		iTrack++;
+		++iTrack;
 	}
 
 	return -1;
@@ -397,7 +461,7 @@ int qtractorTrackList::trackRowAt ( const QPoint& pos )
 		y2 += (iter.next()->track)->zoomHeight();
 		if (y >= y1 && y < y2)
 			return iTrack;
-		iTrack++;
+		++iTrack;
 	}
 
 	return -1;
@@ -457,7 +521,7 @@ QRect qtractorTrackList::trackRect ( int iTrack ) const
 				rect.setHeight(y2 - y1);
 				break;
 			}
-			iTrack--;
+			--iTrack;
 		}
 	}
 
@@ -528,7 +592,12 @@ int qtractorTrackList::trackRowCount (void) const
 }
 
 
-// Retrieves current selected track reference.
+// Current selected track reference.
+void qtractorTrackList::setCurrentTrack ( qtractorTrack *pTrack )
+{
+	setCurrentTrackRow(trackRow(pTrack));
+}
+
 qtractorTrack *qtractorTrackList::currentTrack (void) const
 {
 	if (m_iCurrentTrack < 0 || m_iCurrentTrack >= m_items.count())
@@ -541,18 +610,19 @@ qtractorTrack *qtractorTrackList::currentTrack (void) const
 // Find the list view item from track pointer reference.
 void qtractorTrackList::updateTrack ( qtractorTrack *pTrack )
 {
-	int iTrack = 0;
 	QListIterator<Item *> iter(m_items);
 	while (iter.hasNext()) {
 		Item *pItem = iter.next();
-		if (pTrack == pItem->track) {
+		if (pTrack == NULL || pTrack == pItem->track) {
 			// Force update the data...
 			pItem->update(this);
-			updateContents();
-			break;
+			// Specific bail out...
+			if (pTrack)
+				break;
 		}
-		iTrack++;
 	}
+
+	updateContents();
 }
 
 
@@ -601,20 +671,34 @@ void qtractorTrackList::updateContentsHeight (void)
 // Rectangular contents update.
 void qtractorTrackList::updateContents ( const QRect& rect )
 {
+	if (m_iUpdateContents > 0)
+		return;
+
+	++m_iUpdateContents;
+
 	updatePixmap(
 		qtractorScrollView::contentsX(), qtractorScrollView::contentsY());
 
 	qtractorScrollView::updateContents(rect);
+	
+	--m_iUpdateContents;
 }
 
 
 // Overall contents update.
 void qtractorTrackList::updateContents (void)
 {
+	if (m_iUpdateContents > 0)
+		return;
+
+	++m_iUpdateContents;
+
 	updatePixmap(
 		qtractorScrollView::contentsX(), qtractorScrollView::contentsY());
 
 	qtractorScrollView::updateContents();
+
+	--m_iUpdateContents;
 }
 
 
@@ -732,7 +816,7 @@ void qtractorTrackList::updatePixmap ( int cx, int cy )
 	m_pixmap.fill(pal.window().color());
 
 	QPainter painter(&m_pixmap);
-	painter.initFrom(this);
+	painter.initFrom(pViewport);
 
 	// Update actual contents size...
 	m_pHeader->setOffset(cx);
@@ -782,7 +866,7 @@ void qtractorTrackList::updatePixmap ( int cx, int cy )
 			}
 		}
 		else if (pItem->widget) (pItem->widget)->hide();
-		iTrack++;
+		++iTrack;
 	}
 
 	if (cy + h > ch) {
