@@ -26,6 +26,7 @@
 #include "qtractorRubberBand.h"
 
 #include "qtractorTrack.h"
+#include "qtractorCurve.h"
 
 #include <QPixmap>
 
@@ -37,6 +38,8 @@ class qtractorMidiSequence;
 class qtractorSessionCursor;
 class qtractorTrackListItem;
 
+class qtractorCurveEditCommand;
+
 class QToolButton;
 
 class QDragEnterEvent;
@@ -110,11 +113,16 @@ public:
 	void selectTrack(qtractorTrack *pTrackPtr, bool bReset = true);
 	// Select all contents.
 	void selectAll(bool bSelect = true);
+	// Invert selection on all tracks and clips.
+	void selectInvert();
 
 	// Select all clips of given filename and track/channel.
 	void selectFile(qtractorTrack::TrackType trackType,
 		const QString& sFilename, int iTrackChannel, bool bSelect);
 
+	// Contents update overloaded methods.
+	void updateRect(const QRect& rect);
+
 	// Whether there's any clip currently editable.
 	qtractorClip *currentClip() const;
 
@@ -186,6 +194,10 @@ public:
 	void setToolTips(bool bToolTips);
 	bool isToolTips() const;
 
+	// Automation curve node editing mode.
+	void setCurveEdit(bool bCurveEdit);
+	bool isCurveEdit() const;
+
 protected:
 
 	// Resize event handler.
@@ -197,9 +209,18 @@ protected:
 	// Get track from given contents vertical position.
 	qtractorTrack *trackAt(const QPoint& pos, bool bSelectTrack = false,
 		qtractorTrackViewInfo *pTrackViewInfo = NULL) const;
+
 	// Get clip from given contents position.
 	qtractorClip *clipAt(const QPoint& pos, bool bSelectTrack = false,
 		QRect *pClipRect = NULL) const;
+	// Get clip from given contents position.
+	qtractorClip *clipAtTrack(const QPoint& pos, QRect *pClipRect,
+		qtractorTrack *pTrack, qtractorTrackViewInfo *pTrackViewInfo) const;
+
+	// Get automation curve node from given contents position.
+	qtractorCurve::Node *nodeAtTrack(const QPoint& pos,
+		qtractorTrack *pTrack, qtractorTrackViewInfo *pTrackViewInfo) const;
+	qtractorCurve::Node *nodeAt(const QPoint& pos) const;
 
 	// Get contents visible rectangle from given track.
 	bool trackInfo(qtractorTrack *pTrackPtr,
@@ -255,9 +276,6 @@ protected:
 	// Show selection tooltip...
 	void showToolTip(const QRect& rect, int dx) const;
 
-	// Update single track clip selection.
-	void updateSingleTrack(int y, int h) const;
-
 	// Draw/hide the whole drop rectagle list
 	void updateDropRects(int y, int h) const;
 	void showDropRects() const;
@@ -268,7 +286,7 @@ protected:
 		const QRect& rectDrag, int thick = 1) const;
 
 	// Clip fade-in/out handle and resize detection.
-	bool dragFadeResizeStart(const QPoint& pos);
+	bool dragMoveStart(const QPoint& pos);
 
 	// Clip fade-in/out handle drag-move methods.
 	void dragFadeMove(const QPoint& pos);
@@ -278,6 +296,12 @@ protected:
 	void dragResizeMove(const QPoint& pos);
 	void dragResizeDrop(const QPoint& pos, bool bTimeStretch = false);
 
+	// Automation curve node drag-move methods.
+	void dragCurveNodeMove(const QPoint& pos, bool bAddNode = false);
+
+	// Common tool-tip builder for automation nodes.
+	QString nodeToolTip(qtractorCurve *pCurve, qtractorCurve::Node *pNode) const;
+
 	// Reset drag/select/move state.
 	void resetDragState();
 
@@ -347,7 +371,7 @@ private:
 		DragNone = 0, DragStart, DragSelect,
 		DragMove, DragDrop, DragDropPaste, DragStep,
 		DragPaste, DragFadeIn, DragFadeOut,
-		DragResizeLeft, DragResizeRight
+		DragResizeLeft, DragResizeRight, DragCurveNode
 	} m_dragState, m_dragCursor;
 
 	qtractorClip *m_pClipDrag;
@@ -373,6 +397,9 @@ private:
 	// Floating tool-tips mode.
 	bool m_bToolTips;
 
+	// Automation curve node editing mode.
+	bool m_bCurveEdit;
+
 	// The local clipboard item.
 	struct ClipItem
 	{
@@ -421,12 +448,23 @@ private:
 	int m_iEditHeadX;
 	int m_iEditTailX;
 
-	// Recording state window.
+	// Record rolling update width.
 	int m_iLastRecordX;
 
 	// Paste interim parameters.
 	unsigned short m_iPasteCount;
 	unsigned long  m_iPastePeriod;
+
+	// Single track dragging interim parameters.
+	bool m_bDragSingleTrack;
+	int  m_iDragSingleTrackY;
+	int  m_iDragSingleTrackHeight;
+
+	// Automation curve editing node.
+	qtractorCurve       *m_pDragCurve;
+	qtractorCurve::Node *m_pDragCurveNode;
+
+	qtractorCurveEditCommand *m_pCurveEditCommand;
 };
 
 
