@@ -216,22 +216,28 @@ int qtractorTracks::zoomMode (void) const
 // Zoom view actuators.
 void qtractorTracks::zoomIn (void)
 {
+	ZoomCenter zc;
+	zoomCenterPre(zc);
+
 	if (m_iZoomMode & ZoomHorizontal)
 		horizontalZoomStep(+ ZoomStep);
 	if (m_iZoomMode & ZoomVertical)
 		verticalZoomStep(+ ZoomStep);
 
-	centerContents();
+	zoomCenterPost(zc);
 }
 
 void qtractorTracks::zoomOut (void)
 {
+	ZoomCenter zc;
+	zoomCenterPre(zc);
+
 	if (m_iZoomMode & ZoomHorizontal)
 		horizontalZoomStep(- ZoomStep);
 	if (m_iZoomMode & ZoomVertical)
 		verticalZoomStep(- ZoomStep);
 
-	centerContents();
+	zoomCenterPost(zc);
 }
 
 
@@ -241,37 +247,52 @@ void qtractorTracks::zoomReset (void)
 	if (pSession == NULL)
 		return;
 
+	ZoomCenter zc;
+	zoomCenterPre(zc);
+
 	if (m_iZoomMode & ZoomHorizontal)
 		horizontalZoomStep(ZoomBase - pSession->horizontalZoom());
 	if (m_iZoomMode & ZoomVertical)
 		verticalZoomStep(ZoomBase - pSession->verticalZoom());
 
-	centerContents();
+	zoomCenterPost(zc);
 }
 
 
 void qtractorTracks::horizontalZoomInSlot (void)
 {
+	ZoomCenter zc;
+	zoomCenterPre(zc);
+
 	horizontalZoomStep(+ ZoomStep);
-	centerContents();
+	zoomCenterPost(zc);
 }
 
 void qtractorTracks::horizontalZoomOutSlot (void)
 {
+	ZoomCenter zc;
+	zoomCenterPre(zc);
+
 	horizontalZoomStep(- ZoomStep);
-	centerContents();
+	zoomCenterPost(zc);
 }
 
 void qtractorTracks::verticalZoomInSlot (void)
 {
+	ZoomCenter zc;
+	zoomCenterPre(zc);
+
 	verticalZoomStep(+ ZoomStep);
-	centerContents();
+	zoomCenterPost(zc);
 }
 
 void qtractorTracks::verticalZoomOutSlot (void)
 {
+	ZoomCenter zc;
+	zoomCenterPre(zc);
+
 	verticalZoomStep(- ZoomStep);
-	centerContents();
+	zoomCenterPost(zc);
 }
 
 void qtractorTracks::viewZoomResetSlot (void)
@@ -280,29 +301,63 @@ void qtractorTracks::viewZoomResetSlot (void)
 	if (pSession == NULL)
 		return;
 
+	ZoomCenter zc;
+	zoomCenterPre(zc);
+
 	// All zoom base are belong to us :)
 	verticalZoomStep(ZoomBase - pSession->verticalZoom());
 	horizontalZoomStep(ZoomBase - pSession->horizontalZoom());
-	centerContents();
+	zoomCenterPost(zc);
 }
 
 
-// Try to center horizontally/vertically
-// (usually after zoom change)
-void qtractorTracks::centerContents (void)
+// Zoom centering prepare method.
+// (usually before zoom change)
+void qtractorTracks::zoomCenterPre ( ZoomCenter& zc ) const
 {
 	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return;
-		
-	// Get current session frame location...
-	unsigned long iFrame = m_pTrackView->sessionCursor()->frame();
+
+	QWidget *pViewport = m_pTrackView->viewport();
+	const QRect& rect = pViewport->rect();
+	const QPoint& pos = pViewport->mapFromGlobal(QCursor::pos());
+	if (rect.contains(pos)) {
+		zc.x = pos.x();
+		zc.y = 0; // pos.y();
+	} else {
+	#if 0
+		zc.x = 0;
+		zc.y = 0;
+	#else
+		zc.x = (rect.width() >> 1);
+		zc.y = 0; // (rect.height() >> 1);
+	#endif
+	}
+
+	int cx = m_pTrackView->contentsX();
+	zc.frame = pSession->frameFromPixel(cx + zc.x);
+}
+
+
+// Zoom centering post methods.
+// (usually after zoom change)
+void qtractorTracks::zoomCenterPost ( const ZoomCenter& zc )
+{
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return;
+
+	int cx = pSession->pixelFromFrame(zc.frame);
+	int cy = m_pTrackView->contentsY();
+
+	if (cx > zc.x) cx -= zc.x; else cx = 0;
+	if (cy > zc.y) cy -= zc.y; else cy = 0;
 
 	// Update the dependant views...
 	m_pTrackList->updateContentsHeight();
 	m_pTrackView->updateContentsWidth();
-	m_pTrackView->setContentsPos(
-		pSession->pixelFromFrame(iFrame), m_pTrackView->contentsY());
+	m_pTrackView->setContentsPos(cx, cy);
 	m_pTrackView->updateContents();
 
 	// Make its due...
@@ -324,17 +379,17 @@ void qtractorTracks::updateContents ( bool bRefresh )
 	// Update/sync from session tracks.
 	int iRefresh = 0;
 	if (bRefresh)
-		iRefresh++;
+		++iRefresh;
 	int iTrack = 0;
 	qtractorTrack *pTrack = pSession->tracks().first();
 	while (pTrack) {
 		// Check if item is already on list
 		if (m_pTrackList->trackRow(pTrack) < 0) {
 			m_pTrackList->insertTrack(iTrack, pTrack);
-			iRefresh++;
+			++iRefresh;
 		}
 		pTrack = pTrack->next();
-		iTrack++;
+		++iTrack;
 	}
 
 	// Update dependant views.
@@ -423,8 +478,8 @@ bool qtractorTracks::newClip (void)
 				pSession->ticksPerBeat() * pSession->beatsPerBar()));
 		}
 		// Create a clip filename from scratch...
-		const QString& sFilename = pSession->createFilePath(
-			pTrack->trackName(), pTrack->clips().count(), "mid");
+		const QString& sFilename
+			= pSession->createFilePath(pTrack->trackName(), "mid");
 		pClip->setFilename(sFilename);
 		pClip->setClipName(QFileInfo(sFilename).baseName());
 		// Proceed to setup the MDII clip properly...
@@ -446,6 +501,15 @@ bool qtractorTracks::newClip (void)
 			}
 			// Make it a brand new revision...
 			pMidiClip->setRevision(1);
+			// Save/replace the clip track...
+			qtractorMidiFile::saveCopyFile(sFilename,
+				pMidiClip->filename(),
+				pMidiClip->trackChannel(),
+				pMidiClip->format(),
+				pMidiClip->sequence(),
+				pSession->timeScale(),
+				pSession->tickFromFrame(pMidiClip->clipStart()));
+			pMainForm->addMidiFile(sFilename);
 			// Insert the clip right away...
 			qtractorClipCommand *pClipCommand
 				= new qtractorClipCommand(tr("new clip"));
@@ -597,12 +661,16 @@ bool qtractorTracks::normalizeClip ( qtractorClip *pClip )
 	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
 
 	// Multiple clip selection...
-	qtractorClipSelect *pClipSelect = m_pTrackView->clipSelect();
-	if (pClipSelect->items().count() > 0) {
+	if (isClipSelected()) {
+		qtractorClipSelect *pClipSelect = m_pTrackView->clipSelect();
 		const qtractorClipSelect::ItemList& items = pClipSelect->items();
 		qtractorClipSelect::ItemList::ConstIterator iter = items.constBegin();
-		for ( ; iter != items.constEnd(); ++iter)
-			normalizeClipCommand(pClipCommand, iter.key());
+		for ( ; iter != items.constEnd(); ++iter) {
+			// Make sure it's legal selection...
+			pClip = iter.key();
+			if (pClip->track() && pClip->isClipSelected())
+				normalizeClipCommand(pClipCommand, pClip);
+		}
 	}	// Single, current clip instead?
 	else normalizeClipCommand(pClipCommand, pClip);
 
@@ -720,12 +788,16 @@ bool qtractorTracks::executeClipTool ( int iTool, qtractorClip *pClip )
 	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
 
 	// Multiple clip selection...
-	qtractorClipSelect *pClipSelect = m_pTrackView->clipSelect();
-	if (pClipSelect->items().count() > 0) {
+	if (isClipSelected()) {
+		qtractorClipSelect *pClipSelect = m_pTrackView->clipSelect();
 		const qtractorClipSelect::ItemList& items = pClipSelect->items();
 		qtractorClipSelect::ItemList::ConstIterator iter = items.constBegin();
-		for ( ; iter != items.constEnd(); ++iter)
-			executeClipToolCommand(pMidiClipCommand, iter.key(), &toolsForm);
+		for ( ; iter != items.constEnd(); ++iter) {
+			// Make sure it's legal selection...
+			qtractorClip *pClip = iter.key();
+			if (pClip->track() && pClip->isClipSelected())
+				executeClipToolCommand(pMidiClipCommand, pClip, &toolsForm);
+		}
 	}	// Single, current clip instead?
 	else executeClipToolCommand(pMidiClipCommand, pClip, &toolsForm);
 
@@ -881,7 +953,7 @@ bool qtractorTracks::importClips (
 				// Depending of SMF format...
 				int iTrackChannel = pTrack->midiChannel();
 				if (file.format() == 1)
-					iTrackChannel++;
+					++iTrackChannel;
 				// Add the MIDI clip at once...
 				qtractorMidiClip *pMidiClip = new qtractorMidiClip(pTrack);
 				pMidiClip->setFilename(sPath);
@@ -993,16 +1065,18 @@ struct audioClipBufferItem
 {
 	// Constructor.
 	audioClipBufferItem(qtractorClip *pClip,
+		qtractorAudioBufferThread *pSyncThread,
 		unsigned short iChannels,
 		unsigned int iSampleRate)
 		: clip(static_cast<qtractorAudioClip *> (pClip))
 	{
-		buff = new qtractorAudioBuffer(iChannels, iSampleRate);
+		buff = new qtractorAudioBuffer(pSyncThread, iChannels, iSampleRate);
 		buff->setOffset(clip->clipOffset());
 		buff->setLength(clip->clipLength());
 		buff->setTimeStretch(clip->timeStretch());
 		buff->setPitchShift(clip->pitchShift());
 		buff->open(clip->filename());
+		buff->syncExport();
 	}
 
 	// Destructor.
@@ -1045,7 +1119,7 @@ bool qtractorTracks::mergeExportAudioClips ( qtractorClipCommand *pClipCommand )
 #else
 	// Construct save-file dialog...
 	QFileDialog fileDialog(this, sTitle,
-		pSession->createFilePath(pTrack->trackName(), 0, sExt), sFilter);
+		pSession->createFilePath(pTrack->trackName(), sExt), sFilter);
 	// Set proper open-file modes...
 	fileDialog.setAcceptMode(QFileDialog::AcceptSave);
 	fileDialog.setFileMode(QFileDialog::AnyFile);
@@ -1098,18 +1172,26 @@ bool qtractorTracks::mergeExportAudioClips ( qtractorClipCommand *pClipCommand )
 	const qtractorClipSelect::ItemList& items = pClipSelect->items();
 	qtractorClipSelect::ItemList::ConstIterator iter = items.constBegin();
 
+	unsigned short iChannels = pAudioBus->channels();
+	unsigned int iSampleRate = pSession->sampleRate();
+
 	// Multi-selection extents (in frames)...
 	QList<audioClipBufferItem *> list;
 	unsigned long iSelectStart = pSession->sessionLength();
 	unsigned long iSelectEnd = 0;
 	for ( ; iter != items.constEnd(); ++iter) {
 		qtractorClip *pClip = iter.key();
-		if (iSelectStart > pClip->clipSelectStart())
-			iSelectStart = pClip->clipSelectStart();
-		if (iSelectEnd < pClip->clipSelectEnd())
-			iSelectEnd = pClip->clipSelectEnd();
-		list.append(new audioClipBufferItem(
-			pClip, pAudioBus->channels(), pSession->sampleRate()));
+		qtractorTrack *pTrack = pClip->track();
+		// Make sure it's a legal selection...
+		if (pTrack && pClip->isClipSelected()) {
+			qtractorAudioBufferThread *pSyncThread = pTrack->syncThread();
+			if (iSelectStart > pClip->clipSelectStart())
+				iSelectStart = pClip->clipSelectStart();
+			if (iSelectEnd < pClip->clipSelectEnd())
+				iSelectEnd = pClip->clipSelectEnd();
+			list.append(new audioClipBufferItem(
+				pClip, pSyncThread, iChannels, iSampleRate));
+		}
 	}
 
 	// A progress indication might be friendly...
@@ -1124,7 +1206,6 @@ bool qtractorTracks::mergeExportAudioClips ( qtractorClipCommand *pClipCommand )
 
 	// Allocate merge audio scratch buffer...
 	unsigned int iBufferSize = pSession->audioEngine()->bufferSize();
-	unsigned short iChannels = pAudioBus-> channels();
 	unsigned short i;
 	float **ppFrames = new float * [iChannels];
 	for (i = 0; i < iChannels; ++i)
@@ -1167,10 +1248,10 @@ bool qtractorTracks::mergeExportAudioClips ( qtractorClipCommand *pClipCommand )
 			if ((count % 33) == 0) pBuff->syncExport();
 			// Quite similar to qtractorAudioClip::process()...
 			unsigned long iClipStart = pClip->clipStart();
-			unsigned long iClipEnd   = iClipStart + pClip->clipLength();
+			unsigned long iClipEnd   = iClipStart + pClip->clipLength();;
 			if (iFrameStart < iClipStart && iFrameEnd > iClipStart) {
 				unsigned long iOffset = iFrameEnd - iClipStart;
-				while (!pBuff->inSync(0, iOffset))
+				while (!pBuff->inSync(iClipStart - iFrameStart, iOffset))
 					pBuff->syncExport();
 				pBuff->readMix(ppFrames, iOffset,
 					iChannels, iClipStart - iFrameStart,
@@ -1298,7 +1379,7 @@ bool qtractorTracks::mergeExportMidiClips ( qtractorClipCommand *pClipCommand )
 #else
 	// Construct save-file dialog...
 	QFileDialog fileDialog(this, sTitle,
-		pSession->createFilePath(pTrack->trackName(), 0, sExt), sFilter);
+		pSession->createFilePath(pTrack->trackName(), sExt), sFilter);
 	// Set proper open-file modes...
 	fileDialog.setAcceptMode(QFileDialog::AcceptSave);
 	fileDialog.setFileMode(QFileDialog::AnyFile);
@@ -1357,10 +1438,13 @@ bool qtractorTracks::mergeExportMidiClips ( qtractorClipCommand *pClipCommand )
 	unsigned long iSelectEnd = 0;
 	for ( ; iter != items.constEnd(); ++iter) {
 		qtractorClip *pClip = iter.key();
-		if (iSelectStart > pClip->clipSelectStart())
-			iSelectStart = pClip->clipSelectStart();
-		if (iSelectEnd < pClip->clipSelectEnd())
-			iSelectEnd = pClip->clipSelectEnd();
+		// Make sure it's a legal selection...
+		if (pClip->track() && pClip->isClipSelected()) {
+			if (iSelectStart > pClip->clipSelectStart())
+				iSelectStart = pClip->clipSelectStart();
+			if (iSelectEnd < pClip->clipSelectEnd())
+				iSelectEnd = pClip->clipSelectEnd();
+		}
 	}
 
 	// Multi-selection extents (in ticks)...
@@ -1388,63 +1472,66 @@ bool qtractorTracks::mergeExportMidiClips ( qtractorClipCommand *pClipCommand )
 	iter = items.constBegin();
 	for ( ; iter != items.constEnd(); ++iter) {
 		qtractorClip *pClip = iter.key();
-		// Clip parameters.
-		unsigned long iClipStart  = pClip->clipStart();
-		unsigned long iClipOffset = pClip->clipOffset();
-		unsigned long iClipLength = pClip->clipLength();
-		unsigned long iClipEnd    = iClipStart + iClipLength;
-		// Determine and keep clip regions...
-		if (pClipCommand) {
-			if (iSelectStart > iClipStart) {
-				// -- Left clip...
-				pClipCommand->resizeClip(pClip,
-					iClipStart,
-					iClipOffset,
-					iSelectStart - iClipStart);
-				// Done, left clip.
+		// Make sure it's a legal selection...
+		if (pClip->track() && pClip->isClipSelected()) {
+			// Clip parameters.
+			unsigned long iClipStart  = pClip->clipStart();
+			unsigned long iClipOffset = pClip->clipOffset();
+			unsigned long iClipLength = pClip->clipLength();
+			unsigned long iClipEnd    = iClipStart + iClipLength;
+			// Determine and keep clip regions...
+			if (pClipCommand) {
+				if (iSelectStart > iClipStart) {
+					// -- Left clip...
+					pClipCommand->resizeClip(pClip,
+						iClipStart,
+						iClipOffset,
+						iSelectStart - iClipStart);
+					// Done, left clip.
+				}
+				else
+				if (iSelectEnd < iClipEnd) {
+					// -- Right clip...
+					pClipCommand->resizeClip(pClip,
+						iSelectEnd,
+						iClipOffset + (iSelectEnd - iClipStart),
+						iClipEnd - iSelectEnd);
+					// Done, right clip.
+				} else {
+					// -- Inner clip...
+					pClipCommand->removeClip(pClip);
+					// Done, inner clip.
+				}
 			}
-			else
-			if (iSelectEnd < iClipEnd) {
-				// -- Right clip...
-				pClipCommand->resizeClip(pClip,
-					iSelectEnd,
-					iClipOffset + (iSelectEnd - iClipStart),
-					iClipEnd - iSelectEnd);
-				// Done, right clip.
-			} else {
-				// -- Inner clip...
-				pClipCommand->removeClip(pClip);
-				// Done, inner clip.
-			}
-		}
-		// Do the MIDI merge, itself...
-		qtractorMidiClip *pMidiClip
-			= static_cast<qtractorMidiClip *> (pClip);
-		if (pMidiClip) {
-			unsigned long iTimeClip
-				= pSession->tickFromFrame(pClip->clipStart());
-			unsigned long iTimeOffset = iTimeClip - iTimeStart;
-			// For each event...
-			qtractorMidiEvent *pEvent
-				= pMidiClip->sequence()->events().first();
-			while (pEvent && iTimeClip + pEvent->time() < iTimeStart)
-				pEvent = pEvent->next();
-			while (pEvent && iTimeClip + pEvent->time() < iTimeEnd) {
-				qtractorMidiEvent *pNewEvent
-					= new qtractorMidiEvent(*pEvent);
-				pNewEvent->setTime(iTimeOffset + pEvent->time());
-				if (pNewEvent->type() == qtractorMidiEvent::NOTEON) {
-					unsigned long iTimeEvent = iTimeClip + pEvent->time();
-					float fGain = pMidiClip->gain(
-						pSession->frameFromTick(iTimeEvent)
-						- pClip->clipStart());
-					pNewEvent->setVelocity((unsigned char)
-						(fGain * float(pEvent->velocity())) & 0x7f);
-					if (iTimeEvent + pEvent->duration() > iTimeEnd)
-						pNewEvent->setDuration(iTimeEnd - iTimeEvent);
+			// Do the MIDI merge, itself...
+			qtractorMidiClip *pMidiClip
+				= static_cast<qtractorMidiClip *> (pClip);
+			if (pMidiClip) {
+				unsigned long iTimeClip
+					= pSession->tickFromFrame(pClip->clipStart());
+				unsigned long iTimeOffset = iTimeClip - iTimeStart;
+				// For each event...
+				qtractorMidiEvent *pEvent
+					= pMidiClip->sequence()->events().first();
+				while (pEvent && iTimeClip + pEvent->time() < iTimeStart)
+					pEvent = pEvent->next();
+				while (pEvent && iTimeClip + pEvent->time() < iTimeEnd) {
+					qtractorMidiEvent *pNewEvent
+						= new qtractorMidiEvent(*pEvent);
+					pNewEvent->setTime(iTimeOffset + pEvent->time());
+					if (pNewEvent->type() == qtractorMidiEvent::NOTEON) {
+						unsigned long iTimeEvent = iTimeClip + pEvent->time();
+						float fGain = pMidiClip->gain(
+							pSession->frameFromTick(iTimeEvent)
+							- pClip->clipStart());
+						pNewEvent->setVelocity((unsigned char)
+							(fGain * float(pEvent->velocity())) & 0x7f);
+						if (iTimeEvent + pEvent->duration() > iTimeEnd)
+							pNewEvent->setDuration(iTimeEnd - iTimeEvent);
+					}
+					seq.insertEvent(pNewEvent);
+					pEvent = pEvent->next();
 				}
-				seq.insertEvent(pNewEvent);
-				pEvent = pEvent->next();
 			}
 		}
 	}
@@ -1563,6 +1650,13 @@ void qtractorTracks::selectAll ( bool bSelect )
 }
 
 
+// Invert selection on all tracks and clips.
+void qtractorTracks::selectInvert (void)
+{
+	m_pTrackView->selectInvert();
+}
+
+
 // Adds a new track into session.
 bool qtractorTracks::addTrack (void)
 {
@@ -1682,7 +1776,7 @@ bool qtractorTracks::addAudioTracks (
 	if (pSession == NULL)
 		return false;
 
-	pSession->lock();
+//	pSession->lock();
 
 	// Account for actual updates...
 	int iUpdate = 0;
@@ -1729,11 +1823,11 @@ bool qtractorTracks::addAudioTracks (
 		pTrack->addClip(pAudioClip);
 		if (iTrackClip == 0)
 			pTrack->setTrackName(pAudioClip->clipName());
-		iTrackClip++;
+		++iTrackClip;
 		// Add the new track to composite command...
 		if (bDropSpan)
 			iClipStart += pAudioClip->clipLength();
-		iUpdate++;
+		++iUpdate;
 		// Don't forget to add this one to local repository.
 		if (pMainForm) {
 			pMainForm->addAudioFile(sPath);
@@ -1758,7 +1852,8 @@ bool qtractorTracks::addAudioTracks (
 		delete pImportTrackCommand;
 	}
 
-	pSession->unlock();
+//	pSession->unlock();
+
 	return bResult;
 }
 
@@ -1775,7 +1870,7 @@ bool qtractorTracks::addMidiTracks (
 	if (pSession == NULL)
 		return false;
 
-	pSession->lock();
+//	pSession->lock();
 
 	// Account for actual updates...
 	int iUpdate = 0;
@@ -1809,7 +1904,7 @@ bool qtractorTracks::addMidiTracks (
 			continue;
 		// It all depends on the format...
 		int iTracks = (file.format() == 1 ? file.tracks() : 16);
-		for (int iTrackChannel = 0; iTrackChannel < iTracks; iTrackChannel++) {
+		for (int iTrackChannel = 0; iTrackChannel < iTracks; ++iTrackChannel) {
 			// Create a new track right away...
 			const QColor& color = qtractorTrack::trackColor(++iTrack);
 			qtractorTrack *pTrack
@@ -1832,7 +1927,7 @@ bool qtractorTracks::addMidiTracks (
 			// only the first track/channel has some tempo/time signature...
 			if (iTrackChannel == 0) {
 				// Some adjustment required...
-				iImport++;
+				++iImport;
 				iClipStart = pSession->frameFromTick(iTimeStart);
 				pMidiClip->setClipStart(iClipStart);
 			}
@@ -1842,7 +1937,7 @@ bool qtractorTracks::addMidiTracks (
 				pTrack->setTrackName(pMidiClip->clipName());
 				pTrack->setMidiChannel(pMidiClip->channel());
 				pImportTrackCommand->addTrack(pTrack);
-				iUpdate++;
+				++iUpdate;
 				// Don't forget to add this one to local repository.
 				if (pMainForm)
 					pMainForm->addMidiFile(sPath);
@@ -1875,7 +1970,8 @@ bool qtractorTracks::addMidiTracks (
 		delete pImportTrackCommand;
 	}
 
-	pSession->unlock();
+//	pSession->unlock();
+
 	return bResult;
 }
 
@@ -1996,7 +2092,7 @@ void qtractorTracks::selectionChangeNotify (void)
 }
 
 
-// Simple main-form dirty-flag redirector.
+// Simple main-form dirty-flag redirectors.
 void qtractorTracks::contentsChangeNotify (void)
 {
 	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
@@ -2004,6 +2100,27 @@ void qtractorTracks::contentsChangeNotify (void)
 		pMainForm->changeNotifySlot(NULL);
 }
 
+void qtractorTracks::dirtyChangeNotify (void)
+{
+	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+	if (pMainForm)
+		pMainForm->dirtyNotifySlot();
+}
+
+
+// Track-list update (current track only).
+void qtractorTracks::updateTrackList (void)
+{
+	m_pTrackList->updateTrack(currentTrack());
+}
+
+
+// Track-view update (obviously a slot).
+void qtractorTracks::updateTrackView (void)
+{
+	m_pTrackView->update();
+}
+
 
 // Overall contents reset.
 void qtractorTracks::clear(void)
