@@ -110,10 +110,12 @@ public:
 	void selectCurrentTrack(bool bReset = false);
 	void selectCurrentTrackRange(bool bReset = false);
 	void selectAll(bool bSelect = true);
+	void selectInvert();
 
 	// Simple main-form redirectors.
 	void selectionChangeNotify();
 	void contentsChangeNotify();
+	void dirtyChangeNotify();
 
 	// Overall contents reset.
 	void clear();
@@ -129,6 +131,9 @@ public:
 	void zoomOut();
 	void zoomReset();
 
+	// Track-list update (current track only).
+	void updateTrackList();
+
 protected:
 
 	// Zoom factor constants.
@@ -138,9 +143,16 @@ protected:
 	void horizontalZoomStep(int iZoomStep);
 	void verticalZoomStep(int iZoomStep);
 
-	// Try to center horizontally/vertically
-	// (usually after zoom change)
-	void centerContents();
+	// Zoom centering context.
+	struct ZoomCenter
+	{
+		int x, y;
+		unsigned long frame;
+	};
+
+	// Zoom centering prepare and post methods.
+	void zoomCenterPre(ZoomCenter& zc) const;
+	void zoomCenterPost(const ZoomCenter& zc);
 
 	// Multi-clip command builders.
 	bool normalizeClipCommand(
@@ -156,6 +168,11 @@ protected:
 	bool mergeExportAudioClips(qtractorClipCommand *pClipCommand);
 	bool mergeExportMidiClips(qtractorClipCommand *pClipCommand);
 
+public slots:
+
+	// Track-view update (obviously a slot).
+	void updateTrackView();
+
 protected slots:
 
 	// Zoom view slots.
